function out = get( m, prop )
%get function for class miller
%
if nargin == 1
    s = struct( ...
        'algorithm', get(m, 'algorithm'), ...
        'ref_alg', get(m, 'ref_alg'), ...
        'diff_method', get(m, 'diff_method'), ...
        'error_measure', get(m, 'error_measure'), ...
        'stop_crit', get(m, 'stop_crit'), ...
        'is_compare', get(m, 'is_compare'), ...
        'num_of_algs', get(m, 'num_of_algs'), ...
        'num_of_inputs', get(m, 'num_of_inputs'), ...
        'num_of_operations', get(m, 'num_of_operations'), ...
        'num_of_constants', get(m, 'num_of_constants'), ...
        'num_of_outputs', get(m, 'num_of_outputs'), ...
        'num_of_constraints', get(m, 'num_of_constraints'), ...
        'num_of_calls', get(m, 'num_of_calls'), ...
        'output', get(m, 'output'), ...
        'constraints', get(m, 'constraints'), ...
        'rho', get(m, 'rho'), ...
        'penalty', get(m, 'penalty'), ...
        'graph', get(m, 'graph'), ...
        'memory', get(m, 'memory'), ...
        'exceptions', get(m, 'exceptions'), ...
        'mode', get(m, 'mode'), ...
        'is_stop', get(m, 'is_stop'), ...
        'is_terminated', get(m, 'is_terminated'), ...
        'size_of_w', get(m, 'size_of_w') );
    if nargout == 0
        disp( s );
    else
        out = s;
    end
    return;
end
if ~ischar( prop )
    error( 'Property identificator must be a string' );
end
prop = lower( strtrim( prop ) );
switch prop
    case 'algorithm'
        out = roundoff( m.ptr, 1002, 1 );
    case 'ref_alg'
        out = roundoff( m.ptr, 1002, 2 );
    case 'diff_method'
        value = roundoff( m.ptr, 1002, 3 );
        if value == 1.0
            out = 'Ukkonen';
        elseif value == 2.0
            out = 'Ukkonen with graph';
        else
            out = 'Miller';
        end
    case 'error_measure'
        if isempty( roundoff( m.ptr, 1002, 2 ) )
            switch roundoff( m.ptr, 1002, 4 )
                case 4.0
                    out = 'wkl';
                case 2.0
                    out = 'wke';
                case 3.0
                    out = 'jwl';
                case 1.0
                    out = 'jwe';
                case -1.0
                    out = 'ere';
                case -3.0
                    out = 'erl';
                case 6.0
                    out = 'cne';
                case 7.0
                    out = 'cnl';
                otherwise
                    out = 'not set';
            end
        else
            switch roundoff( m.ptr, 1002, 4 )
                case 1.0
                    out = 'jw';
                case -1.0
                    out = 'er';
                case 2.0
                    out = 'jw2/1';
                case -2.0
                    out = 'er2/1';
                otherwise
                    out = 'not set';
            end
        end
    case 'stop_crit'
        out = roundoff( m.ptr, 1002, 5 );
    case 'is_compare'
        out = ~isempty( roundoff( m.ptr, 1002, 2 ) );
    case 'num_of_algs'
        if isempty( roundoff( m.ptr, 1002, 1 ) );
            out = 0;
        elseif isempty( roundoff( m.ptr, 1002, 2 ) );
            out = 1;
        else
            out = 2;
        end
    case 'num_of_inputs'
        out = roundoff( m.ptr, 1002, 6 );
    case 'num_of_operations'
        out = roundoff( m.ptr, 1002, 7 );
    case 'num_of_constants'
        out = roundoff( m.ptr, 1002, 8 );
    case 'num_of_outputs'
        out = roundoff( m.ptr, 1002, 9 );
    case 'num_of_constraints'
        out = roundoff( m.ptr, 1002, 10 );
    case 'num_of_calls'
        out = roundoff( m.ptr, 1002, 11 );
    case 'output'
        out = roundoff( m.ptr, 1002, 12 );
    case 'constraints'
        out = roundoff( m.ptr, 1002, 13 );
    case 'rho'
        out = roundoff( m.ptr, 1002, 14 );
    case 'penalty'
        out = roundoff( m.ptr, 1002, 15 );
    case 'graph'
        out = roundoff( m.ptr, 1002, 16 );
    case 'memory'
        out = roundoff( m.ptr, 1002, 17 );
    case 'exceptions'
        out = roundoff( m.ptr, 1002, 18 );
    case 'mode'
        out = roundoff( m.ptr, 1002, 19 );
        if out == 0
            out = 'normal';
        elseif out == 1
            out = 'search';
        end
    case 'is_stop'
        out = roundoff( m.ptr, 1002, 20 );
    case 'is_terminated'
        out = roundoff( m.ptr, 1002, 21 );
    case 'size_of_w'
        out = roundoff( m.ptr, 1002, 22 );
%     case 'all'
%         
%         get( m, 'algorithm' )
%         get( m, 'ref_alg' )
%         get( m, 'diff_method' )
%         get( m, 'error_measure' )
%         get( m, 'stop_crit' )
%         get( m, 'num_of_algs' )
%         get( m, 'num_of_inputs' )
%         get( m, 'num_of_operations' )
%         get( m, 'num_of_constants' )
%         get( m, 'num_of_outputs' )
%         get( m, 'num_of_constraints' )
%         get( m, 'num_of_calls' )
%         get( m, 'output' )
%         get( m, 'constraints' )
%         get( m, 'rho' )
%         get( m, 'penalty' )
%         get( m, 'graph' )
%         get( m, 'memory' )
%         get( m, 'exceptions' )
    otherwise
        error( '%s is an invalid property name for object miller',...
                prop );
    
end
