function [rho,dfinal] = maxsearch( m, dinit, methodcode, varargin )
    error( nargchk(3, 4, nargin ) );
    rho = 0;
    dfinal = dinit;
    
    ecv = get( m, 'error_measure' );
    if ( strcmp( ecv, 'not set' ) )
        error( 'Error measuring value is not set!' );
    end
    disp( sprintf( 'The chosen error measuring number is %s.\n', ecv ) );
    try
        rho = roundoff( m.ptr, 1003, dinit, varargin{:} );
    catch
        s = lasterror;
        error( sprintf( ['An error occured while computing %s at the initial data.\n',...
            'The type of the error is: %s\nNo maximization performed!'], ecv, s.message ) );
    end
    
    disp( sprintf( 'The error measuring number at the initial data: %g', rho ) );
    C = get( m, 'constraints' );
    n = length( C );
    if n ~= 0
        disp( 'The constraints are:\n' );
        disp( sprintf( ['C(%' num2str( ceil( log10( n ) ) ) 'd) = %g\n'], [1:n ;C'] ) );
    else
        disp( 'There are no constraints' );
    end
    if get( m, 'penalty' ) <= 0
        error( 'The initial data are out of constraints!' );
    end
    tmp = get( m, 'stop_crit' );
    disp( sprintf( 'The stopping value is: %g\n', tmp ) );
    if rho > tmp && tmp > 0
        disp( sprintf( ['The error measuring value at the initial data' ...
            ' exceeds the stopping criterion.\n No maximization performed. \n'] ) );
        return;
    end
    
    methodcode = lower( strtrim( methodcode ) );
    switch methodcode
        case 'ros'
            mthdstr = 'ROSENBROCK';
            method = 1;
        case 'mds'
            mthdstr = 'MULTIDIRECTIONAL SERCH';
            method = 2;
        case 'nms'
            mthdstr = 'NELDER-MEAD SIMPLEX';
            method = 3;
        otherwise
            error( sprintf(['The third input argument must be:\n',...
                '"ros" for ROSENBROCK\n',...
                '"mds" for MULTIDIRECTIONAL SEARCH\n',...
                '"nms" for NELDER-MEAD SIMPLEX method\n',...
                '\nNo maximization performed!'] ) );
    end
    disp( sprintf( 'The choosen search method is %s method.', mthdstr ) );
    disp( 'Starting the maximizer...' );
    roundoff( m.ptr, 1005 );
    [rho, dfinal] = roundoff( m.ptr, 1004, dinit, method );
    if get( m, 'num_of_algs' ) == 1 && rho > tmp && get( m, 'penalty' ) && tmp > 0
        if ecv(3) == 'l'
            set(m, 'error_measure', 'cnl' );
        else
            set(m, 'error_measure', 'cne' );
        end
        try
            tmp = roundoff( m.ptr, 1003, dfinal, varargin{:} );
            if tmp > 0
                disp( sprintf( 'The condition number at the final set of data is: %g\n', tmp ) );
            else
                warning( 'An error occured while computing the condition number!') ;
            end
        catch
            warning( 'An error occured while computing the condition number!') ;
        end
        set(m, 'error_measure', ecv );
    end
    tmp = get( m, 'exceptions' );
    if tmp(1) > 0
        disp( sprintf( '%d attempted divisions by zero.\n', tmp(1) ) );
    end
    if tmp(2) > 0
        disp( sprintf( '%d attempted SQRT(V), V LT 0.\n', tmp(2) ) );
    end
    if tmp(3) > 0
		disp( sprintf( '%d GETER failures.\n', tmp(3) ) );
    end
    if tmp(4) > 0
        disp( sprintf( '%d OMEGA failures.\n', tmp(4) ) );
    end
    if tmp(5) > 0
        disp( sprintf( '%d DIAGON failures.\n', tmp(5) ) );
    end
    if tmp(6) > 0
        disp( sprintf( '%d CONDIT failures.\n', tmp(6) ) );
    end
    if tmp(7) > 0
        disp( sprintf( '%d CHOLESKY failures.\n', tmp(7) ) );
    end
    if tmp(8) > 0
        disp( sprintf( '%d GENEIG failures.\n', tmp(8) ) );
    end
    if tmp(9) > 0
        disp( sprintf( '%d TRIDIAGON failures.\n', tmp(9) ) );
    end
    if tmp(10) > 0
        disp( sprintf( '%d EIGENV failures.\n', tmp(10) ) );
    end
    if tmp(11) > 0
        disp( sprintf( '%d USER failures.\n', tmp(11) ) );
    end
        try
            rho2 = roundoff( m.ptr, 1003, dfinal, varargin{:} );
            rho = rho2;
        catch
            warning( ['At the final set of data '...
                'the error measuring number cannot be computed']);
        end

end
        