function m = miller( par, ukkonen )
%   Constructor for class miller
%
%   M = miller( m_obj )         
%       Creates a deep copy of m_obj object of miller class.
%
%   M = miller( 'num_alg' )     
%       Creates a new object for analysing num_alg.
%       num_alg is an m-file or mex-file (without file extension)
%       implementing the method to analyse.
%
%   M = miller( 'num_alg', diff_method )     
%       Creates a new object for analysing num_alg.
%       diff_method can be: 'miller', 'ukkonen', or 'ukkonengraph'. 
%           If omitted the method 'miller' will be applied. 
%
%   M = miller( 'num_alg1/num_alg2' )
%       Creates an object for comparing num_alg1 with num_alg2
%
%   M = miller( 'num_alg1/num_alg2', diff_method )
%       Creates an object for comparing num_alg1 with num_alg2.
%       diff_method is the same as above



if nargin == 0
    error( 'Class miller does not have default constructor.' );
elseif nargin == 1
    if isa( par, 'miller' )
        m.ptr = roundoff( par.ptr, 1007 );
        m = class( m, 'miller' );
        return
    else
        ukkonen  = 0.0;
    end
elseif nargin == 2
    if ~ischar( ukkonen )
        error( 'Second argument must be a string.' );
    end
    ukkonen = lower( strtrim( ukkonen ) );
    if strcmp( ukkonen, 'ukkonen' )
        ukkonen = 1.0;
    elseif strcmp( ukkonen, 'ukkonengraph' )
        ukkonen = 2.0;
    elseif strcmp( ukkonen, 'miller' )
        ukkonen = 0.0;
    else
        error( 'Invalid differentiation method: ''%s''!', ukkonen );
    end
end
if ~ischar( par )
    error( ['Invalid constructor call for class miller:', ...
        ' First argument shuld be a string'] );
end
tmp = size(findstr( '/', par ),2);
if tmp > 1
    error( ['Invalid constructor call for class miller:', ...
        ' More than one "/" character in the first argument'] );
end
if tmp == 0
    if ~isempty( par ) 
        tmp = exist( par, 'file' );
        if tmp ~= 2 && tmp ~= 3 && tmp ~= 6
            error( 'Algorithm: ''%s'' not found!', par );
        end
    end
else
    [par1, par2] = strtok( par, '/' );
    tmp = exist( par1, 'file' );
    if tmp ~= 2 && tmp ~= 3 && tmp ~= 6
        error( 'Algorithm: ''%s'' not found!', par1 );
    end
    [par2, par1] = strtok( par2, '/' );
    tmp = exist( par2, 'file' );
    if tmp ~= 2 && tmp ~= 3 && tmp ~= 6
        error( 'Algorithm: ''%s'' not found!', par2 );
    end
end

m.ptr = roundoff();

try
    roundoff( m.ptr, 1000, par, ukkonen );
catch
    roundoff( m.ptr, 1006 );
    m = 0;
    rethrow( lasterror );
end
m = class( m, 'miller' );
end


