function out = set( m, varargin )
%set Sets the properties of a miller-type object
%   
if nargin == 1
    if ~get( m, 'is_compare' )
        err_m={ 'wkl', 'wke', 'jwl', 'jwe', 'erl', 'ere', 'cnl', 'cne', 'clear' };
    else
        err_m={ 'jw1/2', 'er1/2', 'jw2/1', 'er2/1', 'clear' };
    end
        
    s = struct( ...
        mode, {'normal', 'search' }, ...
        error_measure, err_m, ...
        stop_crit, {} );
    if nargout == 0
        disp( s );
    else
        out = s;
    end
    return;
end
if length( varargin ) == 1
    s = varargin{1};
    if isstruct( s )
        names = fieldnames(s);
        n = length( names );
        values = struct2cell( s );
        ArgIn = cell( 1, 2*n );
        for i = 1 : n
            ArgIn( i ) = names( i );
            ArgIn( i + 1 ) = values( i );
        end
    else
        ArgIn = varargin;
    end
else
    ArgIn = varargin;
end
while length( ArgIn ) >= 2
    prop = ArgIn{1};
    val = ArgIn{2};
    ArgIn = ArgIn( 3:end );
    if ~ischar( prop )
        error( 'Property identificator must be a string' );
    end
    prop = lower( strtrim( prop ) );
    switch prop
        case 'mode'
            if ~ischar( val )
                error( 'Value of mode property must be a string' );
            end
            val = lower( strtrim( val ) );
            if strcmp( val, 'normal' )
                roundoff( m.ptr, 1001, 3, 0.0 );
            elseif strcmp( val , 'search' );
                roundoff( m.ptr, 1001, 3, 1.0 );
            end
                
            
        case 'error_measure'
            if ~ischar( val )
                error( 'Value of error_measure property must be a string' );
            end
            val = lower( strtrim( val ) );
            if isempty( val )
                val = 0.0;
            else
                if ~get( m, 'is_compare' )
                    switch val
                        case 'wkl'
                            val = 4.0;
                        case 'wke'
                            val = 2.0;
                        case 'jwl'
                            val = 3.0;
                        case 'jwe'
                            val = 1.0;
                        case 'ere'
                            val = -1.0;
                        case 'erl'
                            val = -3.0;
                        case 'cne'
                            val = 6.0;
                        case 'cnl'
                            val = 7.0;
                        case 'clear'
                            val = 0.0;
                        otherwise
                            error( ['%s is an invalid error measuring '...
                                'number specifier for analysing a '...
                                'single algorithm'], val );
                    end
                else
                    switch val
                        case {'jw', 'jw1/2'}
                            val = 1.0;
                        case {'er', 'er1/2'}
                            val = -1.0;
                        case 'jw2/1'
                            val = 2.0;
                        case 'er2/1'
                            val = -2.0;
                        case 'clear'
                            val = 0.0;
                        otherwise
                            error( ['%s is an invalid error measuring '...
                                'number specifier for comparing two '...
                                'algorithms'], val );
                    end
                end
            end
            roundoff( m.ptr, 1001, 4, val );
        case 'stop_crit'
            if ~isnumeric( val ) || length( val ) ~= 1 || val < 0
                error( ['Value for property stop_crit must be '...
                    'a nonnegative scalar' ]);
            else
                roundoff( m.ptr, 1001, 5, val );
            end
        otherwise
            error( '%s is not a user-settable property for object miller',...
                prop );
    end
end
out = m;
end

