      SUBROUTINE ROUND1_FOR( VALUE, DERIVS, CONS, RHO, LIMITS, NLOP, 
     *              NOPER, NROP, NDXOUT, NVARY, NFIND, NPICK, 
     *              NTOP, NERR, WGTLFT, WGTRGT, NDCOL, TOCOPY, A, B )
C THIS SUBROUTINE FINDS THE INTERMEDIATE COMPUTED VALUES PLUS THEIR
C PARTIAL DERIVATIVES WITH RESPECT TO THEIR ARGUMENTS.  A METHOD DUE
C TO LARSON AND SAMEH (EFFICIENT CALCULATION OF THE EFFECTS OF
C ROUNDING ERRORS, TOMS 4 (1978), PP. 228-236), IS
C APPLIED TO COMPUTE DERIVS(I,J), THE PARTIAL DERIVATIVE OF THE J-TH
C OUTPUT WRT THE I-TH PROGRAM VALUE.  IN STATEMENT 100 DERIVS IS
C MODIFIED TO BE THE DERIVATIVE WRT THE I-TH ROUNDING ERROR.
C
      INTEGER LIMITS(*)
      DOUBLE PRECISION VALUE(*), DERIVS(LIMITS(2),*), CONS(*) 
      DOUBLE PRECISION RHO
      INTEGER NLOP(*), NOPER(*), NROP(*), NDXOUT(*) 
      INTEGER NVARY, NVARYP
      INTEGER NFIND, NPICK, NTOP, NERR
      INTEGER NDCOL(*)
      
      INTEGER TOCOPY;
      DOUBLE PRECISION A(NFIND, *), B(NFIND, *)
      
      INTEGER NODES      
      INTEGER I, J
      INTEGER ILOP, MILOP, IIOPER, IOPER
      INTEGER IROP, MIROP
      INTEGER JOUT, JOUTP, LIM, IBACK, MINUS1
      
      DOUBLE PRECISION DIJ, ONE, ARGL, ARGR, HALF, ZERO,
     * WGTLFT(*), WGTRGT(*), XPRMNT
      DOUBLE PRECISION DSQRT
      SAVE
      DATA ONE /1.0D0/, HALF /0.5D0/, ZERO /0.0D0/
      NODES = LIMITS(1)
      
      NERR = 0
      NVARYP=NVARY+1
      DO 80 I=NVARYP,NODES
C COMPUTE THE I-TH VALUE AND ITS INPUT ARC WEIGHTS.
        ILOP = NLOP(I)
        MILOP = -ILOP
        IF (ILOP.LT.0) ARGL = CONS(MILOP)
        IF (ILOP.GT.0) ARGL = VALUE(ILOP)
        IIOPER = NOPER(I)
        IOPER = IABS(IIOPER)
        IF (IOPER.GT.4) GO TO 10
        IROP = NROP(I)
        MIROP = -IROP
        IF (IROP.LT.0) ARGR = CONS(MIROP)
        IF (IROP.GT.0) ARGR = VALUE(IROP)
   10   GO TO (20, 30, 40, 50, 60, 70), IOPER
C ADDITION
   20   VALUE(I) = ARGL + ARGR
        WGTLFT(I) = ONE
        WGTRGT(I) = ONE
        GO TO 80
C SUBTRACTION
   30   VALUE(I) = ARGL - ARGR
        WGTLFT(I) = ONE
        WGTRGT(I) = -ONE
        GO TO 80
C MULTIPLICATION
   40   VALUE(I) = ARGL*ARGR
        WGTLFT(I) = ARGR
        WGTRGT(I) = ARGL
        GO TO 80
C DIVISION
   50   IF (ARGR.EQ.ZERO) GO TO 190
        VALUE(I) = ARGL/ARGR
        WGTLFT(I) = ONE/ARGR
        WGTRGT(I) = -VALUE(I)/ARGR
        GO TO 80
C SQUARE ROOT
   60   IF (ARGL.LE.ZERO) GO TO 200
        VALUE(I) = DSQRT(ARGL)
        WGTLFT(I) = HALF/VALUE(I)
        GO TO 80
C UNARY MINUS
   70   VALUE(I) = -ARGL
        WGTLFT(I) = -ONE
   80 CONTINUE
C
C      IF (NPICK.EQ.0) GO TO 180
C COMPUTE THE PARTIAL DERIVATIVES.
      DO 120 J=1,NFIND
        JOUT = NDXOUT(J)
        DO 90 I=1,JOUT
          DERIVS(I,J) = ZERO
   90   CONTINUE
        DERIVS(JOUT,J) = ONE
        JOUTP = JOUT + 1
        LIM = JOUT - NVARY
        DO 110 IBACK=1,LIM
          I = JOUTP - IBACK
C I = NDXOUT(J), NDXOUT(J)-1, ..., NVARY + 1
          DIJ = DERIVS(I,J)
          IF (DIJ.EQ.ZERO) GO TO 110
          ILOP = NLOP(I)
          IF (ILOP.GT.0) DERIVS(ILOP,J) = DERIVS(ILOP,J) + WGTLFT(I)*DIJ
          IIOPER = NOPER(I)
          IOPER = IABS(IIOPER)
          IF (IOPER.GT.4) GO TO 100
          IROP = NROP(I)
          IF (IROP.GT.0) DERIVS(IROP,J) = DERIVS(IROP,J) + WGTRGT(I)*DIJ
  100     DERIVS(I,J) = DIJ*VALUE(I)
C UNARY MINUS OPERATIONS ARE CONSIDERED ERROR-FREE.
          IF (IOPER.EQ.6) DERIVS(I,J) = ZERO
  110   CONTINUE
  120 CONTINUE
      IF (NTOP.EQ.0) GO TO 170
C IF SOME OPERATIONS ARE TO BE CONSIDERED ERROR-FREE, THEN
C RECOMPUTE DERIVATIVES WRT THEM, IGNORING PATHS THROUGH NODES
C WITH ERRORS.
C IF NTOP = -1, THEN ALL NODES ARE ERROR-FREE.
      MINUS1 = -1
      DO 160 J=1,NFIND
        JOUT = NDXOUT(J)
        DO 130 I=NVARYP,JOUT
          DERIVS(I,J) = ZERO
  130   CONTINUE
        IF (NTOP.EQ.MINUS1) GO TO 160
        DERIVS(JOUT,J) = ONE
        JOUTP = JOUT + 1
C        LIM = JOUT - NTOP
        LIM = JOUTP - NTOP
        DO 150 IBACK=1,LIM
          I = JOUTP - IBACK
C I = NDXOUT(J), NDXOUT(J)-1, ..., NTOP + 1
          IIOPER = NOPER(I)
C IF THE CURRENT NODE HAS AN ERROR, DON'T UPDATE
C THE SPP'S OF ITS OPERANDS.
          DIJ = DERIVS(I,J)
          IF (IIOPER.GT.0) DERIVS(I,J) = DIJ*VALUE(I)
          IF (IIOPER.GT.0 .OR. DIJ.EQ.ZERO) GO TO 150
          ILOP = NLOP(I)
          IF (ILOP.GE.NTOP) DERIVS(ILOP,J) = DERIVS(ILOP,J) +
     *     WGTLFT(I)*DIJ
          IOPER = IABS(IIOPER)
          IF (IOPER.GT.4) GO TO 140
          IROP = NROP(I)
          IF (IROP.GE.NTOP) DERIVS(IROP,J) = DERIVS(IROP,J) +
     *     WGTRGT(I)*DIJ
C THIS NODE IS TO BE CONSIDERED ERROR-FREE.
  140     DERIVS(I,J) = ZERO
  150   CONTINUE
  160 CONTINUE
  170 IF (TOCOPY.EQ.0) GO TO 179
         DO 174 J=1,NVARY
            DO 172 I=1,NFIND
               B(I,J) = DERIVS(J,I)
  172       CONTINUE
  174    CONTINUE
         IF (NPICK.GE.6) GO TO 179
C            DO 178 J=NVARYP,NODES
C               LIM = J - NVARY
C               DO 176 I=1,NFIND
C                  A( I, LIM ) = DERIVS(J,I)
C  176          CONTINUE
C  178       CONTINUE
            DO 178 I=1,NFIND
               JOUT = NDXOUT(I)
               JOUTP = JOUT + 1
               DO 176 J=NVARYP,JOUT
                  A( I, J - NVARY ) = DERIVS(J,I)
  176         CONTINUE
               DO 177 J=JOUTP,NODES
                  A( I, J - NVARY ) = ZERO
  177          CONTINUE
  178       CONTINUE
      
  179 IF (NPICK.EQ.5) RHO = XPRMNT(DERIVS,VALUE,VALUE(NVARYP),
     *   NDXOUT,NVARY,LIMITS,NFIND)
      IF (NPICK.GT.0 .AND. NPICK.LE.4) 
     *     CALL OMEGA1( VALUE, DERIVS, RHO, NDXOUT, NVARY, LIMITS,
     *                  NFIND, NPICK, NERR, NDCOL )
      IF (NPICK.GE.6) CALL CONDIT( VALUE, DERIVS, RHO, NDXOUT, NVARY,
     *                  LIMITS(2), NFIND, NPICK, NERR, NDCOL )
      IF (NPICK.LT.0) CALL GETER1( VALUE, DERIVS, RHO, NDXOUT,
     *                NVARY, LIMITS(2), NFIND, NPICK, NERR )
  180 RETURN
C
C ATTEMPTED DIVISION BY ZERO.
  190 NERR = 1
      RHO = ZERO
      GO TO 180
C ATTEMPTED SQRT(V), V.LE.ZERO.
  200 NERR = 2
      RHO = ZERO
      GO TO 180
      END
      
      
      SUBROUTINE OMEGA1( VALUE, DERIVS, RHO, NDXOUT, NVARY, LIMITS,
     *              NFIND, NPICK, NERR, NDCOL )
C WE WILL USE THE NOTATION
C     A = DERIVATIVE OF THE OUTPUT WITH RESPECT TO ROUNDING ERRORS
C     B = DERIVATIVE OF THE OUTPUT WITH RESPECT TO PROBLEM PERTURBATIONS
C THIS SUBROUTINE COMPUTES OMEGA(AK,BK), WHERE K DENOTES THE
C APPROPRIATE 2-NORM UNIT BALL.
      INTEGER LIMITS(*)
      INTEGER NODES
      DOUBLE PRECISION VALUE(*), DERIVS(LIMITS(2),*), RHO
      INTEGER NDXOUT(*), NVARY, NVARYP
      INTEGER NFIND,NPICK,NERR
      
      
      DOUBLE PRECISION T,TRY,ZERO,DENOM
     *                 ,VALNRM,HUGESV,TINYSV,TEN,DNORM
      DOUBLE PRECISION DABS
      INTEGER NDCOL(*)
      LOGICAL MOVEA, NPICK4
      
      INTEGER I, J, NROWS, NDCOLJ, LIM, LIM1, LIM2
      INTEGER NTOPA, IPNF, JOUT, ND, IERR, IPTOPA
      
      SAVE
      DATA ZERO /0.0D0/, TEN /10.0D0/
      NODES = LIMITS(1)
      
      NVARYP = NVARY + 1
C FIRST REDUCE A TO A SQUARE MATRIX BY APPLYING ORTHOGONAL
C TRANSFORMATIONS IN THE SPACE OF ROUNDING ERRORS.
C
      
      
      DO 1 J = 1,NFIND
         NDCOL(J) = NDXOUT(J) - NVARY
    1    CONTINUE
      NROWS = NODES - NVARY
      IF (NROWS .LT. NFIND) GO TO 1001
      IF (NROWS .GT. NFIND) CALL
     *     SQUARE (DERIVS(NVARYP,1),NDCOL,NFIND,.TRUE.,LIMITS(2))
      IF (NROWS. GT. NFIND) GO TO 4
      DO 3 J = 1,NFIND
         NDCOLJ = NDCOL(J)
         IF (NDCOLJ .GE. NFIND) GO TO 3
         LIM1 = NDXOUT(J) + 1
         LIM2 = NVARY + NFIND
         DO 2 I = LIM1,LIM2
            DERIVS(I,J) = ZERO
    2       CONTINUE
    3    CONTINUE
C
C SECOND, REDUCE B TO A SQUARE MATRIX BY APPLYING ORTHOGONAL
C TRANSFORMATIONS IN THE SPACE OF PROBLEM PERTURBATIONS.
C
    4 IF (NPICK .LE. 2) GO TO 8
      DNORM = ZERO
      DO 5 I = 1,NVARY
         TRY = DABS(VALUE(I))
         IF (TRY .GT. DNORM) DNORM = TRY
    5    CONTINUE
      DO 7 J = 1,NFIND
         DO 6 I = 1,NVARY
            DERIVS(I,J) = DNORM*DERIVS(I,J)
    6       CONTINUE
    7    CONTINUE
      GO TO 11
    8 DO 10 J = 1,NFIND
         DO 9 I = 1,NVARY
            DERIVS(I,J) = VALUE(I)*DERIVS(I,J)
    9       CONTINUE
   10    CONTINUE
   11 MOVEA = NPICK.EQ.2 .OR. NPICK.EQ.4
      IF (MOVEA) GO TO 14
      NTOPA = NVARYP
      DO 12 J = 1,NFIND
         NDCOL(J) = NVARY
   12    CONTINUE
      GO TO 100
   14 NTOPA = NVARYP + NFIND
C SHIFT THE REDUCED 'A' MATRIX TO MAKE ROOM FOR THE DERIVATIVE WITH
C RESPECT TO OUTPUT PERTURBATIONS.
      LIM = NVARY + NFIND
      DO 18 J = 1,NFIND
         DO 16 I = NVARYP,LIM
            IPNF = I + NFIND
            DERIVS(IPNF,J) = DERIVS(I,J)
   16       CONTINUE
   18    CONTINUE
      IF (NPICK.EQ.2) GO TO 40
C COMPUTE THE NORM OF THE OUTPUT IF NEEDED.
      VALNRM = ZERO
      DO 30 J = 1,NFIND
         JOUT = NDXOUT(J)
         T = DABS(VALUE(JOUT))
         IF (T .GT. VALNRM) VALNRM = T
   30    CONTINUE
C ADD DERIVATIVES WITH RESPECT TO OUTPUT PERTURBATIONS.
   40 DO 70 J = 1,NFIND
         NDCOLJ = NVARY + J
         NDCOL(J) = NDCOLJ
         IF (J .EQ. 1) GO TO 60
         ND = NDCOLJ - 1
         DO 50 I = NVARYP,ND
            DERIVS(I,J) = ZERO
   50       CONTINUE
   60    NPICK4 = NPICK .EQ. 4
         IF (NPICK4) DERIVS(NDCOLJ,J) = VALNRM
         IF (NPICK4) GO TO 70
         JOUT = NDXOUT(J)
         DERIVS(NDCOLJ,J) = DABS(VALUE(JOUT))
   70    CONTINUE
  100 NROWS = NTOPA - 1
      IF (NROWS .LT. NFIND) GO TO 1001
      IF (NROWS .GT. NFIND) CALL
     *     SQUARE (DERIVS(1,1),NDCOL,NFIND,.FALSE.,LIMITS(2))
C
C THIRD, DIAGONALIZE B. (ANY ORTHOGONAL TRANSFORMATIONS IN THE
C OUTPUT SPACE MUST ALSO BE APPLIED TO A.)  THEN FORM (B INVERSE)*A,
C UNLESS B IS SO ILL-CONDITIONED AS TO RENDER THE COMPUTATIONS
C MEANINGLESS.
C
      CALL DIAGON (DERIVS(1,1),DERIVS(NTOPA,1),
     *              NFIND,.TRUE.,IERR,LIMITS(2))
      IF (IERR .NE. 0) GO TO 1002
      IF (NFIND .EQ. 1) GO TO 108
      HUGESV = DERIVS(1,1)
      TINYSV = HUGESV
      DO 104 J = 2,NFIND
         TRY = DERIVS(J,J)
         IF (TRY .LT. TINYSV) TINYSV = TRY
         IF (TRY .GT. HUGESV) HUGESV = TRY
  104    CONTINUE
      T = TINYSV/TEN
      TRY = HUGESV + T
      IF (TRY .EQ. HUGESV) GO TO 1002
  108 LIM = NTOPA + NFIND - 1
      DO 120 J = 1,NFIND
         DENOM = DERIVS(J,J)
         DO 110 I = NTOPA,LIM
            DERIVS(I,J) = DERIVS(I,J)/DENOM
  110       CONTINUE
  120    CONTINUE
C
C FOURTH (AND LAST), FIND THE SINGULAR VALUES OF (B INVERSE)*A.
C THE LARGEST IS OMEGA(AK,BK).
C
      CALL DIAGON (DERIVS(NTOPA,1),DERIVS(1,1),
     *                NFIND,.FALSE.,IERR,LIMITS(2))
      IF (IERR .NE. 0) GO TO 1002
      RHO = DERIVS(NTOPA,1)
      IF (NFIND .EQ. 1) GO TO 800
      DO 130 I = 2,NFIND
         IPTOPA = I + NTOPA - 1
         TRY = DERIVS(IPTOPA,I)
         IF (TRY .GT. RHO) RHO = TRY
  130    CONTINUE
  800 RETURN
C
C THIS CODE IS REACHED IF EITHER THE NUMBER OF ROUNDING ERRORS
C OR THE NUMBER OF PROBLEM PERTURBATIONS IS LESS THAN THE NUMBER
C OF OUTPUTS.
 1001 NERR = 4
      RHO = ZERO
      RETURN
C
C THIS CODE IS REACHED IF OMEGA CAN NOT BE COMPUTED ACCURATELY.
 1002 NERR = 5
      RHO = ZERO
      RETURN
      END




      SUBROUTINE GETER1( VALUE, DERIVS, RHO, NDXOUT, NVARY, LIMITS,
     *              NFIND, NPICK, NERR )
C
C THIS SUBROUTINE COMPUTES ERROR RATIOS ERE,ERL AS RHO = TOP/BOTTOM,
C WHERE TOP = (INFINITY NORM OF DERIVATIVE OF RESULT WRT ROUNDING
C ERRORS) AND BOTTOM = (INFINITY NORM OF DERIVATIVE OF RESULT WRT THE
C APPROPRIATE PERTURBATIONS). TWO OTHER RATIOS ARE ALSO COMPUTED TO
C MAINTAIN COMPATIBILITY WITH EARLIER VERSIONS OF THIS SOFTWARE.
C
      INTEGER LIMITS
      
      DOUBLE PRECISION VALUE(*), DERIVS(LIMITS,*), RHO
      INTEGER NDXOUT(*), NVARY, NVARYP, NFIND, NPICK, NERR
      
      DOUBLE PRECISION T, SUM, TOP, ZERO, DNORM, VNORM
      DOUBLE PRECISION BOTTOM
      DOUBLE PRECISION DABS
      
      INTEGER MPICK, I, K, LIM, IOUT
      
      SAVE
C FOR SINGLE PRECISION OMIT 'D0' FROM THE FOLLOWING CONSTANTS.
C ALSO CHANGE 'DABS'.
      DATA ZERO /0.0D0/
     
      NVARYP = NVARY + 1
C
      MPICK = -NPICK
C
C GET NORM OF DATA, IF NEEDED.
      IF (MPICK.LE.2) GO TO 20
      DNORM = ZERO
      DO 10 I=1,NVARY
        T = DABS(VALUE(I))
        IF (T.GT.DNORM) DNORM = T
   10 CONTINUE
C
   20 TOP = ZERO
      DO 40 I=1,NFIND
        LIM = NDXOUT(I)
        SUM = ZERO
        DO 30 K=NVARYP,LIM
          SUM = SUM + DABS(DERIVS(K,I))
   30   CONTINUE
        IF (SUM.GT.TOP) TOP = SUM
   40 CONTINUE
C
      BOTTOM = ZERO
      GO TO (50, 80, 110, 140), MPICK
C ERE.
   50 DO 70 I=1,NFIND
        SUM = ZERO
        DO 60 K=1,NVARY
          SUM = SUM + DABS(DERIVS(K,I)*VALUE(K))
   60   CONTINUE
        IF (SUM.GT.BOTTOM) BOTTOM = SUM
   70 CONTINUE
      GO TO 170
C RHO2.
   80 DO 100 I=1,NFIND
        SUM = ZERO
        DO 90 K=1,NVARY
          SUM = SUM + DABS(DERIVS(K,I)*VALUE(K))
   90   CONTINUE
        IOUT = NDXOUT(I)
        SUM = SUM + DABS(VALUE(IOUT))
        IF (SUM.GT.BOTTOM) BOTTOM = SUM
  100 CONTINUE
      GO TO 170
C ERL.
  110 DO 130 I=1,NFIND
        SUM = ZERO
        DO 120 K=1,NVARY
          SUM = SUM + DABS(DERIVS(K,I))
  120   CONTINUE
        SUM = SUM*DNORM
        IF (SUM.GT.BOTTOM) BOTTOM = SUM
  130 CONTINUE
      GO TO 170
C RHO4.
  140 VNORM = ZERO
      DO 160 I=1,NFIND
        SUM = ZERO
        DO 150 K=1,NVARY
          SUM = SUM + DABS(DERIVS(K,I))
  150   CONTINUE
        SUM = SUM*DNORM
        IF (SUM.GT.BOTTOM) BOTTOM = SUM
        IOUT = NDXOUT(I)
        T = DABS(VALUE(IOUT))
        IF (T.GT.VNORM) VNORM = T
  160 CONTINUE
      BOTTOM = BOTTOM + VNORM
  170 IF (BOTTOM.EQ.ZERO) GO TO 190
      RHO = TOP/BOTTOM
  180 RETURN
  190 RHO = ZERO
      NERR = 3
      GO TO 180
      END
C

      SUBROUTINE CONDIT( VALUE, DERIVS, RHO, NDXOUT, NVARY,
     *                  LIMITS, NFIND, NPICK, NERR, NDCOL )
C THIS SUBROUTINE COMPUTES CONDITION NUMBERS OMEGA(BK,FK).  HERE
C B IS THE DERIVATIVE OF THE OUTPUT WITH RESPECT TO THE INPUT,
C F IS A DIAGONAL MATRIX OF OUTPUT VALUES (OR THE NORM OF THE
C OUTPUT), AND THE K ARE 2-NORM UNIT BALLS.
      INTEGER LIMITS
      DOUBLE PRECISION VALUE(*),DERIVS(LIMITS,*),RHO
      INTEGER NDXOUT(*),NVARY,NFIND,NPICK
      
      
      DOUBLE PRECISION DATNRM,OUTJ,TRY,ZERO
      INTEGER NDCOL(*)
      INTEGER NERR
      
      INTEGER I, J, JOUT, IERR
      DOUBLE PRECISION OUTNRM
      
      LOGICAL LENTRY
      SAVE
      DATA ZERO /0.0D0/
      LENTRY = IABS(NPICK) .EQ. 7
      IF (LENTRY) GO TO 40
C
C                                       HERE FOR JWE,WKE,ERE.
C
C ULP = UNIT IN THE LAST PLACE OF EACH ENTRY.
      DO 20 J = 1,NFIND
         NDCOL(J) = NVARY
         JOUT = NDXOUT(J)
         OUTJ = VALUE(JOUT)
         IF (OUTJ .EQ. ZERO) GO TO 1000
         DO 10 I = 1,NVARY
            DERIVS(I,J) = VALUE(I)*DERIVS(I,J)/OUTJ
   10       CONTINUE
   20    CONTINUE
      GO TO 65
C
C                                        HERE FOR JWL,WKL,ERL.
C
C ULP = UNIT IN THE LAST PLACE OF THE LARGEST ENTRY.
C COMPUTE THE NORM OF THE DATA.
   40 DATNRM = ZERO
      DO 50 I = 1,NVARY
         TRY = DABS(VALUE(I))
         IF (TRY .GT. DATNRM) DATNRM = TRY
   50    CONTINUE
C COMPUTE THE NORM OF THE OUTPUT
      OUTNRM = ZERO
      DO 60 J = 1,NFIND
         NDCOL(J) = NVARY
         JOUT = NDXOUT(J)
         TRY = DABS(VALUE(JOUT))
         IF (TRY .GT. OUTNRM) OUTNRM = TRY
   60    CONTINUE
      IF (OUTNRM .EQ. ZERO) GO TO 1000
C
C                                            FINISH UP.
C
   65 IF (NVARY .GT. NFIND) 
     *    CALL SQUARE(DERIVS,NDCOL,NFIND,.FALSE.,LIMITS)
      CALL DIAGON (DERIVS,DERIVS,NFIND,.FALSE.,IERR,LIMITS)
      IF (IERR .NE. 0) GO TO 1000
      RHO = ZERO
      DO 70 I = 1,NFIND
         TRY = DERIVS(I,I)
         IF (TRY .GT. RHO) RHO = TRY
   70    CONTINUE
      IF (LENTRY) RHO = DATNRM*RHO/OUTNRM
      RETURN
C
C THE COMPUTATION FAILED.
 1000 RHO = ZERO
      NERR = 6
      RETURN
      END



      SUBROUTINE SQUARE (A,NDCOL,NFIND,SWAP,LIMITS)
C THIS PROCEDURE APPLIES HOUSEHOLDER TRANSFORMATIONS TO A PORTION
C OF THE 'DERIVS' MATRIX, REDUCING IT TO AN NFIND-BY-NFIND MATRIX.
C     A IS THE MATRIX TO BE REDUCED.
C     NDCOL(I) IS THE END OF COLUMN I.
C     SWAP = .TRUE. IF THE COLUMNS NEED NOT BE IN NONDECREASING
C      ORDER OF LENGTH.
      
      INTEGER LIMITS
      DOUBLE PRECISION A(LIMITS,*),P,S,T,SUM,ZERO
      DOUBLE PRECISION DSQRT
      INTEGER NDCOL(*), ICOL(1000)
      
      INTEGER I, J, NFIND, NFM1, NSWEEP, LAST, ICI, ICIP1
      INTEGER KK, K, LIM, KKP1, JJ
      
      LOGICAL SWAP
      SAVE
      DATA ZERO /0.0D0/
      
      DO 10 I = 1,NFIND
   10    ICOL(I) = I
      IF (NFIND.EQ.1 .OR. .NOT.SWAP) GO TO 35
C BUBBLE SORT.  ORDER COLUMNS BY INCREASING LENGTH.
C THE K-TH SHORTEST COLUMN IS LOCATED IN A(*,ICOL(K)).
      NFM1 = NFIND - 1
      DO 30 NSWEEP = 1,NFM1
         LAST = NFIND - NSWEEP
         DO 20 I = 1,LAST
            ICI = ICOL(I)
            ICIP1 = ICOL(I+1)
            IF (NDCOL(ICIP1) .GE. NDCOL(ICI)) GO TO 20
            ICOL(I) = ICIP1
            ICOL(I+1) = ICI
   20       CONTINUE
   30    CONTINUE
C
   35 DO 90 KK = 1,NFIND
         K = ICOL(KK)
         LIM = NDCOL(K)
         SUM = ZERO
         DO 40 I = KK,LIM
            SUM = SUM + A(I,K)**2
   40       CONTINUE
         S = DSQRT(SUM)
         IF (SUM .EQ. ZERO) GO TO 70
         IF (A(KK,K) .LT. ZERO) S = -S
         A(KK,K) = A(KK,K) + S
         P = S*A(KK,K)
         IF (KK .EQ. NFIND) GO TO 90
         KKP1 = KK + 1
         DO 60 JJ = KKP1,NFIND
            J = ICOL(JJ)
            T = ZERO
            DO 50 I = KK,LIM
               T = T + A(I,K)*A(I,J)
   50          CONTINUE
            T = T/P
            DO 60 I = KK,LIM
               A(I,J) = A(I,J) - T*A(I,K)
   60          CONTINUE
   70    DO 80 I = KKP1,NFIND
            A(I,K) = ZERO
   80       CONTINUE
   90    A(KK,K) = -S
      RETURN
      END
C
C
C
      SUBROUTINE DIAGON (B,A,NFIND,DOATOO,IERR,LIMITS)
      
      INTEGER LIMITS
  
      DOUBLE PRECISION B(LIMITS,*), A(LIMITS,*),
     *                  BX,C,CS,EMM1,EL,F,G,ONE
     *                 ,P,R,S,SCALE,SHIFT,SL,SM,SMM1,SN,SUM,T,TEST,T1
     *                 ,TWO,ZERO,ZTEST
      DOUBLE PRECISION DSQRT,DABS
      
      INTEGER NFIND, LIM, I, J, K, L, LL, M, KP1, MAXIT, ITER
      INTEGER KASE, LP1, MP1, LLS, LS, MM1, KK, LM1
      INTEGER IROW, IERR
      
      LOGICAL DOATOO
      SAVE
      DATA ONE /1.0D0/, TWO /2.0D0/, ZERO /0.0D0/

C THIS SUBROUTINE USES ORTHOGONAL TRANSFORMATIONS TO REDUCE THE
C SQUARE MATRIX B TO DIAGONAL FORM.  IT IS INSPIRED BY, INDEED
C MUCH OF IT IS COPIED FROM, SUBROUTINE SSVDC OF LINPACK.
C IF DOATOO THEN ANY ORTHOGONAL TRANSFORMATIONS IN THE OUTPUT SPACE
C MUST BE DONE TO A, TOO.  OTHERWISE, A IS NOT REFERENCED IN ANY WAY.
C
C FIRST REDUCE B TO BIDIAGONAL FORM.
C
      IF (NFIND .NE. 1) GO TO 5
      IF (B(1,1) .GE. ZERO ) GO TO 620
      B(1,1) = -B(1,1)
      IF (.NOT. DOATOO) GO TO 620
      A(1,1) = -A(1,1)
      GO TO 620
   5  LIM = NFIND -1
      DO 200 K = 1,LIM
C PERFORM AN ORTHOGONAL TRANSFORMATION IN THE OUTPUT SPACE.
         KP1 = K + 1
         SUM = ZERO
         DO 10 I = K,NFIND
            SUM = SUM + B(K,I)**2
   10       CONTINUE
         IF (SUM .EQ. ZERO) GO TO 100
         S = DSQRT(SUM)
         IF (B(K,K) .LT. ZERO) S = -S
         B(K,K) = B(K,K) + S
         P = S*B(K,K)
         DO 40 J = KP1,NFIND
            T = ZERO
            DO 20 I = K,NFIND
               T = T + B(K,I)*B(J,I)
   20          CONTINUE
            T = T/P
            DO 30 I = K,NFIND
               B(J,I) = B(J,I) - T*B(K,I)
   30          CONTINUE
   40       CONTINUE
         IF (.NOT.DOATOO) GO TO 80
         DO 70 J = 1,NFIND
            T = ZERO
            DO 50 I = K,NFIND
               T = T + B(K,I)*A(J,I)
   50          CONTINUE
            T = T/P
            DO 60 I = K,NFIND
               A(J,I) = A(J,I) - T*B(K,I)
   60          CONTINUE
   70       CONTINUE
   80    B(K,K) = -S
C
  100    IF (K .EQ. LIM) GO TO 200
C PERFORM AN ORTHOGONAL TRANSFORMATION IN THE ERROR SPACE.
         SUM = ZERO
         DO 110 I = KP1,NFIND
            SUM = SUM + B(I,K)**2
  110       CONTINUE
         IF (SUM .EQ. ZERO) GO TO 200
         S = DSQRT(SUM)
         IF (B(KP1,K) .LT. ZERO) S = -S
         B(KP1,K) = B(KP1,K) + S
         P = S*B(KP1,K)
         DO 140 J = KP1,NFIND
            T = ZERO
            DO 120 I = KP1,NFIND
               T = T + B(I,K)*B(I,J)
  120          CONTINUE
           T = T/P
            DO 130 I = KP1,NFIND
               B(I,J) = B(I,J) - T*B(I,K)
  130          CONTINUE
  140       CONTINUE
         B(KP1,K) = -S
  200    CONTINUE
C
C MAIN ITERATION LOOP FOR THE SINGULAR VALUES.
C
      MAXIT = 50
      M = NFIND
      ITER = 0
  360 CONTINUE
C
C QUIT IF ALL THE SINGULAR VALUES HAVE BEEN FOUND.
C
C ...EXIT
      IF (M .EQ. 0) GO TO 620
C
C IF TOO MANY ITERATIONS HAVE BEEN PERFORMED, GO TO ERROR EXIT.
C
      IF (ITER .GE. MAXIT) GO TO 1000
C
C THIS SECTION OF THE PROGRAM INSPECTS FOR NEGLIGIBLE ELEMENTS OF B.
C ON COMPLETION THE VARIABLES KASE AND L ARE SET AS FOLLOWS.
C
C   KASE = 1 IF B(M,M) AND B(L,L-1) ARE NEGLIGIBLE AND L.LT.M
C   KASE = 2 IF B(L,L) IS NEGLIGIBLE AND L.LT.M
C   KASE = 3 IF B(L,L-1) IS NEGLIGIBLE, L.LT.M, AND B(L,L),...,
C            B(M,M) ARE NOT NEGLIGIBLE (QR STEP).
C   KASE = 4 IF B(M,M-1) IS NEGLIGIBLE (CONVERGENCE).
C
         DO 390 LL = 1,M
            L = M - LL
C        ...EXIT
            IF (L .EQ. 0) GO TO 400
            TEST = DABS(B(L,L)) + DABS(B(L+1,L+1))
            ZTEST = TEST + DABS(B(L+1,L))
            IF (ZTEST .NE. TEST) GO TO 380
               B(L+1,L) = ZERO
C        ......EXIT
               GO TO 400
  380       CONTINUE
  390    CONTINUE
  400    CONTINUE
         IF (L .NE. M - 1) GO TO 410
             KASE = 4
         GO TO 480
  410    CONTINUE
            LP1 = L + 1
            MP1 = M + 1
            DO 430 LLS = LP1, MP1
               LS = M - LLS + LP1
C         .....EXIT
               IF (LS .EQ. L) GO TO 440
               TEST = ZERO
               IF (LS .NE. M) TEST = TEST + DABS(B(LS+1,LS))
               IF (LS .NE. L + 1) TEST = TEST + DABS(B(LS,LS-1))
               ZTEST = TEST + DABS(B(LS,LS))
               IF (ZTEST .NE. TEST) GO TO 420
                  B(LS,LS) = ZERO
C             ....EXIT
                  GO TO 440
  420          CONTINUE
  430       CONTINUE
  440       CONTINUE
            IF (LS .NE. L) GO TO 450
               KASE = 3
            GO TO 470
  450       CONTINUE
            IF (LS .NE. M) GO TO 460
               KASE = 1
            GO TO 470
  460       CONTINUE
               KASE = 2
               L = LS
  470       CONTINUE
  480    CONTINUE
         L = L + 1
C
C        PERFORM THE TASK INDICATED BY KASE.
C
         GO TO (490,520,540,570), KASE
C
C        DEFLATE NEGLIGIBLE B(M,M).
C
  490    CONTINUE
            MM1 = M - 1
            F = B(M,M-1)
            B(M,M-1) = ZERO
            DO 510 KK = L,MM1
               K = MM1 - KK + L
               T1 = B(K,K)
               SCALE = DABS(F) + DABS(T1)
               IF (SCALE .NE. ZERO) GO TO 498
                  CS = ONE
                  SN = ZERO
                  R = ZERO
                  GO TO 499
  498          R = SCALE*DSQRT((F/SCALE)**2 + (T1/SCALE)**2)
               CS = T1/R
               SN = F/R
  499          B(K,K) = R
               IF (K .EQ. L) GO TO 500
                  F = -SN*B(K,K-1)
                  B(K,K-1) = CS*B(K,K-1)
  500          CONTINUE
  510       CONTINUE
         GO TO 610
C
C SPLIT AT NEGLIGIBLE B(L,L).
C
  520    CONTINUE
            LM1 = L - 1
            F = B(L,L-1)
            B(L,L-1) = ZERO
            DO 530 K = L,M
               T1 = B(K,K)
               SCALE = DABS(F) + DABS(T1)
               IF (SCALE .NE. ZERO) GO TO 523
                  CS = ONE
                  SN = ZERO
                  R = ZERO
                  GO TO 524
  523          R = SCALE*DSQRT((F/SCALE)**2 + (T1/SCALE)**2)
               CS = T1/R
               SN = F/R
  524          B(K,K) = R
               IF (.NOT.DOATOO) GO TO 528
               DO 525 IROW = 1,NFIND
                  T = -SN*A(IROW,K) + CS*A(IROW,LM1)
                  A(IROW,K) = CS*A(IROW,K) + SN*A(IROW,LM1)
                  A(IROW,LM1) = T
  525             CONTINUE
  528          IF (K .EQ. M) GO TO 530
               F = -SN*B(K+1,K)
               B(K+1,K) = CS*B(K+1,K)
  530       CONTINUE
         GO TO 610
C
C PERFORM ONE QR STEP.
C
  540    CONTINUE
C
C CALCULATE THE SHIFT.
C
            SCALE = DABS(B(M,M)) + DABS(B(M-1,M-1)) + DABS(B(M,M-1))
     *            + DABS(B(L,L)) + DABS(B(L+1,L))
            SM = B(M,M)/SCALE
            SMM1 = B(M-1,M-1)/SCALE
            EMM1 = B(M,M-1)/SCALE
            SL = B(L,L)/SCALE
            EL = B(L+1,L)/SCALE
            BX = ((SMM1 + SM)*(SMM1 - SM) + EMM1**2)/TWO
            C = (SM*EMM1)**2
            SHIFT = ZERO
            IF (BX .EQ. ZERO .AND. C .EQ. ZERO) GO TO 550
               SHIFT = DSQRT(BX**2+C)
               IF (BX .LT. ZERO) SHIFT = -SHIFT
               SHIFT = C/(BX + SHIFT)
  550       CONTINUE
            F = (SL + SM)*(SL - SM) - SHIFT
            G = SL*EL
C
C CHASE ZEROS.
C
            MM1 = M -1
            DO 560 K = L,MM1
               SCALE = DABS(F) + DABS(G)
               IF (SCALE .NE. ZERO) GO TO 552
                  CS = ONE
                  SN = ZERO
                  R = ZERO
                  GO TO 553
  552          R = SCALE*DSQRT((F/SCALE)**2 + (G/SCALE)**2)
               CS = F/R
               SN = G/R
  553          IF (K .NE. L) B(K,K-1) = R
               F = CS*B(K,K) + SN*B(K+1,K)
               B(K+1,K) = CS*B(K+1,K) - SN*B(K,K)
               G = SN*B(K+1,K+1)
               B(K+1,K+1) = CS*B(K+1,K+1)
               SCALE = DABS(F) + DABS(G)
               IF (SCALE .NE. ZERO) GO TO 556
                  CS = ONE
                  SN = ZERO
                  R = ZERO
                  GO TO 557
  556          R = SCALE*DSQRT((F/SCALE)**2 + (G/SCALE)**2)
               CS = F/R
               SN = G/R
  557          B(K,K) = R
               F = CS*B(K+1,K) + SN*B(K+1,K+1)
               B(K+1,K+1) = -SN*B(K+1,K) + CS*B(K+1,K+1)
               IF (.NOT.DOATOO) GO TO 559
               DO 558 IROW = 1,NFIND
                  T = -SN*A(IROW,K) + CS*A(IROW,K+1)
                  A(IROW,K) = CS*A(IROW,K) + SN*A(IROW,K+1)
                  A(IROW,K+1) = T
  558             CONTINUE
  559          IF (K .EQ.MM1) GO TO 560
               G = SN*B(K+2,K+1)
               B(K+2,K+1) = CS*B(K+2,K+1)
  560       CONTINUE
            B(M,M-1) = F
            ITER = ITER + 1
         GO TO 610
C
C CONVERGENCE
C
  570    CONTINUE
C
C MAKE THE SINGULAR VALUE POSITIVE.
C
            IF (B(L,L) .LT. ZERO) B(L,L) = -B(L,L)
  600       CONTINUE
            ITER = 0
            M = M - 1
  610    CONTINUE
      GO TO 360
  620 CONTINUE
      IERR = 0
      RETURN
 1000 IERR = 1
      RETURN
      END

      double precision function XPRMNT(deriv, data, value,
     1       ndxout, nvary, limits, nfind)
      integer nvary, nodes, nfind
      integer limits(*)
      double precision deriv(limits(2),*), data(*), value(*)
      integer ndxout(*)
      nodes = limits(1)
      XPRMNT = 0.0D0
      return
      end


