#include "CGlobals.h"
#include "CRound.h"
#include "CRoundUkkonen.h"
#include "miller.h"

void *DecodeMATLABptr( const mxArray *MATLABptr );
void CGlobals::ClearErr( void )
{
	int i;
	for ( i = 0; i < 11; ++i )
		m_pNERR[i] = 0L;
	m_bStop = false;
	m_bTerminated = false;
}

int CGlobals::SetAlgName( const mxArray *MatlabStr )
{
	int NameLen;
	int nNumOfAlgs = 0;

	delete [] m_pAlgName[0];
	m_pAlgName[0] = 0;
	m_pAlgName[1] = 0;
	NameLen = mxGetNumberOfElements( MatlabStr ) + 1;
    if ( NameLen <= 1 )
		return 0;
	m_pAlgName[0] = new char[ NameLen ];
	if ( mxGetString( MatlabStr, m_pAlgName[0], NameLen ) != 0 )
		throw CException( "mxGetString returned a bad exit status!" );
		
	m_pAlgName[1] = strchr( m_pAlgName[0], '/' );

	
	if ( m_pAlgName[1] != 0 && strlen( m_pAlgName[1] ) > 1 )
	{
		nNumOfAlgs = 2;
		*( m_pAlgName[1]++ ) = '\0';
	}
	else
	{
		nNumOfAlgs = 1;
		m_pAlgName[1] = 0;
	}

	VerifyAlgName();


	return nNumOfAlgs;

}

void CGlobals::VerifyAlgName( void )
{
	mxArray *parg;
	mxArray *plhs;

	int stop;

	char ErrorText[256];
	int i;
	
	for ( i = 0; i < GetNumOfAlgs(); ++i )
	{
		parg = mxCreateString( m_pAlgName[i] );
		mexSetTrapFlag( 1 );
		stop = mexCallMATLAB( 1, &plhs, 1, &parg, "exist" );
		mexSetTrapFlag( 0 );
		mxDestroyArray( parg );
		if ( stop != 0 )
		{
			mxDestroyArray( plhs );
			throw CException( "MATLAB routine \"exist\" returned a bad exit status!" );
		}
		stop = int( mxGetScalar( plhs ) );
		mxDestroyArray( plhs );
		if ( stop != 2.0 && stop != 6.0 && stop != 3.0 )
		{
			sprintf( ErrorText, "Algorithm: \"%s\" not found!", m_pAlgName[i] ); 
			throw CException( ErrorText );
		}
	}
}
	


void CGlobals::cppCallBack( mxArray *plhs[], const mxArray *prhs[] )
{
	int CommCode;
	double dRet;
	long parindex;
	

	CommCode = int( mxGetScalar( prhs[0] ) );
	switch ( CommCode )
	{
	case 1:
		//CONSTANT
		plhs[0] = mxCreateDoubleScalar ( cppInsertConst( mxGetScalar( prhs[1] ) ) );
		break;
	case 2:
		//INPUT
		try
		{
			plhs[0] = mxCreateDoubleScalar( cppInsertInput( &dRet ) );
			plhs[1] = mxCreateDoubleScalar( dRet );
		}
		catch (...)
		{
			plhs[1] = mxCreateDoubleScalar( 0.0 );
			throw;
		}
		break;
	case 3:
		//UNARY
		plhs[0] = mxCreateDoubleScalar( cppInsertUnary( long(mxGetScalar( prhs[1] )), 
		enum operation(int( mxGetScalar( prhs[2] ) ) ) ) );
		break;
	case 4:
		//BINARY
		plhs[0] = mxCreateDoubleScalar( cppInsertBinary( long(mxGetScalar( prhs[1] )), enum operation(int(mxGetScalar( prhs[2] ))),
		long(mxGetScalar( prhs[3] ) ) ) );
		break;
	case 5:
		//OUTPUT
		plhs[0] = mxCreateDoubleScalar( cppInsertOutput( long(mxGetScalar( prhs[1] )) ) );
		break;
	case 6:
		//COMPUTATIONAL GRAPH IS VARIABLE
		plhs[0] = mxCreateDoubleScalar( cppMakeGraphVariable() );
		break;
	case 7:
		//EXCEPTION
		plhs[0] = mxCreateDoubleScalar( cppException( enum err(int(mxGetScalar( prhs[1] ))) ) );
		break;
	case 8:
		//CONSTRAINT
		plhs[0] = mxCreateDoubleScalar( cppConstraint( mxGetScalar( prhs[1] ) ) );
		break;
	case 9:
		//COMPOSITION
		plhs[0] = mxCreateDoubleScalar( cppComposition() );
		break;
	case 10:
		//COPY CONSTRUCTOR
		plhs[0] = mxDuplicateArray( prhs[1] );
		break;
	case 11:
		//DESTRUCTOR
		plhs[0] = mxCreateDoubleScalar( 1.0 );
		break;
	case 12:
		//ASSIGNMENT OPERATOR
		plhs[0] = mxDuplicateArray( prhs[1] ); 
		break;
		//GET THE NUMBER OF INPUTS
	case 13:
		plhs[0] = mxCreateDoubleScalar( cppGetDatSize() );
		break;
		//GET PARAMETER ARGUMENT
	case 14:
		parindex = long( mxGetScalar( prhs[1] ) );
		if ( parindex <= 0 || parindex > m_nNumOfPar )
		{
			plhs[1] = mxCreateDoubleScalar( 0.0 );
			throw CException( invalid_par_index );
		}
		plhs[1] = mxCreateDoubleScalar( m_pParArray[ parindex - 1L ] );
		plhs[0] = mxCreateDoubleScalar( 1.0 );
		break;
		//FILL END
	case 15:
		ClearCallBack();
		if ( m_bCallBackError )
		{
			plhs[0] = mxCreateDoubleScalar( 0.0 );
		}
		else
		{
			FillEnd();
			plhs[0] = mxCreateDoubleScalar( 1.0 );
		}
		break;
		//ASSIGN GRAPH
	case 16:
		{
			void *ptr;
			if ( ( ptr = DecodeMATLABptr( prhs[1] ) ) != 0 )
			{
				const CGlobals &other = *( CGlobals * )ptr;
				CAssert::NotNull( m_pTheGraph );
				CAssert::NotNull( other.m_pTheGraph );
				CCompGraph &target = *m_pTheGraph;
				const CCompGraph &source = *other.m_pTheGraph;
				CGraphManager *tmp = &target.GetManager();
				*tmp = source;
				target = source;
				target.attach( tmp );
				plhs[0] = mxCreateDoubleScalar( 1.0 );

			}
			else
			{
				throw CException( "Invalid handle!" );
			}
		}

		break;
	default:
		throw CException( "Invalid method code!" );
		break;
	}
	
	return;
}

void CGlobals::UkkCallBack( mxArray *plhs[], const mxArray *prhs[] )
{
	int CommCode;
	double dRet;
	long parindex;
	
	CommCode = int( mxGetScalar( prhs[0] ) );
	switch ( CommCode )
	{
	case 1:
		//CONSTANT
		plhs[0] = mxCreateDoubleScalar( m_pUkkonen->Insert() );
		break;
	case 2:
		//INPUT
		plhs[0] = mxCreateDoubleScalar(  m_pUkkonen->input( &dRet ) );
		plhs[1] = mxCreateDoubleScalar( dRet );
		break;
	case 3:
		//UNARY
		plhs[0] = mxCreateDoubleScalar( m_pUkkonen->Insert( enum operation( int( mxGetScalar( prhs[2] ) ) ), 
			long(mxGetScalar( prhs[1] )), mxGetScalar( prhs[4] ), mxGetScalar( prhs[3] ) ) );
		break;
	case 4:
		//BINARY
		plhs[0] = mxCreateDoubleScalar( m_pUkkonen->Insert( enum operation( int( mxGetScalar( prhs[2] ) ) ), 
			long(mxGetScalar( prhs[1] )), mxGetScalar( prhs[6] ),
			mxGetScalar( prhs[4] ), long(mxGetScalar( prhs[3] ) ), mxGetScalar( prhs[5] )  ) );
		break;
	case 5:
		//OUTPUT
		m_pUkkonen->Output( long( mxGetScalar( prhs[1] ) ), mxGetScalar( prhs[2] ) );
		plhs[0] = mxCreateDoubleScalar( 1.0 );
		break;
	case 6:
		//COMPUTATIONAL GRAPH IS VARIABLE
		plhs[0] = mxCreateDoubleScalar( 1.0 );
		break;
	case 7:
		//EXCEPTION
		throw CException( enum err(int( mxGetScalar( prhs[1] ) ) ) );
		break;
	case 8:
		//CONSTRAINT
		m_pConstr->insert( mxGetScalar( prhs[1] ) );
		plhs[0] = mxCreateDoubleScalar( 1.0 );
		break;
	case 9:
		//COMPOSITION
		if ( m_pAlgName[1] != 0 )
			throw CException( comp_error_free );
		plhs[0] = mxCreateDoubleScalar( m_pUkkonen->Composition() );
		break;
	case 10:
		//COPY CONSTRUCTOR
		plhs[0] = mxCreateDoubleScalar( m_pUkkonen->Copy( long( mxGetScalar( prhs[1] ) ) ) );
		break;
	case 11:
		//DESTRUCTOR
		m_pUkkonen->Release( long( mxGetScalar( prhs[1] ) ) );
		plhs[0] = mxCreateDoubleScalar( 1.0 );
		break;
	case 12:
		//ASSIGNMENT OPERATOR
		plhs[0] = mxCreateDoubleScalar( m_pUkkonen->Copy( long( mxGetScalar( prhs[1] ) ), 
			long( mxGetScalar( prhs[2] ) ) ) );
		break;
		//GET THE NUMBER OF INPUTS
	case 13:
		plhs[0] = mxCreateDoubleScalar( cppGetDatSize() );
		break;
		//GET PARAMETER ARGUMENT
	case 14:
		parindex = long( mxGetScalar( prhs[1] ) );
		if ( parindex <= 0 || parindex > m_nNumOfPar )
		{
			plhs[1] = mxCreateDoubleScalar( 0.0 );
			throw CException( invalid_par_index );
		}
		plhs[1] = mxCreateDoubleScalar( m_pParArray[ parindex - 1L ] );
		plhs[0] = mxCreateDoubleScalar( 1.0 );
		break;

	}
	

	return;
}

CRoundBase *CGlobals::InitEngine( double *pData, long Datsize, long nPick, double dStopx )
{
	delete m_pTheEngine;
	m_pTheEngine = 0;
	int NumOfAlgs = GetNumOfAlgs();
	if ( NumOfAlgs == 0 )
		NumOfAlgs = 1;
	if ( m_bUkkonen )
		m_pTheEngine = new CRoundUkkonen( pData, Datsize, nPick, dStopx, NumOfAlgs, 
		m_bUkkonenGraph );
	else
		m_pTheEngine = new CRound( pData, Datsize, nPick, dStopx, NumOfAlgs );
	return m_pTheEngine;
}

void CGlobals::ListExceptions( void )
{
	char OutStr[256];
	if ( m_pNERR[0] > 0 )
	{
		sprintf( OutStr, "\n %d attempted divisions by zero.\n", m_pNERR[0] );
		mPrint( OutStr );
	}
	if ( m_pNERR[1] > 0 )
	{
		sprintf( OutStr, "\n %d attempted SQRT(V), V LT 0.\n", m_pNERR[1] );
		mPrint( OutStr );
	}
	if ( m_pNERR[2] > 0 )
	{
		sprintf( OutStr, "\n %d GETER failures.\n", m_pNERR[2] );
		mPrint( OutStr );
	}
	if ( m_pNERR[3] > 0 )
	{
		sprintf( OutStr, "\n %d OMEGA failures.\n", m_pNERR[3] );
		mPrint( OutStr );
	}
	if ( m_pNERR[4] > 0 )
	{
		sprintf( OutStr, "\n %d DIAGON failures.\n", m_pNERR[4] );
		mPrint( OutStr );
	}
	if ( m_pNERR[5] > 0 )
	{
		sprintf( OutStr, "\n %d CONDIT failures.\n", m_pNERR[5] );
		mPrint( OutStr );
	}
	if ( m_pNERR[6] > 0 )
	{
		sprintf( OutStr, "\n %d User error.\n", m_pNERR[6] );
		mPrint( OutStr );
	}


}
	
	

void CGlobals::selfinit( const CGlobals &other )
	{
		int i;

		ClearEngine();

	    
		if ( other.m_pAlgName[0] != 0 )
		{
			m_pAlgName[0] = new char[ strlen( other.m_pAlgName[0] ) + 
				( other.m_pAlgName[1] ? strlen( other.m_pAlgName[1] ) : -1L ) + 2L ];
			strcpy( m_pAlgName[0], other.m_pAlgName[0] );
			if ( other.m_pAlgName[1] != 0 )
			{
				m_pAlgName[1] = m_pAlgName[0] + ( other.m_pAlgName[1] - other.m_pAlgName[0] );
				strcpy( m_pAlgName[1], other.m_pAlgName[1] );
			}
		}
		m_bCallBack = other.m_bCallBack;
		m_bCallBackError = other.m_bCallBackError;

		m_bUkkonen = other.m_bUkkonen;
		m_bUkkonenGraph = other.m_bUkkonenGraph;

		if ( other.m_pTheEngine != 0 )
		{
			if ( m_bUkkonen )
				m_pTheEngine = new CRoundUkkonen( *(CRoundUkkonen *)(other.m_pTheEngine) );
			else
				m_pTheEngine = new CRound( *(CRound *)(other.m_pTheEngine) );
		}

		CAssert::Null( m_pTheGraph );

		m_pTheGraph = other.m_pTheGraph;
		m_pConstr = other.m_pConstr;
		m_pUkkonen = other.m_pUkkonen;

		m_Mode = other.m_Mode;

		m_CallBackException = other.m_CallBackException;

		m_bStop = other.m_bStop;
		m_bTerminated = other.m_bTerminated;

		for( i = 0; i < 11; ++i )
			m_pNERR[i] = other.m_pNERR[i];

		if ( other.m_nNumOfPar != 0 )
		{
			m_pParArray = new double[ other.m_nNumOfPar ];
			m_nNumOfPar = m_nParArraySize = other.m_nNumOfPar;
			for ( i = 0; i < m_nNumOfPar; ++i )
				m_pParArray[i] = other.m_pParArray[i];
		}

		if ( other.m_nNumOfData != 0 )
		{
			m_pDatArray = new double[ other.m_nNumOfData ];
			m_nNumOfData = m_nDatArraySize = other.m_nNumOfData;
			for ( i = 0; i < m_nNumOfData; ++i )
				m_pDatArray[i] = other.m_pDatArray[i];
		}
	}

void CGlobals::GetExceptions( double *pr )
{
	int i;
	for( i = 0; i < 11; ++i )
		pr[i] = m_pNERR[i];
}

double *CGlobals::SetDataCopy( long size, double *pD )
{
	long i;
	if ( size != m_nNumOfData )
	{
		
		if ( size > m_nDatArraySize )
		{
			delete [] m_pDatArray;
		    m_pDatArray = 0;
			m_pDatArray = new double[ size ];
			m_nDatArraySize = size;
		}
		m_nNumOfData = size;
	}
	for ( i = 0; i < size; ++i )
	{
		m_pDatArray[i] = pD[i];
	}
	return m_pDatArray;

}


bool CGlobals::SetParams( long size, double *pPar )
{
	
	long i;
	if ( size != m_nNumOfPar )
	{
		
		if ( size > m_nParArraySize )
		{
			delete [] m_pParArray;
		    m_pParArray = 0;
			m_pParArray = new double[ size ];
			m_nParArraySize = size;
		}
		for ( i = 0; i < size; ++i )
		{
			m_pParArray[i] = pPar[i];
		}
		m_nNumOfPar = size;
		return true;
	}
	i = 0;
	while ( i < size && pPar[i] == m_pParArray[i]  )
		++i;
	if ( i < size )
	{
		
		for ( ; i < size; ++i )
		{
			m_pParArray[i] = pPar[i];
		}
		return true;
	}
	return false;
}

