#ifndef _CGLOBALS_
#define _CGLOBALS_

#include "CRoundBase.h"
#include "CUkkonen.h"
#include "CGraphPtr.h"

#include <mex.h>
class CGlobals
{
private:
	CRoundBase *m_pTheEngine;
	char *m_pAlgName[2];
	
	CException m_CallBackException;
	bool m_bCallBackError;
	bool m_bCallBack;

	bool m_bStop;
	bool m_bTerminated;
	
	int m_nLhsIndex;
	int m_nLhs;
	mxArray **m_gPlhs;

	long m_nParArraySize;
	long m_nNumOfPar;
	double *m_pParArray;

	long m_nDatArraySize;
	long m_nNumOfData;
	double *m_pDatArray;


	bool m_bUkkonen;
	bool m_bUkkonenGraph;

	

	CCompGraph* m_pTheGraph;
	CConstraints* m_pConstr;
	CUkkonen *m_pUkkonen;

	long m_pNERR[11];
	enum EnumMode m_Mode;

	void selfinit( const CGlobals &other );

	long cppInsertConst( double constant )
	{
		CAssert::NotNull( m_pTheGraph );
		return CGraphPtr( m_pTheGraph ).insert( constant ).GetIndex();
	}

	long cppInsertInput( double *RValue )
	{
		CAssert::NotNull( m_pTheGraph );
		return CGraphPtr( m_pTheGraph ).insert( RValue ).GetIndex();
	}

	long cppInsertUnary( long left, enum operation op )
	{
		CAssert::NotNull( m_pTheGraph );
		return CGraphPtr( m_pTheGraph ).insert( left, op, 0L ).GetIndex();
	}

	long cppInsertBinary( long left, enum operation op, long right )
	{
		CAssert::NotNull( m_pTheGraph );
		return CGraphPtr( m_pTheGraph ).insert( left, op, right ).GetIndex();
	}

	long cppInsertOutput( long LValue )
	{
		CAssert::NotNull( m_pTheGraph );
		CGraphPtr( m_pTheGraph, LValue ).output();
		return 1L;
	}

	long cppMakeGraphVariable( void )
	{
		CAssert::NotNull( m_pTheGraph );
		CGraphPtr( m_pTheGraph ).SetVar();
		return 1L;
	}

	long cppConstraint( double constraint )
	{
		CAssert::NotNull( m_pTheGraph );
		CGraphPtr( m_pTheGraph ).SetVar();
		m_pConstr->insert( constraint );
		return 1L;
	}

	long cppException( enum err ErrorCode )
	{
		throw CException( ErrorCode );
	}

	long cppComposition( void )
	{
		CAssert::NotNull( m_pTheGraph );
		CGraphPtr( m_pTheGraph ).composition();
		return 1L;
	}

	long cppGetDatSize( void )
	{
		return m_pTheEngine->GetDatSize();
	}

	void cppCallBack( mxArray *plhs[], const mxArray *prhs[] );

	void UkkCallBack( mxArray *plhs[], const mxArray *prhs[] );

	


public:

	CGlobals( void )
	{			   
		m_pTheEngine = 0;
		m_pAlgName[0] = 0;
		m_pAlgName[1] = 0;
		m_bCallBack = false;
		m_bCallBackError = false;

		m_bUkkonen = false;
		m_bUkkonenGraph = false;

		m_pTheGraph = 0;
		m_pConstr = 0;
		m_pUkkonen = 0;

		ClearErr();
		m_Mode = normal;

		m_bStop = false;
		m_bTerminated = false;

		m_nLhsIndex = 0;
		m_nLhs = 0;
		m_gPlhs = 0;
		
		m_nParArraySize = 0;
		m_nNumOfPar = 0;
		m_pParArray = 0;

		m_nDatArraySize = 0;
		m_nNumOfData = 0;
		m_pDatArray = 0;

	}

	~CGlobals( void )
	{
		ClearCallBackException();
	    delete m_pTheEngine;
		delete [] m_pAlgName[0];
		delete [] m_pParArray;
		delete [] m_pDatArray;
		
		delete m_pTheGraph;
	}

	CGlobals( const CGlobals &other )
	{
		selfinit( other );
	}

	CGlobals &operator=( const CGlobals &other )
	{
		if ( &other == this )
			return *this;
		selfinit( other );
		return *this;
		
	}

	void reset( void )
	{
	    delete m_pTheEngine;
		
		m_pTheEngine = 0;
		m_bCallBack = false;
		ClearCallBackException();
		
		delete m_pTheGraph;
		m_pTheGraph = 0;
		m_pConstr = 0;
		m_pUkkonen = 0;
		ClearErr();
		m_Mode = normal;
		VerifyAlgName();
		m_bStop = false;
		m_bTerminated = false;

		m_nLhsIndex = 0;
		m_nLhs = 0;
		m_gPlhs = 0;


		m_nNumOfPar = 0;
        delete [] m_pParArray;
		m_pParArray = 0;
		m_nParArraySize = 0;

		m_nNumOfData = 0;
		delete [] m_pDatArray;
		m_pDatArray = 0;
		m_nDatArraySize = 0;


	}




	void ClearEngine( void )
	{
	    delete m_pTheEngine;
		m_pTheEngine = 0;
		delete [] m_pAlgName[0];
		m_pAlgName[0] = 0;
		m_pAlgName[1] = 0;
		m_bCallBack = false;
		
		ClearCallBackException();

		m_nLhsIndex = 0;
		m_nLhs = 0;
		m_gPlhs = 0;

		m_nNumOfPar = 0;
		delete [] m_pParArray;
		m_pParArray = 0;
		m_nParArraySize = 0;

		m_nNumOfData = 0;
		delete [] m_pDatArray;
		m_pDatArray = 0;
		m_nDatArraySize = 0;

		m_bUkkonen = false;
		m_bUkkonenGraph = false;


		delete m_pTheGraph;
		m_pTheGraph = 0;
		m_pConstr = 0;
		m_pUkkonen = 0;
		ClearErr();
		m_Mode = normal;

		m_bStop = false;
		m_bTerminated = false;

	}


	void SetHandlerMode( enum EnumMode mode )
	{
		m_Mode = mode;
	}

	enum EnumMode GetMode( void )
	{
		return m_Mode;
	}
		
	
	void ClearErr( void );
	
	const char *GetAlgName( int index ) const
	{
		return m_pAlgName[ index - 1 ];
	}

	int GetNumOfAlgs( void )
	{
	
		return ( m_pAlgName[0] == 0 ? 0 : ( m_pAlgName[1] == 0 ? 1 : 2 ) );
	}

	int SetAlgName( const mxArray *MatlabStr );

	void VerifyAlgName( void );

	bool IsCallBack( void )
	{
		return m_bCallBack;
	}

	void SetCallBack( void )
	{
		m_bCallBack = true;
		ClearCallBackException();
	}

	void ClearCallBack( void )
	{
		m_bCallBack = false;
	}

	void SetMatlabArgs( int nlhs, mxArray *plhs[] )
	{
		m_nLhsIndex = 0;
		m_nLhs = nlhs;
		m_gPlhs = plhs;
	}

	void ClearMatlabArgs( void )
	{
		m_nLhsIndex = 0;
		m_nLhs = 0;
		m_gPlhs = 0;

	}

	double *SetDataCopy( long size, double *pD );

	void CGlobals::GetDataCopy( long &size, double* &pD )
	{
		size = m_nNumOfData;
		pD = m_pDatArray;
	}


	void ClearDataCopy( void )
	{
		delete [] m_pDatArray;
		m_pDatArray = 0;
		m_nNumOfData = m_nDatArraySize = 0;

	}


	bool SetParams( long size, double *pPar );

	void SetUkkonen( void )
	{
		m_bUkkonen = true;
	}

	void SetUkkonenGraph( void )
	{
		m_bUkkonenGraph = true;
	}

	bool IsUkkonen( void )
	{
		return m_bUkkonen;
	}

	bool IsUkkonenGraph( void )
	{
		return m_bUkkonenGraph;
	}

	void SetStop( void )
	{
		m_bStop = true;
	}

	bool IsStop( void )
	{
		return m_bStop;
	}
	void SetTerminated( void )
	{
		m_bTerminated = true;
	}

	bool IsTerminated( void )
	{
		return m_bTerminated;
	}


	CRoundBase *InitEngine( double *pData, long Datsize, long nPick, double dStopx = 0.0 );




	double round( double *input, double &penalty )
	{
		return m_pTheEngine->round( input, penalty, *this );
	}

	bool FillBegin( double *input )
	{
		return m_pTheEngine->FillBegin( input, *this );
	}

	void FillEnd( void )
	{
		m_pTheEngine->FillEnd( *this );
	}

	double ComputeRho( double &penalty )
	{
		ThrowException();
		return m_pTheEngine->ComputeRho( penalty, *this );
	}


/*	void IncCounter( void )
	{
		m_pTheEngine->IncCounter();
	}
*/
	CRoundBase *GetTheEngine( void )
	{
		return m_pTheEngine;
	}

	void InitConstraints( void )
	{
		m_pConstr->InitSpace();
	}

	CCompGraph &GetGraph( void )
	{
		CAssert::NotNull( m_pTheGraph );
		return *m_pTheGraph;
	}

	void SetGraph( CCompGraph* pGraph )
	{
		CAssert::Null( m_pTheGraph );
		if ( m_pTheGraph && m_pTheGraph != pGraph )
			delete m_pTheGraph;
		m_pTheGraph = pGraph;
	}

	void ClearGraph( void )
	{
		CAssert::NotNull( m_pTheGraph );
		delete m_pTheGraph;
		m_pTheGraph = 0;
	}


	void SetConstr( CConstraints *constr )
	{
		m_pConstr = constr;
	}

	void SetUkkonenPtr( CUkkonen *pUkkonen )
	{
		m_pUkkonen = pUkkonen;
	}








	void CallBack( mxArray *plhs[], const mxArray *prhs[] )
	{
		try
		{
			if ( m_bUkkonen && !m_bUkkonenGraph )
				UkkCallBack( plhs, prhs );
			else
				cppCallBack( plhs, prhs );
		}
		catch ( CException exception )
		{
			CAssert::DEBUG_ERROR && CAssert::Assert( !m_bCallBackError );
			if ( int( mxGetScalar( prhs[0] ) ) >= 1000 )
				exception.FinalHandler();
			m_bCallBackError = true;
			m_CallBackException = exception;
			plhs[0] = mxCreateDoubleScalar( 0.0 );
		}
		return;
	}


	void ThrowException( void )
	{
		if ( m_bCallBackError )
		{
			m_bCallBackError = false;
			throw m_CallBackException;
		}
	}

	void ClearCallBackException( void )
	{
		if ( m_bCallBackError )
		{
			m_bCallBackError = false;
			m_CallBackException.FinalHandler( this, false );
		}
	}


/*
	void SetScalarLhs( double rho )
	{
		if ( m_nLhsIndex < m_nLhs || m_nLhsIndex == 0 )
			m_gPlhs[ m_nLhsIndex++ ] = mxCreateScalarDouble( rho );
	
	}
	*/

	double *GetArrayLhs( long SizeM, long SizeN = 1L )
	{
		if ( m_nLhsIndex < m_nLhs )
		{
			m_gPlhs[ m_nLhsIndex ] = mxCreateDoubleMatrix( SizeM, SizeN, mxREAL );
			return mxGetPr( m_gPlhs[ m_nLhsIndex++ ] );
		}
		else
			return 0;
	}

	void IncErrorCounter( enum err Error )
	{
		if ( Error >= div_by_zero && Error <= user_error )
			++m_pNERR[ Error ];
	}

	void ListExceptions( void );

	void GetExceptions( double *pr );

};

#endif
