#include "CCompGraph.h"


#include "miller.h"
#include "CGlobals.h"

CGraphManager &CGraphManager::operator=( const CCompGraph &OthersGraph )
	{
		if ( &OthersGraph.GetManager() == this )
			return *this;

		delete [] m_pCONS;
		delete [] m_pNLOP;
		delete [] m_pNOPER;
		delete [] m_pNROP;
		delete [] m_pNDXOUT;

		delete [] m_pOptArray;

		selfinit( OthersGraph );
		return *this;
	}

void CGraphManager::selfinit( const CGraphManager &other )
{
		long i;
		long size;
		long maxnop;
		long maxnfix;

		m_pDATA = other.m_pDATA;
		m_nDATSIZE = other.m_nDATSIZE;
		maxnop = other.m_nNOP[0] > other.m_nNOP[1] ? other.m_nNOP[0] : other.m_nNOP[1];
		maxnfix = other.m_nNFIX[0] > other.m_nNFIX[1] ? other.m_nNFIX[0] : other.m_nNFIX[1];

		nullinit();
		
		size = ( other.m_nNumOfAlgs == 1 ) ? m_nDATSIZE + maxnop : 2 * maxnop;

		m_pCONS = new double[ other.m_nNumOfAlgs * maxnfix ];

		if ( maxnop > 0L )
		{
			m_pNLOP = new long[ size ];
			m_pNOPER = new long[ size ];
			m_pNROP = new long[ size ];
		}
		if ( other.m_nNFIND > 0 )
			m_pNDXOUT = new long[ other.m_nNumOfAlgs * other.m_nNFIND ];

		m_nNumOfAlgs = other.m_nNumOfAlgs;
		m_nCSPONT = maxnop;
		m_nCONSIZE = maxnfix;
		m_nOUTSIZE = other.m_nNFIND;

		
		m_nNOP[0] = other.m_nNOP[0];
		m_nNOP[1] = other.m_nNOP[1];
		m_nNVARY = other.m_nNVARY;
		m_nNFIND = other.m_nNFIND;
		m_nNFIX[0] = other.m_nNFIX[0];
		m_nNFIX[1] = other.m_nNFIX[1];
		
		m_nNTOP = other.m_nNTOP;
		

		m_bVarGraph[0] = other.m_bVarGraph[0];
		m_bVarGraph[1] = other.m_bVarGraph[1];
		m_nState = other.m_nState;
		m_bValid = other.m_bValid;

		for ( i = 0; i < m_nNFIX[0]; ++i )
			m_pCONS[i] = other.m_pCONS[i];
		
		for ( i = 0; i < m_nOUTSIZE; ++i )
		{
			m_pNDXOUT[i]=other.m_pNDXOUT[i];
		}
		if ( m_nNumOfAlgs == 1 )
		{
			for ( i = m_nDATSIZE; i < size; ++i )
			{
				m_pNLOP[i] = other.m_pNLOP[i];
				m_pNOPER[i] = other.m_pNOPER[i];
				m_pNROP[i] = other.m_pNROP[i];
			}
		}
		else
		{
			long j = other.m_nCONSIZE;
			for ( i = m_nCONSIZE; i < m_nCONSIZE + m_nNFIX[1]; ++i )
			{
				m_pCONS[i] = other.m_pCONS[j];
				++j;
			}
			
			j = other.m_nOUTSIZE;
			for ( i = m_nOUTSIZE; i < 2 * m_nOUTSIZE; ++i )
			{			
				m_pNDXOUT[i] = other.m_pNDXOUT[j];
				++j;
			}

			for ( i = 0; i < m_nNOP[0]; ++i )
			{
				m_pNLOP[i] = other.m_pNLOP[i];
				m_pNOPER[i] = other.m_pNOPER[i];
				m_pNROP[i] = other.m_pNROP[i];
			}
			j = other.m_nCSPONT;
			for ( i = m_nCSPONT; i < m_nCSPONT + m_nNOP[1]; ++i )
			{
				m_pNLOP[i] = other.m_pNLOP[j];
				m_pNOPER[i] = other.m_pNOPER[j];
				m_pNROP[i] = other.m_pNROP[j];
				++j;
			}
		}

}

void CGraphManager::selfinit( const CCompGraph &OthersGraph )
{
	const CGraphManager &other = OthersGraph.GetManager();
	m_pDATA = other.m_pDATA;
	m_nDATSIZE = other.m_nDATSIZE;
	m_nNumOfAlgs = 1;
	nullinit();
	m_nState = 1;
	if ( OthersGraph.IsCounting() )
	{
		m_pCONS = new double[ m_nCONSIZE ];
		return;
	}
	OthersGraph.GetSizes( m_nNOP[0], m_nNVARY, m_nNFIND, m_nNFIX[0] );
	int AlgIndex = other.m_nState;
	CAssert::DEBUG_ERROR && CAssert::Assert( AlgIndex == 1 || AlgIndex == 2 );
	CAssert::DEBUG_ERROR && CAssert::Assert( AlgIndex == 1 || other.m_nNumOfAlgs == 2 );
	--AlgIndex;
	long i;
	long size;
	long maxnop;
	long maxnfix;
	maxnop = m_nNOP[ 0 ];
	maxnfix = m_nNFIX[ 0 ];
	size = m_nDATSIZE + maxnop;
	m_pCONS = new double[ maxnfix ];
	if ( maxnop > 0L )
	{
		m_pNLOP = new long[ size ];
		m_pNOPER = new long[ size ];
		m_pNROP = new long[ size ];
	}
	long nNFIND = m_nNFIND + 20L;
	m_pNDXOUT = new long[ nNFIND ];
	m_nCSPONT = maxnop;
	m_nCONSIZE = maxnfix;
	m_nOUTSIZE = nNFIND;
		
	m_bVarGraph[0] = other.m_bVarGraph[0];
	long j = AlgIndex ? other.m_nCONSIZE : 0L;

	for ( i = 0; i < maxnfix; ++i, ++j )
		m_pCONS[i] = other.m_pCONS[j];
		
	j = AlgIndex ? other.m_nOUTSIZE : 0L;
	for ( i = 0; i < m_nNFIND; ++i, ++j )
	{
		m_pNDXOUT[i] = other.m_pNDXOUT[j];
	}
	j = AlgIndex ? other.m_nCSPONT : other.m_nNumOfAlgs == 2 ? 0L : other.m_nDATSIZE;
	for ( i = m_nDATSIZE; i < size; ++i, ++j )
	{
		m_pNLOP[i] = other.m_pNLOP[j];
		m_pNOPER[i] = other.m_pNOPER[j];
		m_pNROP[i] = other.m_pNROP[j];
	}
}
/*
CGraphManager::CGraphManager( const CCompGraph &OthersGraph )
{
	const CGraphManager &other = OthersGraph.GetManager();
	m_pDATA = other.m_pDATA;
	m_nDATSIZE = other.m_nDATSIZE;
	m_nNumOfAlgs = 1;
	nullinit();
	if ( OthersGraph.IsCounting() )
	{
		m_pCONS = new double[ m_nCONSIZE ];
		return;
	}
	OthersGraph.GetSizes( m_nNOP[0], m_nNVARY, m_nNFIND, m_nNFIX[0] );
	int AlgIndex = other.m_nState;
	CAssert::DEBUG_ERROR && CAssert::Assert( AlgIndex == 1 || AlgIndex == 2 );
	CAssert::DEBUG_ERROR && CAssert::Assert( AlgIndex == 1 || other.m_nNumOfAlgs == 2 );
	--AlgIndex;
	long i;
	long size;
	long maxnop;
	long maxnfix;
	maxnop = m_nNOP[ 0 ];
	maxnfix = m_nNFIX[ 0 ];
	size = m_nDATSIZE + maxnop;
	m_pCONS = new double[ maxnfix ];
	if ( maxnop > 0L )
	{
		m_pNLOP = new long[ size ];
		m_pNOPER = new long[ size ];
		m_pNROP = new long[ size ];
	}
	long nNFIND = m_nNFIND + 20L;
	m_pNDXOUT = new long[ nNFIND ];
	m_nCSPONT = maxnop;
	m_nCONSIZE = maxnfix;
	m_nOUTSIZE = nNFIND;
		
	m_bVarGraph[0] = other.m_bVarGraph[0];
	m_nState = 1;
	long j = AlgIndex ? other.m_nCONSIZE : 0L;

	for ( i = 0; i < maxnfix; ++i, ++j )
		m_pCONS[i] = other.m_pCONS[j];
		
	j = AlgIndex ? other.m_nOUTSIZE : 0L;
	for ( i = 0; i < m_nNFIND; ++i, ++j )
	{
		m_pNDXOUT[i] = other.m_pNDXOUT[j];
	}
	j = AlgIndex ? other.m_nCSPONT : other.m_nNumOfAlgs == 2 ? 0L : other.m_nDATSIZE;
	for ( i = m_nDATSIZE; i < size; ++i, ++j )
	{
		m_pNLOP[i] = other.m_pNLOP[j];
		m_pNOPER[i] = other.m_pNOPER[j];
		m_pNROP[i] = other.m_pNROP[j];
	}


}
*/
/*
void CGraphManager::SetWithCurrent( const CGraphManager &other )
{
	int AlgIndex = other.m_nState;
	CAssert::DEBUG_ERROR && CAssert::Assert( AlgIndex == 1 || AlgIndex == 2 );
	CAssert::DEBUG_ERROR && CAssert::Assert( AlgIndex == 1 || other.m_nNumOfAlgs == 2 );
	--AlgIndex;
	long i;
	long size;
	long maxnop;
	long maxnfix;
	m_pDATA = other.m_pDATA;
	m_nDATSIZE = other.m_nDATSIZE;
	maxnop = other.m_nNOP[AlgIndex];
	maxnfix = other.m_nNFIX[AlgIndex];
	m_nNumOfAlgs = 1;
	nullinit();
	size = m_nDATSIZE + maxnop;
	m_pCONS = new double[ maxnfix ];
	if ( maxnop > 0L )
	{
		m_pNLOP = new long[ size ];
		m_pNOPER = new long[ size ];
		m_pNROP = new long[ size ];
	}
	long nNFIND = other.m_nNFIND + 20L;
	m_pNDXOUT = new long[ nNFIND ];
	m_nCSPONT = maxnop;
	m_nCONSIZE = maxnfix;
	m_nOUTSIZE = nNFIND;
	
	m_nNOP[0] = maxnop;
	m_nNVARY = other.m_nNVARY;
	m_nNFIND = other.m_nNFIND;
	m_nNFIX[0] = maxnfix;
		
	m_nNTOP = other.m_nNTOP;
	m_bVarGraph[0] = other.m_bVarGraph[0];
	m_nState = 1;
	long j = AlgIndex ? other.m_nCONSIZE : 0L;

	for ( i = 0; i < m_nNFIX[0]; ++i, ++j )
		m_pCONS[i] = other.m_pCONS[j];
		
	j = AlgIndex ? other.m_nOUTSIZE : 0L;
	for ( i = 0; i < m_nNFIND; ++i, ++j )
	{
		m_pNDXOUT[i] = other.m_pNDXOUT[j];
	}
	j = AlgIndex ? other.m_nCSPONT : other.m_nNumOfAlgs == 2 ? 0L : other.m_nDATSIZE;
	for ( i = m_nDATSIZE; i < size; ++i, ++j )
	{
		m_pNLOP[i] = other.m_pNLOP[j];
		m_pNOPER[i] = other.m_pNOPER[j];
		m_pNROP[i] = other.m_pNROP[j];
	}

}
*/
void CGraphManager::InitSpace( void )
		{
			
			long size;
			int state;
			long maxnop;
			maxnop = m_nNOP[0] > m_nNOP[1] ? m_nNOP[0] : m_nNOP[1];
			state = m_nState;
			m_nState = 0;

			size = ( m_nNumOfAlgs == 1 ) ? m_nDATSIZE + maxnop : 2 * maxnop;

			delete [] m_pNLOP;
			delete [] m_pNOPER;
			delete [] m_pNROP;
			delete [] m_pNDXOUT;
			m_pNLOP = m_pNOPER = m_pNROP = m_pNDXOUT = 0;
			m_pNLOP = new long[ size ];
			m_pNOPER = new long[ size ];
			m_pNROP = new long[ size ];
			m_pNDXOUT = new long[ m_nNumOfAlgs * m_nNFIND ];
			m_nCSPONT = maxnop;
			m_nOUTSIZE = m_nNFIND;
			m_nState = state;
			

		}


void CGraphManager::duplicate( long &consize, double *&pCons, long nFix )
	{
		
		if ( m_nState > 0 )
		{
			long i;
			long OldConsize = m_nCONSIZE;
			double *OldpCons = m_pCONS;
			int state = m_nState;
			m_nState = 0;
			consize *= 2L;
			if ( !consize )
				consize = CONSIZE;
			m_pCONS = new double[ m_nNumOfAlgs * consize ];
			m_nCONSIZE = consize;
			m_nState = state;
			if ( m_nNumOfAlgs == 1 )
			{
				for ( i = 0; i < nFix; ++i )
					m_pCONS[i] = OldpCons[i];
			}
			else
			{
				long nfix1;
				long nfix2;
				long j;
				if ( m_nState == 1 )
				{
					nfix1 = nFix;
					nfix2 = m_nNFIX[1];
				}
				else
				{
					nfix1 = m_nNFIX[0];
					nfix2 = nFix;
				}
				for ( i = 0; i < nfix1; ++i )
					m_pCONS[i] = OldpCons[i];
				j = OldConsize;
				for ( i = m_nCONSIZE; i < m_nCONSIZE + nfix2; ++i )
				{
					m_pCONS[i] = OldpCons[j];
					++j;
				}
			}
			delete [] OldpCons;

			pCons = ( m_nState == 1 ) ? m_pCONS : m_pCONS + m_nCONSIZE;
			
		}
	}

void CGraphManager::duplicate( long &outsize, long *&pNdxout, long nFind )
	{
		
		if ( m_nState > 0 )
		{
			long i;
			int state = m_nState;
			long OldOutsize = m_nOUTSIZE;
			long *OldpNdxout = m_pNDXOUT;
			long size;
			m_nState = 0;
			outsize *= 2L;
			if ( !outsize )
				outsize = 1L;
			m_pNDXOUT = new long[ m_nNumOfAlgs * outsize ];
			m_nOUTSIZE = outsize;
			m_nState = state;

			size = ( state == 1 ) ? nFind : m_nNFIND;
			for ( i = 0; i < size; ++i )
				m_pNDXOUT[i] = OldpNdxout[i];
			if ( m_nNumOfAlgs == 2 )
			{
				long j = OldOutsize;
				size = ( state == 2 ) ? nFind : m_nNFIND;

				for ( i = m_nOUTSIZE; i < m_nOUTSIZE + size; ++i )
				{
					m_pNDXOUT[i] = OldpNdxout[j++];
				}
			}

			delete [] OldpNdxout;

			pNdxout = ( m_nState == 1 ) ? m_pNDXOUT : m_pNDXOUT + m_nOUTSIZE;
		}
	}

void CGraphManager::duplicate( long &cspont, long *&pNlop, long *&pNoper, long *&pNrop, long nNop )
	{
		if ( m_nState > 0 )
		{
			long i;
			long j;

			long size;
			long *OldLop = m_pNLOP;
			long *OldOper = m_pNOPER;
			long *OldRop = m_pNROP;
			int state = m_nState;
			long OldCspont = m_nCSPONT;

			m_nState = 0;
			cspont *= 2L;
			if ( !cspont )
				cspont = 1L;

			size = ( m_nNumOfAlgs == 1 ) ? m_nDATSIZE + cspont : 2L * cspont;

			m_pNLOP = new long[ size ];
			m_pNOPER = new long[ size ];
			m_pNROP = new long[ size ];
			
			m_nCSPONT = cspont;
			m_nState = state;

			size = ( m_nState == 1 ) ? nNop : m_nNOP[0];
			j = ( m_nNumOfAlgs == 1 ) ? m_nDATSIZE : 0L;
			for ( i = j; i < j + size; ++i )
			{
				m_pNLOP[ i ] = OldLop[i];
				m_pNOPER[ i ] = OldOper[i];
				m_pNROP[ i ] = OldRop[i];
			}

			if ( m_nNumOfAlgs == 2 )
			{
				size = ( m_nState == 2 ) ? nNop : m_nNOP[1];
				j = OldCspont;
				for ( i = m_nCSPONT; i < m_nCSPONT + size; ++i )
				{
					m_pNLOP[ i ] = OldLop[j];
					m_pNOPER[ i ] = OldOper[j];
					m_pNROP[ i ] = OldRop[j];
					++j;
				}
			}


			delete [] OldLop;
			delete [] OldOper;
			delete [] OldRop;
            
			if ( m_nNumOfAlgs == 1 )
			{
				size = m_nDATSIZE;
			}
			else if ( m_nState == 1 )
			{
				size = 0L;
			}
			else
			{
				size = m_nCSPONT;
			}
			pNlop = m_pNLOP + size;
			pNoper = m_pNOPER + size;
			pNrop = m_pNROP + size;
		


		
		}
	}

			



void CGraphManager::UpdateSizes( CCompGraph &graph )
{
	long nfind;

	if ( m_nState == 1 )
	{
		graph.GetSizes( m_nNOP[0], m_nNVARY, nfind, m_nNFIX[0] );
		if ( m_nNumOfAlgs == 1 )
			m_nNFIND = nfind;
		
		if ( m_nNVARY != m_nDATSIZE )
		{
			if ( m_nNumOfAlgs == 1 )
				throw CException( input_too_large );
			else
				throw CException( input_too_large_1 );
		}
		if ( nfind == 0 )
		{
			if ( m_nNumOfAlgs == 1 )
				throw CException( no_output );
			else
				throw CException( no_output_1 );
		}
		if ( m_nNumOfAlgs == 2 && !m_bVarGraph[1] && nfind != m_nNFIND )
		{
			m_bVarGraph[1] = true;
			throw CException( diff_output_num );
		}
		else
		{
			m_nNFIND = nfind;
		}

		if ( m_nNOP[0] == 0  )
		{
			if ( m_nNumOfAlgs == 1 )
				throw CException( no_operation );
			else
				throw CException( no_operation_1 );
		}
	}
	else if ( m_nState == 2 )
	{

		CAssert::EQ( m_nNumOfAlgs, 2 );

		graph.GetSizes( m_nNOP[1], m_nNVARY, nfind, m_nNFIX[1] );
		
		if ( m_nNVARY != m_nDATSIZE )
		{
			throw CException( input_too_large_2 );
		}
		if ( nfind == 0 )
		{
			throw CException( no_output_2 );
		}

		if ( nfind != m_nNFIND )
		{
			m_bVarGraph[0] = true;
			throw CException( diff_output_num );
		}
		if ( m_nNOP[1] == 0  )
		{
			throw CException( no_operation_2 );
		}
	}
}

bool CGraphManager::FillBegin( double *input, CGlobals &globals )
{
	CAssert::Null( m_nState );
	CAssert::EQ( m_nNumOfAlgs, 1 );
	m_pDATA = input;
	if ( m_bVarGraph[0] )
	{
		m_bValid = false;
		globals.InitConstraints();
		CCompGraph *pTmpGraph = new CCompGraph( this );
		globals.SetGraph( pTmpGraph );
		m_nNTOP = 0L;
		m_nState = 1;
		pTmpGraph->reset( false );
		return true;
	}
	return false;
}

class CGraphManager::CLEARER
{
private:
	CGlobals &m_globals;
	int &m_state;
public:
	CLEARER( CGlobals &_globals, int &state ) : m_globals( _globals ), m_state( state )
	{
	}

	~CLEARER( void )
	{
		m_globals.ClearGraph();
		m_state = 0;
		
	}
};

void CGraphManager::FillEnd( CGlobals &globals )
{
	CAssert::EQ( m_nNumOfAlgs, 1 );
	if ( m_bVarGraph[0] )
	{
		CAssert::EQ( m_nState, 1 );
		CLEARER clearer( globals, m_nState );
		globals.ThrowException();
		CCompGraph &graph = globals.GetGraph();
		if ( m_nCSPONT != 0 )
		{
			graph.optimize( m_nNumOfAlgs );
			m_nNTOP = graph.ComputeNtop();
		}
		UpdateSizes( graph );
		m_bVarGraph[0] = graph.IsVar();
        if ( m_nCSPONT != 0 )
			m_bValid = true;
//		CAssert::NotNull( m_nCSPONT );
	}
	else
	{
		CAssert::Null( m_nState );
	}

}

void CGraphManager::CreateGraph( double *input, CGlobals &globals )
	{
        m_pDATA = input;
		if ( m_bVarGraph[0] || m_bVarGraph[1] )
		{
			m_bValid = false;
//			try
			{
				globals.InitConstraints();
				CCompGraph *pTmpGraph = new CCompGraph( this );
				CCompGraph &graph = *pTmpGraph;
				CLEARER clearer( globals, m_nState );
				globals.SetGraph( pTmpGraph );
				m_nNTOP = 0L;
				if ( m_bVarGraph[0] )
				{
					m_nState = 1;
					graph.reset();
					RunAlgorithm( 1, globals );
					if ( m_nCSPONT != 0 )
					{
						graph.optimize( m_nNumOfAlgs );
						m_nNTOP = graph.ComputeNtop();
					}
/*
					graph.GetSizes( m_nNOP[0], m_nNVARY, m_nNFIND, m_nNFIX[0] );
					
					if ( m_nNVARY != m_nDATSIZE )
					{
						if ( m_nNumOfAlgs == 1 )
							throw CException( input_too_large );
						else
							throw CException( input_too_large_1 );
					}
					if ( m_nNFIND == 0 )
					{
						if ( m_nNumOfAlgs == 1 )
							throw CException( no_output );
						else
							throw CException( no_output_1 );
					}
					if ( m_nNOP[0] == 0  )
					{
						if ( m_nNumOfAlgs == 1 )
							throw CException( no_operation );
						else
							throw CException( no_operation_1 );
					}
*/
					UpdateSizes( graph );
					m_bVarGraph[0] = graph.IsVar();
					
				}
				if ( m_bVarGraph[1] )
				{
//					long nvary2;
//					long nfind2;
					m_nState = 2;
					graph.reset();
					RunAlgorithm( 2, globals );
					if ( m_nCSPONT != 0 )
					{
						if ( m_nNTOP != 0 )
							throw CException( comp_error_free );
						graph.optimize( m_nNumOfAlgs );
						if ( graph.ComputeNtop() != 0 )
							throw CException( comp_error_free );
					}
/*
					graph.GetSizes( m_nNOP[1], nvary2, nfind2, m_nNFIX[1] );
					
					if ( nvary2 != m_nDATSIZE )
					{
						throw CException( input_too_large_2 );
					}
					if ( nfind2 == 0 )
					{
						throw CException( no_output_2 );
					}
	
					if ( nfind2 != m_nNFIND )
					{
						m_bVarGraph[0] = true;
						throw CException( diff_output_num );
					}
					if ( m_nNOP[1] == 0  )
					{
						throw CException( no_operation_2 );
					}

*/
					UpdateSizes( graph );
					m_bVarGraph[1] = graph.IsVar();

				}
				if ( m_nCSPONT == 0 )
				{
					reset();
					m_nState = 1;
					InitSpace();
					graph.reset();
					globals.InitConstraints();
					RunAlgorithm( 1, globals );
					graph.optimize( m_nNumOfAlgs );
//					graph.GenCodFile( "egyeske" );
					m_nNTOP = graph.ComputeNtop();
//					graph.GetSizes( m_nNOP[0], m_nNVARY, m_nNFIND, m_nNFIX[0] );
					UpdateSizes( graph );
					m_bVarGraph[0] = graph.IsVar();
					if ( m_nNumOfAlgs == 2 )
					{
						if ( m_nNTOP != 0 )
						{
							m_bVarGraph[0] = true;
							throw CException( comp_error_free );
						}
						m_nState = 2;
						graph.reset();
						RunAlgorithm( 2, globals );
						graph.optimize( m_nNumOfAlgs );
						if ( graph.ComputeNtop() != 0 )
						{
							throw CException( comp_error_free );
						}

//						graph.GenCodFile("ketteske" );
//						graph.GetSizes( m_nNOP[1], m_nNVARY, m_nNFIND, m_nNFIX[1] );
						UpdateSizes( graph );
						m_bVarGraph[1] = graph.IsVar();						
					}
				}
			}
//			catch ( CException )
//			{
//				m_nState = 0;
//				throw;
//			}
//			m_nState = 0;
			m_bValid = true;
		}

	}

void CGraphManager::RunAlgorithm( int AlgIndex, CGlobals &globals )
{
	CallAlgorithm( AlgIndex, globals );
}


void CGraphManager::SetData( double *input, long nDatSize, bool ParChanged )
{
	m_pDATA = input;
	
	if ( m_nDATSIZE != nDatSize )
	{
		reset();
		
		if ( m_nNumOfAlgs == 1 && m_nCSPONT != 0 )
		{
			long diff = nDatSize - m_nDATSIZE;
			if ( diff >= m_nCSPONT )
			{
				long consize = m_nCONSIZE;
				delete [] m_pCONS;
				delete [] m_pNLOP;
				delete [] m_pNOPER;
				delete [] m_pNROP;
				delete [] m_pNDXOUT;
				delete [] m_pOptArray;
				nullinit();
				m_nCONSIZE = consize;
				m_pCONS = new double[ m_nCONSIZE ];
			}
			else
			{
				m_nCSPONT -= nDatSize - m_nDATSIZE;
			}
		}
			

		m_nDATSIZE = nDatSize;
	}
	else if ( ParChanged )
	{
		reset();
	}
}

