#ifndef _CGRAPHMAN_
#define _CGRAPHMAN_


#include "CException.h"

class CCompGraph;
class CGraphManager
{
private:

		static const int CONSIZE = 32L;
		class CLEARER;

     	long m_nCSPONT;
		long m_nCONSIZE;
		long m_nDATSIZE;
		long m_nOUTSIZE;

        double *m_pDATA;
		
		double *m_pCONS;
        long *m_pNLOP;
        long *m_pNOPER;
        long *m_pNROP;
        long *m_pNDXOUT;

        long m_nNOP[2];
        long m_nNVARY;
        long m_nNFIND;
		long m_nNFIX[2];

        long m_nNTOP;

		bool m_bVarGraph[2];

		int m_nState;

		int m_nNumOfAlgs;

		long *m_pOptArray;
		long m_nOptArraySize;

		bool m_bValid;


		void InitSpace( void );

		void selfinit( const CGraphManager &other );

		void selfinit( const CCompGraph &OthersGraph );



		void nullinit( void )
		{
			m_nCSPONT = m_nOUTSIZE = 0L;
			m_nNOP[0] = m_nNOP[1] = m_nNFIX[0] = m_nNFIX[1] = 0L;
			m_nNVARY = m_nNFIND = m_nNTOP = 0L;
            m_nOptArraySize = 0L;
			
			m_pOptArray = m_pNLOP = m_pNOPER = m_pNROP = m_pNDXOUT = 0;
			m_bVarGraph[0] = true;
			m_bVarGraph[1] = ( m_nNumOfAlgs == 2 ) ? true : false;
			m_nState = 0;

			m_nCONSIZE = CONSIZE;
			m_pCONS = 0;

            m_bValid = false;			
		}

		void UpdateSizes( CCompGraph &graph );

protected:
	virtual void RunAlgorithm( int AlgIndex, CGlobals &globals );
	

public:
	CGraphManager( double *pData, long Datsize, int algs )
	{
		m_pDATA = pData;
		m_nDATSIZE = Datsize;
		m_nNumOfAlgs = algs;
		nullinit();
		m_pCONS = new double[ m_nNumOfAlgs * m_nCONSIZE ];
	}

	virtual ~CGraphManager( void )
	{
		delete [] m_pCONS;
		delete [] m_pNLOP;
		delete [] m_pNOPER;
		delete [] m_pNROP;
		delete [] m_pNDXOUT;
		delete [] m_pOptArray;
	}

	void CheckValidity( void )
	{
		if ( !m_bValid )
			throw CException( "Computational graph is not complete!" );
	}


	void SetData( double *input, long nDatSize, bool ParChanged );

	void reset( void )
	{
			m_bVarGraph[0] = true;
			m_bVarGraph[1] = ( m_nNumOfAlgs == 2 ) ? true : false;
			m_nState = 0;
			m_bValid = false;
	}


/*
	void reset( NumAlg *algorithm, double *pData, long datsize )
	{
		delete [] m_pCONS;
		delete [] m_pNLOP;
		delete [] m_pNOPER;
		delete [] m_pNROP;
		delete [] m_pNDXOUT;
		nullinit();
		m_pAlgorithm = algorithm;
		m_pDATA = pData;
		m_nDATSIZE = datsize;
	}
*/

	CGraphManager( const CGraphManager &other ) 
	{

		selfinit( other );

	}
	
	explicit CGraphManager( const CCompGraph &OthersGraph )
	{
		selfinit( OthersGraph );
	}

	CGraphManager &operator=( const CCompGraph &OthersGraph );

	inline CGraphManager &operator=( const CGraphManager &other )
	{
		if ( &other == this )
			return *this;

		delete [] m_pCONS;
		delete [] m_pNLOP;
		delete [] m_pNOPER;
		delete [] m_pNROP;
		delete [] m_pNDXOUT;

		delete [] m_pOptArray;

		selfinit( other );
		return *this;
	}




	void duplicate( long &consize, double *&pCons, long nFix );
	void duplicate( long &outsize, long *&pNdxout, long nFind );
	void duplicate( long &cspont, long *&pNlop, long *&pNoper, long *&m_pNrop, long nNop );


	bool FillBegin( double *input, CGlobals &globals );
	void FillEnd( CGlobals &globals );

	void CreateGraph( double *input, CGlobals &globals );

	void GetLimits( long &cspont, long &datsize, long &consize, long &outsize ) const
	{
		cspont = m_nCSPONT;
		datsize = m_nDATSIZE;
		consize = m_nCONSIZE;
		outsize = m_nOUTSIZE;
	}

	void GetOperations( long *&lop, long *&oper, long *&rop ) const
	{
		long offset = 0L;
		if ( m_nState > 0 && m_nCSPONT != 0 )
		{
			if ( m_nNumOfAlgs == 1 )
				offset = m_nDATSIZE;
			else if ( m_nState == 2 )
				offset = m_nCSPONT;
		}
		lop = m_pNLOP + offset;
		oper = m_pNOPER + offset;
		rop = m_pNROP + offset;
	}

	void GetPointers( long *&ndxout, double *&cons, double *&data ) const
	{
		data = m_pDATA;
		ndxout = m_pNDXOUT;
		if ( m_nState == 2 )
		{
			cons = m_pCONS + m_nCONSIZE;
			if ( m_nCSPONT != 0 )
				ndxout = m_pNDXOUT + m_nOUTSIZE;
		}
		else
		{
			cons = m_pCONS;
		}
		
		
	}

	void GetSizes( long &nop, long &nvary, long &nfind, long &nfix ) const
	{
		nop = m_nNOP[0];
		nvary = m_nNVARY;
		nfind = m_nNFIND;
		nfix = m_nNFIX[0];
	}

	void GetScndSizes( long &nop, long &nfix ) const
	{
		nop = m_nNOP[1];
		nfix = m_nNFIX[1];
	}

	long GetNtop( void ) const
	{
		return m_nNTOP;
	}

	int GetNumOfAlgs( void )
	{
		return m_nNumOfAlgs;
	}

	void GetGraph( long *SizeArray, double *&cons, long *&left, long *&oper, long *&right, long *&output, int alg )
	{

		cons = m_pCONS;
		left = m_pNLOP;
		oper = m_pNOPER;
		right = m_pNROP;
		output = m_pNDXOUT;
		SizeArray[0] = m_nNFIX[0];
		SizeArray[1] = m_nNOP[0];
		SizeArray[2] = m_nNFIND;

		if ( m_nCSPONT != 0 )
		{
			if ( m_nNumOfAlgs == 1  )
			{
				left += m_nDATSIZE;
				oper += m_nDATSIZE;
				right += m_nDATSIZE;
			}
			else if ( alg == 2 )
			{
				cons += m_nCONSIZE;
				left += m_nCSPONT;
				oper += m_nCSPONT;
				right += m_nCSPONT;
				output += m_nOUTSIZE;
				SizeArray[0] = m_nNFIX[1];
				SizeArray[1] = m_nNOP[1];
			}
				
		}
	}

	long GetMemAlloc( void ) const
	{
		if ( m_nNumOfAlgs == 1 )
			return m_nCONSIZE * sizeof( double ) 
			+ ( 3L * ( m_nCSPONT + m_nDATSIZE ) + m_nOUTSIZE ) * sizeof( long );
		else
			return 2L * m_nCONSIZE * sizeof( double ) 
			+ ( 6L * m_nCSPONT + 2L * m_nOUTSIZE ) * sizeof( long );
	}

	long GetMemUsed( void ) const
	{
		if ( m_nNumOfAlgs == 1 )
			return m_nNFIX[0] * sizeof( double ) 
			+ ( 3L * ( m_nNOP[0] + m_nNVARY ) + m_nNFIND ) * sizeof( long );
		else
			return ( m_nNFIX[0] + m_nNFIX[1] ) * sizeof( double )
			+ ( 3L * ( m_nNOP[0] + m_nNOP[1] ) + 2L * m_nNFIND ) * sizeof( long );
	}

	long GetDatSize( void ) const
	{
		return m_nDATSIZE;
	}

	long *GetOptArray( long size )
	{
		if ( size > m_nOptArraySize )
		{
			delete [] m_pOptArray;
			m_pOptArray = 0;
			m_nOptArraySize = 0L;
			m_pOptArray = new long[ size ];
			m_nOptArraySize = size;
		}

		return m_pOptArray;
	}

};



#endif