#ifndef _CMAXIM_
#define _CMAXIM_

#include "CGlobals.h"

extern "C" void ROSEN_FOR( double *D, long *NVARY, double *P, double *DEL, 
                           double *DOLD, double *DSAVE, double *BIGDEL, CGlobals *globals );

extern "C" void MDS_FOR( double *D, long *NVARY, double *V, double *T, CGlobals *globals );

extern "C" void NMS_FOR(double *D, long *NVARY, double *FCUR, double *V, double *VBAR, long *PTR, CGlobals *globals );


class CMaxim
{
private:

	double *m_pArray;
	long *m_pLongArray;
	

	void create_rosen( long size, double *&P, double *&DEL, double *&DOLD,
					double *&DSAVE, double *&BIGDEL )
	{
		delete [] m_pArray;
		P = m_pArray = new double[ ( size + 4 ) * size ];
		DEL = P + ( size * size );
		DOLD = DEL + size;
		DSAVE = DOLD + size;
		BIGDEL = DSAVE + size;
	}

	void create_mds( long size, double *&V, double *&T )
	{
		long sizep = size * ( size + 1 );
        delete [] m_pArray;
		T = ( V = m_pArray = new double[ sizep * 2 ] ) + sizep;
	}

	void create_nms( long size, double *&FCUR, double *&V, double *&VBAR, long *&PTR )
	{
		VBAR = m_pArray = new double[ size * size + 5 * size + 3 ];
		FCUR = VBAR + size;
		V = FCUR + size + 3;
		PTR = new long[ size + 1 ];
	}


public:
	CMaxim( void )
	{
		m_pArray = 0;
		m_pLongArray = 0;
	}

	~CMaxim( void )
	{
		delete [] m_pArray;
		delete [] m_pLongArray;
	}



	void rosen( double *D, long *NVARY, CGlobals *globals )
	{
        
        double *P;
        double *DEL;
        double *DOLD;
        double *DSAVE;
        double *BIGDEL;

        create_rosen( *NVARY, P, DEL, DOLD, DSAVE, BIGDEL );
        ROSEN_FOR( D, NVARY, P, DEL, DOLD, DSAVE, BIGDEL, globals );
		globals->GetTheEngine()->GetInput( D );
        delete [] m_pArray;
		m_pArray = 0;

	}

	void mds(double *D, long *NVARY, CGlobals *globals)
	{
        double *V;
        double *T;
        create_mds( *NVARY, V, T );
        MDS_FOR(D, NVARY, V, T, globals);
		globals->GetTheEngine()->GetInput( D );
		delete [] m_pArray;
		m_pArray = 0;
	}

	void nms(double *D, long *NVARY, CGlobals *globals)
	{
        double *FCUR;
        double *V;
        double *VBAR;
        long *PTR;

        create_nms( *NVARY, FCUR, V, VBAR, PTR );
        NMS_FOR(D, NVARY, FCUR, V, VBAR, PTR, globals );
        globals->GetTheEngine()->GetInput( D );
		delete [] m_pArray;
		m_pArray = 0;
		delete [] m_pLongArray;
		m_pLongArray = 0;

	}

};

#endif