#include "CRound.h"
#include "miller.h"
#include "CGlobals.h"


void CRound::SelfInit( const CRound &other )
{
				long nodesize;
				long findsize;
				long datsize;
				int NumOfAlgs;
				long i;
			
				nullinit();
				NumOfAlgs = m_TheGraph.GetNumOfAlgs();
				m_TheGraph.GetSizes( nodesize, datsize, findsize, i );
				if ( NumOfAlgs == 2 )
				{
					long tmp;
					m_TheGraph.GetScndSizes( tmp, i );
					if ( tmp > nodesize )
						nodesize = tmp;
				}
				else
				{
					long tmp;
					nodesize+=datsize;
					if ( ( tmp = datsize + 2 * findsize ) > nodesize )
					{
						nodesize = tmp;
					}
				}
				if ( other.m_nNodeSize < nodesize )
					nodesize = other.m_nNodeSize;
				if ( other.m_nFindSize < findsize )
					findsize = other.m_nFindSize;
				Syncronize( nodesize, findsize );
                
                for( i = 0; i < NumOfAlgs * nodesize; ++i )
                {
                        m_pVALUE[i] = other.m_pVALUE[i];
                        m_pWGTLFT[i] = other.m_pWGTLFT[i];
                        m_pWGTRGHT[i] = other.m_pWGTRGHT[i];
                }
                for( i = 0; i < findsize; ++i )
                        m_pNDCOL[i] = other.m_pNDCOL[i];
                for( i = 0; i < NumOfAlgs * findsize * nodesize; ++i )
                        m_pDERIVS[i] = other.m_pDERIVS[i];

                m_dRho = other.m_dRho;
				

}

void CRound::Syncronize( long nop, long nfind )
{
	bool grow = false;
	int NumOfAlgs = m_TheGraph.GetNumOfAlgs();

    if ( nop > m_nNodeSize )
	{
		grow = true;
        
        delete [] m_pVALUE;
        delete [] m_pWGTLFT;
        delete [] m_pWGTRGHT;
		m_pVALUE = m_pWGTLFT = m_pWGTRGHT = 0;
               
        m_pVALUE = new double[ NumOfAlgs * nop ];
        m_pWGTLFT = new double[ NumOfAlgs * nop ];
        m_pWGTRGHT = new double[ NumOfAlgs * nop ];
		

	}
    if ( nfind > m_nFindSize )
    {
		grow = true;
        
        delete [] m_pNDCOL;
		m_pNDCOL = 0;
        
		m_pNDCOL = new long[ nfind ];
		
	}

    if ( grow )
    {
		delete [] m_pDERIVS;
		m_pDERIVS = 0;
        m_pDERIVS = new double[ NumOfAlgs * nop * nfind ];
		m_nNodeSize = nop;
		m_nFindSize = nfind;
	}
}

bool CRound::FillBegin( double *input, CGlobals &globals )
{
	CRoundBase::FillBegin( input, globals );
	return m_TheGraph.FillBegin( input, globals );
}

void CRound::FillEnd( CGlobals &globals )
{
	m_TheGraph.FillEnd( globals );
}

double CRound::SpecRound( double *input, CGlobals &globals )
{
	m_TheGraph.CreateGraph( input, globals );
	return ComputeRho( globals );

}

double CRound::ComputeRho( double &penalty, CGlobals &globals )
{
	m_TheGraph.CheckValidity();
	ComputeRho( globals );
	return CRoundBase::ComputeRho( penalty, globals );
}


double CRound::ComputeRho( CGlobals &globals )
{
	double *pCONS;
	double *pDATA;
	long nNOP[4];
	long *pNLOP;
	long *pNOPER;
	long *pNROP;
	long *pNDXOUT;
	long nNVARY;
	long nNFIND;
	long nNTOP;
	long nNERR;

	long nCSPONT;
	long nCONSIZE;
	long nOUTSIZE;

	long i, j, k;
	long nNPICK = GetEcw();
	int NumOfAlgs = m_TheGraph.GetNumOfAlgs();

	double *A = 0;
	double *B = 0;
	long ToCopy;

	m_dRho = 0.0;


//	m_TheGraph.CreateGraph( input, globals );

	m_TheGraph.GetSizes( nNOP[0], nNVARY, nNFIND, i );
	CAssert::NotNull( nNOP[0] );
	CAssert::NotNull( nNVARY );
	CAssert::NotNull( nNFIND );
	if ( NumOfAlgs == 2 )
	{
		m_TheGraph.GetScndSizes( nNOP[1], i );
		CAssert::NotNull( nNOP[1] );
		if ( nNPICK <= 1L && nNPICK >= -1L )
		{
			ToCopy = ( A = globals.GetArrayLhs( nNFIND, nNOP[0] ) ) && 
				( B = globals.GetArrayLhs( nNFIND, nNOP[1] ) );
		}
		else
		{
			ToCopy = ( B = globals.GetArrayLhs( nNFIND, nNOP[1] ) ) && 
				( A = globals.GetArrayLhs( nNFIND, nNOP[0] ) );
		}


		Syncronize( nNOP[0] > nNOP[1] ? nNOP[0] : nNOP[1], nNFIND );
		nNOP[2] = m_nNodeSize;
		nNOP[3] = m_nFindSize;
	}
	else
	{					
		long tmp;
		long SizeB;
		if ( nNPICK == 2L || nNPICK == 4L )
			SizeB = nNVARY + nNFIND;
		else
			SizeB = nNVARY;
		if ( nNPICK >= 6L )
		{
			ToCopy = ( B = globals.GetArrayLhs( nNFIND, SizeB ) ) && 
				( A = globals.GetArrayLhs( nNFIND, nNFIND  ) );
		}
		else
		{
			ToCopy = ( A = globals.GetArrayLhs( nNFIND, nNOP[0] ) ) && 
				( B = globals.GetArrayLhs( nNFIND, SizeB  ) );
		}

		nNOP[0] += nNVARY;
 		if ( ( tmp = nNVARY + 2 * nNFIND ) < nNOP[0] )
		{
			tmp = nNOP[0];
		}

		Syncronize( tmp, nNFIND );
		nNOP[1] = m_nNodeSize;
	}
	
	
	m_TheGraph.GetOperations( pNLOP, pNOPER, pNROP );
	m_TheGraph.GetPointers( pNDXOUT, pCONS, pDATA );
	
	if ( NumOfAlgs == 1 )
	{
		nNTOP = m_TheGraph.GetNtop();
		for ( i = 0; i < nNVARY; ++i )
			m_pVALUE[i] = pDATA[i];
		ROUND1_FOR( m_pVALUE, m_pDERIVS, pCONS, &m_dRho, nNOP, pNLOP, pNOPER, pNROP, pNDXOUT,
             &nNVARY, &nNFIND, &nNPICK, &nNTOP, &nNERR, m_pWGTLFT, m_pWGTRGHT, m_pNDCOL,
			 &ToCopy, A, B);
		if ( ToCopy && nNPICK != 0L )
		{
			long limit;
			bool bLE = nNPICK == -3 || nNPICK == 3 || nNPICK == 4 || nNPICK == 7;
			bool bWK = nNPICK == 2 || nNPICK == 4;
			bool bCN = nNPICK == 6 || nNPICK == 7;
			limit = nNVARY * nNFIND;
			if ( bLE )
			{
				double dDatNorm = 0.0;
				double tmp;
				
				for ( i = 0; i < nNVARY; ++i )
				{
					tmp = fabs( pDATA[i] );
					if ( tmp > dDatNorm )
						dDatNorm = tmp;
				}
				
				for ( i = 0; i < limit; ++i )
				{
					B[i] *= dDatNorm;
				}
				if ( bWK || bCN )
				{
					double *C;
					double dValNorm = 0.0;
					for ( i = 0; i < nNFIND; ++i )
					{
						tmp = fabs( m_pVALUE[ pNDXOUT[i] - 1 ] );
						if ( tmp > dValNorm )
							dValNorm = tmp;
					}
					if ( bWK )
					{
						C = B + limit;
					}
					else
					{
						C = A;
					}
					k = 0L;
					for ( j = 0L; j < nNFIND; ++j )
					{
						for ( i = 0L; i < j; ++i )
						{
							C[ k++ ] = 0.0;
						}
						C[ k++ ] = dValNorm;
						++i;
						for ( ; i < nNFIND; ++i )
						{
							C[ k++ ] = 0.0;
						}
					}
				}
			}
			else //!bLE
			{
			
			
				
				for ( j = 0; j < nNVARY; ++j )
				{
					for ( i = 0; i < nNFIND; ++i )
					{
						B[ nNFIND * j + i ] *= pDATA[j];
					}
				}
				if ( bWK || bCN )
				{
					double *C;
					if ( bWK )
					{
						C = B + limit;
					}
					else
					{
						C = A;
					}
					k = 0L;
					for ( j = 0L; j < nNFIND; ++j )
					{
						for ( i = 0L; i < j; ++i )
						{
							C[ k++ ] = 0.0;
						}
						C[ k++ ] = m_pVALUE[ pNDXOUT[i] - 1 ];
						++i;
						for ( ; i < nNFIND; ++i )
						{
							C[ k++ ] = 0.0;
						}
					}
				}
			}
		}
	}
	else
	{
		long limits[3];
		m_TheGraph.GetLimits( nCSPONT, i, nCONSIZE, nOUTSIZE );
		limits[0] = nCSPONT;
		limits[1] = nCONSIZE;
		limits[2] = nOUTSIZE;
		ROUND2_FOR( m_pVALUE, m_pDERIVS, pDATA, pCONS, &m_dRho, nNOP, pNLOP, pNOPER, pNROP,
			pNDXOUT, &nNVARY, &nNFIND, &nNPICK, &nNERR, m_pWGTLFT, m_pWGTRGHT, m_pNDCOL, 
			limits, &ToCopy, A, B);
	}
	
	if ( nNERR != 0 )
		throw CException( (enum err)(nNERR - 1L)  );
/*	if ( m_nNUMBER < 0 )
		positv_c( m_pDERIVS, m_pVALUE, m_pVALUE + nNVARY, &m_nNUMBER );
	positv_c( m_Constr.GetPtr( m_nNUMBER ), m_pVALUE, m_pVALUE + nNVARY, &m_nNUMBER );
*/
	


	
	return m_dRho;
    
}

/*
double CRound::SpecRound( double *input, long nNPICK, CGlobals &globals )
{
	double *pCONS;
	double *pDATA;
	long nNOP[4];
	long *pNLOP;
	long *pNOPER;
	long *pNROP;
	long *pNDXOUT;
	long nNVARY;
	long nNFIND;
	long nNTOP;
	long nNERR;

	long nCSPONT;
	long nCONSIZE;
	long nOUTSIZE;

	long i, j, k;
	int NumOfAlgs = m_TheGraph.GetNumOfAlgs();

	double *A = 0;
	double *B = 0;
	long ToCopy;

	m_dRho = 0.0;


	m_TheGraph.CreateGraph( input, globals );

	m_TheGraph.GetSizes( nNOP[0], nNVARY, nNFIND, i );
	if ( NumOfAlgs == 2 )
	{
		m_TheGraph.GetScndSizes( nNOP[1], i );
		if ( nNPICK <= 1L && nNPICK >= -1L )
		{
			ToCopy = ( A = globals.GetArrayLhs( nNFIND, nNOP[0] ) ) && 
				( B = globals.GetArrayLhs( nNFIND, nNOP[1] ) );
		}
		else
		{
			ToCopy = ( B = globals.GetArrayLhs( nNFIND, nNOP[1] ) ) && 
				( A = globals.GetArrayLhs( nNFIND, nNOP[0] ) );
		}


		Syncronize( nNOP[0] > nNOP[1] ? nNOP[0] : nNOP[1], nNFIND );
		nNOP[2] = m_nNodeSize;
		nNOP[3] = m_nFindSize;
	}
	else
	{					
		long tmp;
		long SizeB;
		if ( nNPICK == 2L || nNPICK == 4L )
			SizeB = nNVARY + nNFIND;
		else
			SizeB = nNVARY;
		if ( nNPICK >= 6L )
		{
			ToCopy = ( B = globals.GetArrayLhs( nNFIND, SizeB ) ) && 
				( A = globals.GetArrayLhs( nNFIND, nNFIND  ) );
		}
		else
		{
			ToCopy = ( A = globals.GetArrayLhs( nNFIND, nNOP[0] ) ) && 
				( B = globals.GetArrayLhs( nNFIND, SizeB  ) );
		}

		nNOP[0] += nNVARY;
 		if ( ( tmp = nNVARY + 2 * nNFIND ) < nNOP[0] )
		{
			tmp = nNOP[0];
		}

		Syncronize( tmp, nNFIND );
		nNOP[1] = m_nNodeSize;
	}
	
	
	m_TheGraph.GetOperations( pNLOP, pNOPER, pNROP );
	m_TheGraph.GetPointers( pNDXOUT, pCONS, pDATA );
	
	if ( NumOfAlgs == 1 )
	{
		nNTOP = m_TheGraph.GetNtop();
		for ( i = 0; i < nNVARY; ++i )
			m_pVALUE[i] = pDATA[i];
		ROUND1_FOR( m_pVALUE, m_pDERIVS, pCONS, &m_dRho, nNOP, pNLOP, pNOPER, pNROP, pNDXOUT,
             &nNVARY, &nNFIND, &nNPICK, &nNTOP, &nNERR, m_pWGTLFT, m_pWGTRGHT, m_pNDCOL,
			 &ToCopy, A, B);
		if ( ToCopy && nNPICK != 0L )
		{
			long limit;
			bool bLE = nNPICK == -3 || nNPICK == 3 || nNPICK == 4 || nNPICK == 7;
			bool bWK = nNPICK == 2 || nNPICK == 4;
			bool bCN = nNPICK == 6 || nNPICK == 7;
			limit = nNVARY * nNFIND;
			if ( bLE )
			{
				double dDatNorm = 0.0;
				double tmp;
				
				for ( i = 0; i < nNVARY; ++i )
				{
					tmp = fabs( pDATA[i] );
					if ( tmp > dDatNorm )
						dDatNorm = tmp;
				}
				
				for ( i = 0; i < limit; ++i )
				{
					B[i] *= dDatNorm;
				}
				if ( bWK || bCN )
				{
					double *C;
					double dValNorm = 0.0;
					for ( i = 0; i < nNFIND; ++i )
					{
						tmp = fabs( m_pVALUE[ pNDXOUT[i] - 1 ] );
						if ( tmp > dValNorm )
							dValNorm = tmp;
					}
					if ( bWK )
					{
						C = B + limit;
					}
					else
					{
						C = A;
					}
					k = 0L;
					for ( j = 0L; j < nNFIND; ++j )
					{
						for ( i = 0L; i < j; ++i )
						{
							C[ k++ ] = 0.0;
						}
						C[ k++ ] = dValNorm;
						++i;
						for ( ; i < nNFIND; ++i )
						{
							C[ k++ ] = 0.0;
						}
					}
				}
			}
			else //!bLE
			{
			
			
				
				for ( j = 0; j < nNVARY; ++j )
				{
					for ( i = 0; i < nNFIND; ++i )
					{
						B[ nNFIND * j + i ] *= pDATA[j];
					}
				}
				if ( bWK || bCN )
				{
					double *C;
					if ( bWK )
					{
						C = B + limit;
					}
					else
					{
						C = A;
					}
					k = 0L;
					for ( j = 0L; j < nNFIND; ++j )
					{
						for ( i = 0L; i < j; ++i )
						{
							C[ k++ ] = 0.0;
						}
						C[ k++ ] = m_pVALUE[ pNDXOUT[i] - 1 ];
						++i;
						for ( ; i < nNFIND; ++i )
						{
							C[ k++ ] = 0.0;
						}
					}
				}
			}
		}
	}
	else
	{
		long limits[3];
		m_TheGraph.GetLimits( nCSPONT, i, nCONSIZE, nOUTSIZE );
		limits[0] = nCSPONT;
		limits[1] = nCONSIZE;
		limits[2] = nOUTSIZE;
		ROUND2_FOR( m_pVALUE, m_pDERIVS, pDATA, pCONS, &m_dRho, nNOP, pNLOP, pNOPER, pNROP,
			pNDXOUT, &nNVARY, &nNFIND, &nNPICK, &nNERR, m_pWGTLFT, m_pWGTRGHT, m_pNDCOL, 
			limits, &ToCopy, A, B);
	}
	
	if ( nNERR != 0 )
		throw CException( (enum err)(nNERR - 1L)  );
//	if ( m_nNUMBER < 0 )
//		positv_c( m_pDERIVS, m_pVALUE, m_pVALUE + nNVARY, &m_nNUMBER );
//	positv_c( m_Constr.GetPtr( m_nNUMBER ), m_pVALUE, m_pVALUE + nNVARY, &m_nNUMBER );
//
	


	
	return m_dRho;
}

*/
void CRound::GetOutput( double *output, double *output2 )
{
            long nNFIND;
			long i;
			long *pNDXOUT;
			double *s;

			

		    m_TheGraph.GetSizes( i, i, nNFIND, i );
			m_TheGraph.GetPointers( pNDXOUT, s, s );		
			for( i = 0; i < nNFIND; ++i )
				output[i] = m_pVALUE[ pNDXOUT[i] - 1 ];
			if ( m_TheGraph.GetNumOfAlgs() == 2 && output2 != 0 )
			{
				long nOUTSIZE;
				double *value2 = m_pVALUE + m_nNodeSize;
				m_TheGraph.GetLimits( i, i, i, nOUTSIZE );
				pNDXOUT += nOUTSIZE;
				for( i = 0; i < nNFIND; ++i )
					output2[i] = value2[ pNDXOUT[i] - 1 ];
			}
				
        
}


		

long CRound::GetInput( double *input )
{
	long s;
	long nInpSize;
	double *pData;
	long *pn;
	double *pd;
	m_TheGraph.GetSizes( s, nInpSize, s, s );
	m_TheGraph.GetPointers( pn, pd, pData );
	if ( input != pData )
	{
		for( s = 0; s < nInpSize; ++s )
			input[s] = pData[s];
	}
	return nInpSize;
}


void CRound::ResetRho( void )
{
	m_dRho = 0.0;
}