#ifndef _CROUND_
#define _CROUND_


#include "CRoundBase.h"
#include "CGraphPtr.h"

class CRound : public CRoundBase
{
private:

	    long m_nNodeSize;
        long m_nFindSize;

        double *m_pVALUE;
        double *m_pDERIVS;

        double *m_pWGTLFT;
        double *m_pWGTRGHT;

        long *m_pNDCOL;

		CGraphManager m_TheGraph;
		double m_dRho;

        void nullinit( void )
        {
                m_nNodeSize = m_nFindSize = 0L;
         
                m_pVALUE = m_pDERIVS = m_pWGTLFT = m_pWGTRGHT = 0;
                m_pNDCOL = 0;
				m_dRho = 0.0;
        }

        void SelfInit( const CRound &other );
		void Syncronize( long nop, long nfind );

protected:

	virtual double SpecRound( double *input, CGlobals &globals );
	virtual void ResetRho();

public:

	CRound( double *pData, long Datsize, long nPick, double dStopx, int NumOfAlgs ) : 
	  CRoundBase(nPick, dStopx ), m_TheGraph( pData, Datsize, NumOfAlgs ) 
	{
		nullinit();
    }
	


    virtual ~CRound( void )
    {
            delete [] m_pVALUE;
            delete [] m_pDERIVS;
            delete [] m_pWGTLFT;
            delete [] m_pWGTRGHT;
            delete [] m_pNDCOL;
    }


    CRound( const CRound &other ) : CRoundBase( other ), m_TheGraph( other.m_TheGraph ) 
	{
        SelfInit( other );
    }



	CRound& operator=( const CRound &other )
    {
		if ( &other == this )
			return *this;

		CRoundBase::operator=( other );
		m_TheGraph = other.m_TheGraph;

			
        delete [] m_pVALUE;
        delete [] m_pDERIVS;
        delete [] m_pWGTLFT;
        delete [] m_pWGTRGHT;
        delete [] m_pNDCOL;

        SelfInit( other );
		return *this;
	}

	bool FillBegin( double *input, CGlobals &globals );

	void FillEnd( CGlobals &globals );

	double ComputeRho( CGlobals &globals );
	double ComputeRho( double &penalty, CGlobals &globals );

/*        double *InitVALUE( double *pD )
    {
   			if ( !m_bHasGraph )
				return 0;
			
			long i;
			for ( i = 0; i < *m_pNVARY; ++i )
                m_pVALUE[i] = pD[i];
			return m_pVALUE + *m_pNVARY; 
			
    }
*/

	virtual void SetData( double *input, long nDatSize, bool ParChanged )
	{
		m_TheGraph.SetData( input, nDatSize, ParChanged );
	}

	virtual long GetDatSize( void )
	{
		return m_TheGraph.GetDatSize();
	}

    virtual long GetOutSize( void )
    {
        long nNFIND;
		long s;

		

		m_TheGraph.GetSizes( s, s, nNFIND, s ); 

		return nNFIND;
    }

	virtual long GetInput( double *input );

    virtual void GetOutput( double *output, double *output2 = 0 );

    virtual double GetRho( void )
    {
            return m_dRho;
    }
	virtual long GetInputSize( void )
	{
		long nInpSize;
		long s;
		m_TheGraph.GetSizes( s, nInpSize, s, s );
		return nInpSize;
	}

	virtual long GetNumOfOpers( int i )
	{
		long nop = 0;
		long s;
		if ( i == 1 )
		{
			m_TheGraph.GetSizes( nop, s, s, s ); 
		}
		else if ( i == 2 )
			m_TheGraph.GetScndSizes( nop, s );
		return nop;
	}

	virtual long GetNumOfConstants( int i )
	{
		long cons = 0;
		long s;
		if ( i == 1 )
			m_TheGraph.GetSizes( s, s, s, cons ); 
		else if ( i == 2 )
			m_TheGraph.GetScndSizes( s, cons );
		return cons;
	}

	virtual void GetGraph( long *SizeArray, double *&cons, long *&left, long *&oper, long *&right, long *&output, int alg )
	{
		m_TheGraph.GetGraph( SizeArray, cons, left, oper, right, output, alg );
	}

	virtual long GetMemAlloc( void )
	{
		
		return CRoundBase::GetMemAlloc() + m_TheGraph.GetMemAlloc() + 
			m_nFindSize * sizeof( long ) + m_TheGraph.GetNumOfAlgs() * m_nNodeSize *
			( 3L + m_nFindSize ) * sizeof( double );
	}

	virtual long GetMemUsed( void )
	{
		long nop;
		long find;
		long nvary;
		long tmp;
		m_TheGraph.GetSizes( nop, nvary, find, tmp );
		if ( m_TheGraph.GetNumOfAlgs() == 2 )
			m_TheGraph.GetScndSizes( nvary, tmp );
		return CRoundBase::GetMemUsed() + m_TheGraph.GetMemUsed() 
			+ find * sizeof( long ) + ( nop + nvary ) * ( 3L + find ) * sizeof( double );
	}

	virtual void ResetCounter( void )
	{
		CRoundBase::ResetCounter();
		m_TheGraph.reset();
	}


		



//		int RestoreData( void );

		
	
};

#endif
