#ifndef _CROUNDBASE_
#define _CROUNDBASE_

#include "CException.h"
#include "CConstraints.h"

class CRoundBase
{
private:
	
	long m_nNPICK;
	CConstraints m_Constr;
	long m_nNCALLS;
	bool m_nNPRINT;
	double m_dSTOPX;

protected:
	virtual double SpecRound( double *input, CGlobals &globals ) = 0;
	virtual void ResetRho( void ) = 0;


public:
	CRoundBase( long nNPICK, double dSTOPX )
	{
		m_nNPICK = nNPICK;
		m_dSTOPX = dSTOPX;
		
		m_nNCALLS = 0L;
		m_nNPRINT = true;
 
  	}
	virtual ~CRoundBase( void ) = 0;

	void SetEcw( long npick )
	{
		m_nNPICK = npick;
		ResetRho();
	}

	void SetStopCrit( double stopx )
	{
		m_dSTOPX = stopx;
	}

	long GetEcw( void )
	{
		return m_nNPICK;
	}
	
	double GetStopCrit( void )
	{
		return m_dSTOPX;
	}

	double round( double *input, double &penalty, CGlobals &globals );

	virtual bool FillBegin( double *input, CGlobals &globals );
	virtual void FillEnd( CGlobals &globals );// = 0;	
	virtual double ComputeRho( double &penalty, CGlobals &globals );

	virtual void SetData( double *input, long nDatSize, bool ParChanged ) = 0;
	virtual long GetDatSize( void ) = 0;

	virtual long GetOutSize( void ) = 0;
	virtual long GetInput( double *input ) = 0;
	virtual void GetOutput( double *output, double *output2 = 0 ) = 0;
	virtual double GetRho( void ) = 0;
	virtual long GetInputSize( void ) = 0;
	virtual long GetNumOfOpers( int i ) = 0;
	virtual long GetNumOfConstants( int i )
	{
		return 0L;
	}
	virtual void GetGraph( long *SizeArray, double *&cons, long *&left, long *&oper, long *&right, long *&output, int alg ) = 0;

	virtual long GetMemAlloc( void )
	{
		return m_Constr.GetMemAlloc();
	}

	virtual long GetMemUsed( void )
	{
		return m_Constr.GetMemUsed();
	}

	void IncCounter( void )
	{
		long nNVARY;
		nNVARY = GetInputSize();
		++m_nNCALLS;
		if ( m_Constr.GetPenalty() > 0 )
		{
			if ( ( m_nNCALLS ) % ( 5 * nNVARY ) == 0 )
			{
				if ( m_nNPRINT )
				{
					m_nNPRINT = false;
					throw CException( report_process, m_nNCALLS, GetRho(), true );
				}
				throw CException( report_process, m_nNCALLS, GetRho(), false );
	
			}
		}
		if ( m_nNCALLS > 200 * nNVARY )
			throw CException( call_limit_exceeded, m_nNCALLS, GetRho() );
		
	}

	virtual void ResetCounter( void )
	{
		m_nNCALLS = 0L;
		m_nNPRINT = true;
	}

	long GetCounter( void )
	{
		return m_nNCALLS;
	}

	void ReportConstraints( void )
	{
		m_Constr.ReportConstraints();
	}
	
	double GetPenalty( void )
	{
		return m_Constr.GetPenalty();
	}

	long GetNumOfConstr( void )
	{
		return m_Constr.GetSize();
	}

	void GetConstraints( double *pCopy )
	{
		m_Constr.GetConstraints( pCopy );
	}




};

#endif