#include <mex.h>
#include "CRoundUkkonen.h"
#include "CGlobals.h"

extern "C" void dpptrf( char *UPLO, long *N, double *B, long *INFO );
extern "C" void dspgst( long *ITYPE, char *UPLO, long *N, double *A, 
					   double *B, long *INFO );
extern "C" void dsptrd( char *UPLO, long *N, double *A, double *D, double *E, 
					   double *TAU, long *INFO ); 
extern "C" void dstegr( char *JOBZ, char *RANGE, long *N, double *D, double *E, 
					   double *VL, double *VU, long *IL, long *IU, double *ABSTOL, 
					   long *M, double *W, double *Z, long *LDZ, long *ISUPPZ, 
					   double *WORK, long *lwork, double *IWORK, long *liwork, 
					   long *INFO );
 
long CHOLESKY( long N, double *B  )
{
      long info = 0;
	  char *UL = "U";
	  dpptrf( UL, &N, B, &info );
	  return info;
}

long GENEIG( long N, double *A, double *B )
{
      long info = 0;
	  long one = 1L;
	  char *UL = "U";
	  dspgst( &one, UL, &N, A, B, &info );
	  return info;
}

long TRIDIAGON( long N, double *A, double *D, double *E, double *TAU )
{
	long info = 0;
	char *UL = "U";
      
    dsptrd( UL, &N, A, D, E, TAU, &info );
	return info;
}

long EIGENV( long N, double *D, double *E, long *M, double *W, double *Z, 
			double *WORK, long lwork, double *IWORK, long liwork, long *ISUPPZ)
{

    long info = 0;
	char *sN = "N";
	char *sA = "A";
	double zero = 0.0;
	double one = 1.0;
	long lone = 1L;

    dstegr( sN, sA, &N, D, E, &zero, &zero, &lone, &lone, &one, M, W, Z, &N, 
		ISUPPZ, WORK, &lwork, IWORK, &liwork, &info );
    return info;
}
      

long CRoundUkkonen::GetInput( double *input )
{
	long nInpSize;
	long i;
	double *pData;
	nInpSize = m_pDerivs->GetInput( pData );
	
	if ( input != pData )
	{
		for( i = 0; i < nInpSize; ++i )
			input[i] = pData[i];
	}
	return nInpSize;
}

double CRoundUkkonen::SpecRound( double *input, CGlobals &globals )
{
	double *A = 0;
	double *B = 0;

	long n;
	long nNPICK = GetEcw();
	m_dRho = 0.0;
	if ( m_nNumOfAlgs == 1 )
	{
		m_pDerivs->Derive( input, nNPICK, globals );
		n = m_pDerivs->GetOutSize();
	}
	else
	{
		try
		{
			m_pDerivs->Derive( input, 0, globals, 1 );
			n = m_pDerivs->GetOutSize();
		}
		catch ( CException exception )
		{
			if ( exception.ErrorCode() == input_too_large )
				throw CException( input_too_large_1 );
			if ( exception.ErrorCode() == no_output )
				throw CException( no_output_1 );
			if ( exception.ErrorCode() == no_operation )
				throw CException( no_operation_1 );
			throw;
		}
		try
		{
			m_pDerivs2->Derive( input, 0, globals, 2 );
		}
		catch ( CException exception )
		{
			if ( exception.ErrorCode() == input_too_large )
				throw CException( input_too_large_2 );
			if ( exception.ErrorCode() == no_output )
				throw CException( no_output_2 );
			if ( exception.ErrorCode() == no_operation )
				throw CException( no_operation_2 );
			throw;
		}
		if ( n != m_pDerivs2->GetOutSize() )
			throw CException( diff_output_num );
		if ( m_pDerivs->GetInputSize() != m_pDerivs2->GetInputSize() )
			throw CException( diff_input_num );
	}
	m_pDerivs->Pack();
	if ( m_nNumOfAlgs > 1 )
	{
		m_pDerivs2->Pack();
     	if ( ( A = globals.GetArrayLhs( n, n ) ) && 
			( B = globals.GetArrayLhs( n, n  ) ) )
		{
			if ( nNPICK == -2L || nNPICK == 2L )
			{
				m_pDerivs2->GetMatrices( A );
				m_pDerivs->GetMatrices( B );
			}
			else
			{
				m_pDerivs->GetMatrices( A );
				m_pDerivs2->GetMatrices( B );
			}
		}
	}
	else
	{
		m_pDerivs->KahanCorrection();
     	if ( ( A = globals.GetArrayLhs( n, n ) ) && 
			( B = globals.GetArrayLhs( n, n  ) ) )
			m_pDerivs->GetMatrices( A, B );

		m_pDerivs->CndNumCorrection();
	}

	if ( nNPICK != 0 )
	{		
		if ( m_nNumOfAlgs == 1 )
		{
			A = m_pDerivs->GetPackedA();
			B = m_pDerivs->GetPackedB();
		}
		else if( nNPICK % 2 == 0)
		{
			A = m_pDerivs2->GetPackedA();
			B = m_pDerivs->GetPackedA();
		}
		else
		{
			A = m_pDerivs->GetPackedA();
			B = m_pDerivs2->GetPackedA();
		}
		if ( nNPICK > 0 && nNPICK < 6 )
		{
			long info;
			long M;

			info = CHOLESKY( n, B );
				
			if ( info != 0 )
				throw CException( cholesky );
			info = GENEIG( n, A, B );
			if ( info != 0 )
				throw CException( geneig );
			InitSpace( n );
			info = TRIDIAGON( n, A, m_pD, m_pE, m_pTAU );
			if ( info != 0 )
				throw CException( tridiagon );

			info = EIGENV( n, m_pD, m_pE, &M, m_pW, m_pTAU, m_pWORK, 
				LWORK * n, m_pIWORK, LIWORK * n, m_pISUPPZ );

			if ( info != 0 )
				throw CException( eigen );
			m_dRho =  m_pW[ n - 1 ];
		
//			if ( mexCallMATLAB( 1, &plhs, 2, prhs, "jwcompute" ) != 0 )
//				throw CException( omega );
		}
		else
		{
			long info;
			long M;

			InitSpace( n );

			info = TRIDIAGON( n, B, m_pD, m_pE, m_pTAU );
			if ( info != 0 )
				throw CException( tridiagon );

			info = EIGENV( n, m_pD, m_pE, &M, m_pW, m_pTAU, m_pWORK, 
				LWORK * n, m_pIWORK, LIWORK * n, m_pISUPPZ );

			if ( info != 0 )
				throw CException( eigen );
			m_dRho =  m_pW[ n - 1 ];

			if ( nNPICK < 0 )
			{
				info = TRIDIAGON( n, A, m_pD, m_pE, m_pTAU );
				if ( info != 0 )
					throw CException( tridiagon );
	
				info = EIGENV( n, m_pD, m_pE, &M, m_pW, m_pTAU, m_pWORK, 
					LWORK * n, m_pIWORK, LIWORK * n, m_pISUPPZ );

				if ( info != 0 )
					throw CException( eigen );
				m_dRho =  m_pW[ n - 1 ] / m_dRho;
			
			}
			
 
			//			if ( mexCallMATLAB( 1, &plhs, 2, prhs, "ercompute" ) != 0 )
//				throw CException( geter );
		}
		m_dRho = sqrt(m_dRho);
	}
	return m_dRho;
}

void CRoundUkkonen::ResetRho( void )
{
	m_dRho = 0.0;
}


	


