#ifndef _CROUNDUKKONEN_
#define _CROUNDUKKONEN_


#include "CUkkonenAGraph.h"
#include "CRoundBase.h"
#define LWORK 50L
#define LIWORK 25L

class CRoundUkkonen : public CRoundBase
{
private:
	CUkkonen *m_pDerivs;
	CUkkonen *m_pDerivs2;
	double m_dRho;
	int m_nNumOfAlgs;

	double *m_pD;
	double *m_pE;
	double *m_pTAU;
	double *m_pWORK;
	double *m_pW;
	double *m_pIWORK;
	long *m_pISUPPZ;
	long m_nLimit;

	bool m_bIsGraph;

    void InitSpace( long nSize )
	{
		if ( nSize > m_nLimit )
		{
			double *D;
			double *E;
			double *TAU;
			double *WORK;
			double *W;
			double *IWORK;
			long *ISUPPZ;
			D = new double[ nSize ];
			E = new double[ nSize ];
			TAU = new double[ nSize * nSize ];
			WORK = new double[ LWORK * nSize ];
			W = new double[ nSize ];
			IWORK = new double[ LIWORK * nSize ];
			ISUPPZ = new long[ 2L * nSize ];

			delete [] m_pD;
			delete [] m_pE;
			delete [] m_pTAU;
			delete [] m_pWORK;
			delete [] m_pW;
			delete [] m_pIWORK;
			delete [] m_pISUPPZ;

			m_pD = D;
			m_pE = E;
			m_pTAU = TAU;
			m_pWORK = WORK;
			m_pW = W;
			m_pIWORK = IWORK;
			m_pISUPPZ = ISUPPZ;

			m_nLimit = nSize;
		}
	}



protected:
	virtual double SpecRound( double *input, CGlobals &globals );
	virtual void ResetRho( void );

public:
	CRoundUkkonen( double *pData, long Datsize, long nPick, double dStopx, int nNumOfAlgs, 
		bool IsGraph ) : 
	  CRoundBase(nPick, dStopx )
	{
		m_dRho = 0.0;
		m_nNumOfAlgs = nNumOfAlgs;
		m_bIsGraph = IsGraph;

		m_nLimit = 0L;
		m_pD = m_pE = m_pTAU = m_pWORK = m_pW = m_pIWORK = 0;
		m_pISUPPZ = 0;

		if ( m_nNumOfAlgs == 2 )
		{
			if ( IsGraph )
			{
				m_pDerivs = new CUkkonenAGraph( pData, Datsize, 0 );
				m_pDerivs2 = new CUkkonenAGraph( pData, Datsize, 0 );

			}
			else
			{
				m_pDerivs = new CUkkonenA( pData, Datsize, 0 );
				m_pDerivs2 = new CUkkonenA( pData, Datsize, 0 );
			}
		}
		else
		{
			if ( IsGraph )
				m_pDerivs = new CUkkonenGraph( pData, Datsize, nPick  );
			else
				m_pDerivs = new CUkkonen( pData, Datsize, nPick  );
			m_pDerivs2 = 0;
		}

	}

	void SelfInit( const CRoundUkkonen &other )
	{
		m_nNumOfAlgs = other.m_nNumOfAlgs;
		m_dRho = other.m_dRho;
		m_bIsGraph = other.m_bIsGraph;
		m_pDerivs2 = m_pDerivs = 0;
		if ( other.m_pDerivs != 0 )
			m_pDerivs = other.m_pDerivs->clone();
		if ( other.m_pDerivs2 != 0 )
			m_pDerivs = other.m_pDerivs2->clone();
		m_nLimit = 0L;
		m_pD = m_pE = m_pTAU = m_pWORK = m_pW = m_pIWORK = 0;
		m_pISUPPZ = 0;

	}


	CRoundUkkonen( const CRoundUkkonen &other ) : CRoundBase( other )
	{
		SelfInit( other );
	}

   	CRoundUkkonen& operator=( const CRoundUkkonen &other )
    {
		if ( &other == this )
			return *this;

		CRoundBase::operator=( other );

		delete m_pDerivs;
		delete m_pDerivs2;

		delete [] m_pD;
		delete [] m_pE;
		delete [] m_pTAU;
		delete [] m_pWORK;
		delete [] m_pW;
		delete [] m_pIWORK;
		delete [] m_pISUPPZ;

        SelfInit( other );
		return *this;
	}

	virtual ~CRoundUkkonen( void )
	{
		delete m_pDerivs;
		delete m_pDerivs2;

		delete [] m_pD;
		delete [] m_pE;
		delete [] m_pTAU;
		delete [] m_pWORK;
		delete [] m_pW;
		delete [] m_pIWORK;
		delete [] m_pISUPPZ;

	}

	virtual void SetData( double *input, long nDatSize, bool ParChanged )
	{
		m_pDerivs->SetData( input, nDatSize, ParChanged );
		if ( m_pDerivs2 != 0 )
			m_pDerivs2->SetData( input, nDatSize, ParChanged );
	}

	virtual long GetDatSize( void )
	{
		return GetInputSize();
	}




	virtual long GetOutSize( void )
	{
		return m_pDerivs->GetOutSize();
	}

	virtual void GetOutput( double *output, double *output2 = 0 )
	{
		m_pDerivs->GetOutput( output );
		if ( m_nNumOfAlgs == 2 && output2 != 0 )
			m_pDerivs2->GetOutput( output2 );
	}

	virtual long GetInput( double *input );

	virtual double GetRho( void )
	{
		return m_dRho;
	}
	virtual long GetInputSize( void )
	{
		return m_pDerivs->GetInputSize();
	}

	virtual long GetNumOfOpers( int i )
	{
		long nop = 0L;
		if ( i == 1  )
			nop = m_pDerivs->GetNumOfOpers();
		else if ( i == 2 && m_nNumOfAlgs == 2  )
			nop = m_pDerivs2->GetNumOfOpers();
		return nop;
	}

	long GetMatrixSize( int i )
	{
		long ret = 0L;
		if ( i == 1 )
			ret = m_pDerivs->GetMatrixSize();
		else if ( i == 2 && m_nNumOfAlgs == 2 )
			ret = m_pDerivs2->GetMatrixSize();
		return ret;
	}


	virtual void GetGraph( long *SizeArray, double *&cons, long *&left, long *&oper, long *&right, long *&output, int alg )
	{

			if ( alg == 1 )
			{
				m_pDerivs->GetGraph( SizeArray, cons, left, oper, right, output );
			}
			else if ( alg == 2 && m_nNumOfAlgs == 2 )
			{
				m_pDerivs2->GetGraph( SizeArray, cons, left, oper, right, output );
			}
			else
			{
	    		SizeArray[0] = SizeArray[1] = SizeArray[2] = 0L;
				cons = 0;
				left = oper = right = output = 0;
			}
		
	}



	virtual long GetMemAlloc( void )
	{
		long retVal;
		retVal = 0L;
		retVal += m_pDerivs->GetMemAlloc();
		if ( m_pDerivs2 != 0 )
			retVal += m_pDerivs2->GetMemAlloc();
		retVal += ( m_nLimit + LWORK + LIWORK + 3L ) * m_nLimit * sizeof( double )
			+ 2 * m_nLimit * sizeof( long );

		return retVal;
	}

	virtual long GetMemUsed( void )
	{
		long nSize;
		long retVal;
		retVal = 0L;
	    retVal += m_pDerivs->GetMemUsed();
		if ( m_pDerivs2 != 0 )
			retVal += m_pDerivs2->GetMemUsed();
		nSize = GetOutSize();
		retVal += ( nSize + LWORK + LIWORK + 3L ) * nSize * sizeof( double )
			+ 2 * nSize * sizeof( long );
		return retVal;
	}
	





};


#endif
