#include "CUkkonen.h"
#include "miller.h"
#include "CGlobals.h"

CUkkonen::CUkkonen( double *pDATA, long nDATSIZE, long nNPICK )
{
/*
	long i;
	double tmp;
	m_pDATA = pDATA;
	m_nDATSIZE = nDATSIZE;
	m_nNPICK = nNPICK;
	m_nInputCounter = 0L;
	m_nOutputCounter = 0L;
	m_nOperCounter = 0L;

	m_dDatNorm = 0.0;
	m_bEachEntry = 
		( m_nNPICK == -1L || m_nNPICK == 1L || m_nNPICK == 2L || m_nNPICK == 6 );
	m_bLargestEntry = 
		( m_nNPICK == -3L || m_nNPICK == 3L || m_nNPICK == 4L || m_nNPICK == 7 );
	m_bWK = ( m_nNPICK == 2L || m_nNPICK == 4L );
	m_bER = ( m_nNPICK == -1L || m_nNPICK == -3L );
	if ( m_bLargestEntry )
	{
		for ( i = 0; i < m_nDATSIZE; ++i )
		{
			tmp = m_pDATA[i] * m_pDATA[i];
			if ( tmp > m_dDatNorm )
				m_dDatNorm = tmp;
		}
	}
	m_bErrorFree = false;
	m_Wmatrix.InitInput( m_pDATA, m_nDATSIZE, m_dDatNorm );
*/
	Init( pDATA, nNPICK, nDATSIZE );

}
void CUkkonen::Init( double *pDATA, long nNPICK, long nDATSIZE )
{
	double tmp;
	long i;


    m_pDATA = pDATA;
	m_nNPICK = nNPICK;
	if ( nDATSIZE >= 0 )
		m_nDATSIZE = nDATSIZE;	
	m_nInputCounter = 0L;
	m_nOutputCounter = 0L;
	m_nOperCounter = 0L;
	
	m_dDatNorm = 0.0;
	m_bEachEntry = 
		( m_nNPICK == -1L || m_nNPICK == 1L || m_nNPICK == 2L || m_nNPICK == 6 );
	m_bLargestEntry = 
		( m_nNPICK == -3L || m_nNPICK == 3L || m_nNPICK == 4L || m_nNPICK == 7 );
	m_bWK = ( m_nNPICK == 2L || m_nNPICK == 4L );
	m_bER = ( m_nNPICK == -1L || m_nNPICK == -3L );
	m_bCN = ( m_nNPICK == 6L || m_nNPICK == 7L ); 

	if ( m_bLargestEntry )
	{
		for ( i = 0; i < m_nDATSIZE; ++i )
		{
			tmp = m_pDATA[i] * m_pDATA[i];
			if ( tmp > m_dDatNorm )
				m_dDatNorm = tmp;
		}
	}
	else if ( !m_bEachEntry )
	{
		m_dDatNorm = 1.0;
	}
	m_bErrorFree = false;
//	m_Wmatrix.InitInput( m_pDATA, m_nDATSIZE, m_dDatNorm );
}

void CUkkonen::RunAlgorithm( int AlgIndex, CGlobals &globals )
{
	m_Wmatrix.InitInput( m_pDATA, m_nDATSIZE, m_dDatNorm );
	CallAlgorithm( AlgIndex, globals );
}

void CUkkonen::SetData( double *input, long nDatSize, bool ParChanged )
{
	m_pDATA = input;
	m_nDATSIZE = nDatSize;
}

void CUkkonen::Derive( double *input, long nNPICK, CGlobals &globals, int AlgIndex )
{
//	long i;
//	double tmp;
	globals.InitConstraints();
	globals.SetUkkonenPtr( this );
/*	m_pDATA = input;
	m_nNPICK = nNPICK;
	
	m_nInputCounter = 0L;
	m_nOutputCounter = 0L;
	m_nOperCounter = 0L;
	
	m_dDatNorm = 0.0;
	m_bEachEntry = ( m_nNPICK == -1L || m_nNPICK == 1L || m_nNPICK == 2L );
	m_bLargestEntry = ( m_nNPICK == -3L || m_nNPICK == 3L || m_nNPICK == 4L );
	m_bWK = ( m_nNPICK == 2L || m_nNPICK == 4L );
	m_bER = ( m_nNPICK == -1L || m_nNPICK == -3L );

	if ( m_bLargestEntry )
	{
		for ( i = 0; i < m_nDATSIZE; ++i )
		{
			tmp = m_pDATA[i] * m_pDATA[i];
			if ( tmp > m_dDatNorm )
				m_dDatNorm = tmp;
		}
	}
	m_bErrorFree = false;
*/
	
	ReleaseAll();

//	m_Wmatrix.InitInput( m_pDATA, m_nDATSIZE, m_dDatNorm );

	Init( input, nNPICK );

	RunAlgorithm( AlgIndex, globals );
	
	if ( m_nInputCounter != m_nDATSIZE )
		throw CException( input_too_large );
			
	if ( m_nOutputCounter == 0L )
		throw CException( no_output );
		
	if ( m_nOperCounter == 0L  )
		throw CException( no_operation );

}

void CUkkonen::GetMatrices( double *A, double *B )
{
	long j;
	long i;
	long count_j = 0L;
	long count_i = 0L;
	long n;
	long index1;
	long index2;
	double OutputNorm = 0.0;
	n = m_nOutputCounter;
	CUkkonenCol *j_ptr;
	CUkkonenCol *i_ptr;
	double tmp = 0.0;
	if ( m_bCN && m_bLargestEntry )
	{
		for ( j = m_Wmatrix.InitIndex(); j > 0L; j = m_Wmatrix.AdvanceIndex( j ) )
		{
			tmp = m_Wmatrix.GetVariable( j ).GetValue();
			tmp *= tmp;
			if ( tmp > OutputNorm )
				OutputNorm = tmp;
		}
		tmp = OutputNorm;
	}

    if ( m_bCN )
	{
	
		for ( j = m_Wmatrix.InitIndex(); j > 0L; j = m_Wmatrix.AdvanceIndex( j ) )
		{
			if ( ( j_ptr = &m_Wmatrix.GetVariable( j ) )->IsOutput() )
			{
				count_j = j_ptr->GetKind() - 1;
				for ( i = m_Wmatrix.InitIndex(); i > 0L && i < j; i = m_Wmatrix.AdvanceIndex( i ) )
				{
					if ( ( i_ptr =  &m_Wmatrix.GetVariable( i ) )->IsOutput() )
					{
                        count_i = i_ptr->GetKind() - 1;
						index1 = count_j * n + count_i;
						index2 = count_i * n + count_j;
						A[ index1 ] = A[ index2 ] = m_Wmatrix.BBT( i, j );
						B[ index1 ] = B[ index2 ] = 0.0;
					
					}
				}
				count_i = ( m_Wmatrix.GetVariable( i ) ).GetKind() - 1;
				if ( i != j || count_i != count_j )
					throw CException( "i!=j in CUkkonen::GetMatrices" );
				index1 = count_i * n + count_i;
				if ( m_bEachEntry )
				{
					tmp = j_ptr->GetValue();
					tmp *= tmp;
				}
				A[ index1 ] = m_Wmatrix.BBT( i, i );
				B[ index1 ] = tmp;
	
			}
		}
	}
	else
	{
		
		for ( j = m_Wmatrix.InitIndex(); j > 0L; j = m_Wmatrix.AdvanceIndex( j ) )
		{
			if ( ( j_ptr = &m_Wmatrix.GetVariable( j ) )->IsOutput() )
			{
				count_j = j_ptr->GetKind() - 1;
				for ( i = m_Wmatrix.InitIndex(); i > 0L && i < j; i = m_Wmatrix.AdvanceIndex( i ) )
				{
					if ( ( i_ptr = &m_Wmatrix.GetVariable( i ) )->IsOutput() )
					{
						count_i = i_ptr->GetKind() - 1;
						index1 = count_j * n + count_i;
						index2 = count_i * n + count_j;
						A[ index1 ] = A[ index2 ] = m_Wmatrix.AAT( i, j );
						B[ index1 ] = B[ index2 ] = m_Wmatrix.BBT( i, j );
						
					}
				}
				count_i = ( m_Wmatrix.GetVariable( i ) ).GetKind() - 1;
				if ( i != j || count_i != count_j )
					throw CException( "i!=j in CUkkonen::GetMatrices" );
				index1 = count_i * n + count_i;
				A[ index1 ] = m_Wmatrix.AAT( i, i );
				B[ index1 ] = m_Wmatrix.BBT( i, i );
			
			}
		}

	}
}

double *CUkkonen::GetPackedB( void )
{
	if ( m_Wmatrix.GetLimit() > 0L)
		return &m_Wmatrix.BBT( 1L, 1L );
	else
		return 0;
}


void CUkkonen::KahanCorrection( void )
{
	long j;
	double OutputNorm = 0.0;
	double tmp = 0.0;
	if ( !m_bWK  )
		return;
	if ( m_bLargestEntry )
	{
		for ( j = m_Wmatrix.InitIndex(); j > 0L; j = m_Wmatrix.AdvanceIndex( j ) )
		{
			tmp = m_Wmatrix.GetVariable( j ).GetValue();
			tmp *= tmp;
			if ( tmp > OutputNorm )
				OutputNorm = tmp;
		}
		tmp = OutputNorm;
		for ( j = m_Wmatrix.InitIndex(); j > 0L; j = m_Wmatrix.AdvanceIndex( j ) )
		{
			m_Wmatrix.BBT( j, j ) += tmp;
		}
	}
	else if ( m_bEachEntry )
	{
		for ( j = m_Wmatrix.InitIndex(); j > 0L; j = m_Wmatrix.AdvanceIndex( j ) )
		{
			tmp = ( m_Wmatrix.GetVariable( j ) ).GetValue();
			tmp *= tmp;
			m_Wmatrix.BBT( j, j ) += tmp;
		}
	}
}


void CUkkonen::CndNumCorrection( void )
{
	long i, j;
	double OutputNorm = 0.0;
	double tmp = 0.0;
	double jtmp = 0.0;
	if ( !m_bCN )
		return;
	if ( m_bLargestEntry )
	{
		for ( j = m_Wmatrix.InitIndex(); j > 0L; j = m_Wmatrix.AdvanceIndex( j ) )
		{
			tmp = m_Wmatrix.GetVariable( j ).GetValue();
			tmp *= tmp;
			if ( tmp > OutputNorm )
				OutputNorm = tmp;
		}
		tmp = OutputNorm;
		for ( i = m_Wmatrix.InitIndex(); i > 0L; i = m_Wmatrix.AdvanceIndex( i ) )
		{
			for ( j = i; j > 0L; j = m_Wmatrix.AdvanceIndex( j ) )
			{
				m_Wmatrix.BBT( i, j ) /= tmp;
			}
		}
	}
	else if ( m_bEachEntry )
	{
		for ( i = m_Wmatrix.InitIndex(); i > 0L; i = m_Wmatrix.AdvanceIndex( i ) )
		{
			tmp = ( m_Wmatrix.GetVariable( i ) ).GetValue();
			for ( j = i; j > 0L; j = m_Wmatrix.AdvanceIndex( j ) )
			{
				jtmp = ( m_Wmatrix.GetVariable( j ) ).GetValue();
				m_Wmatrix.BBT( i, j ) /= ( tmp * jtmp );
			}
		}
	}
}

/*
long CUkkonen::input( double *value )
{
	long i;
	long j;
	CUkkonenCol &variable = m_Wmatrix.GetVariable( i = m_Wmatrix.Require() );
	double tmp;

	if( m_nInputCounter >= m_nDATSIZE )
		throw CException( input_too_large );
	if ( m_bEachEntry )
	{
		tmp = m_pDATA[ m_nInputCounter ];
		tmp *= tmp;
	}
	else if ( m_bLargestEntry )
		tmp = m_dDatNorm;
	else
		tmp = 1.0;


	
	if ( !variable.IsConst() )
	{
			for ( j = m_Wmatrix.InitIndex(); j > 0L && j < i; j = m_Wmatrix.AdvanceIndex( j ) )
				m_Wmatrix.AAT( j, i ) = m_Wmatrix.BBT( j, i ) = 0.0;
			if ( i != j )
				throw CException( "i!=j in CUkkonen::input." );
			m_Wmatrix.AAT( i, j ) = 0.0;
            m_Wmatrix.BBT( i, j ) = tmp;
			j = m_Wmatrix.AdvanceIndex( j );
			for ( ; j > 0L; j = m_Wmatrix.AdvanceIndex( j ) )
				m_Wmatrix.AAT( i, j ) = m_Wmatrix.BBT( i, j ) = 0.0;
	}
	else
	{
            m_Wmatrix.BBT( i, i ) = tmp;
	}
	variable.MarkAsInput();
	*value = m_pDATA[ m_nInputCounter++ ];
	return i;

}
*/
void CUkkonen::update( long i, long j,  double alpha, double z, long l, double beta )
{
	long t;
	double alpha2;
	if ( alpha == 0.0 )
	{
		t = j;
		j = l;
		l = t;
		alpha = beta;
		beta = 0.0;
	}
	alpha2 = alpha * alpha;
	if ( beta != 0.0 )
	{
		double beta2 = beta * beta;
		double albe2 = 2 * alpha * beta;

		for ( t = m_Wmatrix.InitIndex(); t > 0L && t < i; t = m_Wmatrix.AdvanceIndex( t ) )
		{
			m_Wmatrix.AAT( t, i ) = alpha * m_Wmatrix.AAT( j, t ) + beta * m_Wmatrix.AAT( l, t );
			m_Wmatrix.BBT( t, i ) = alpha * m_Wmatrix.BBT( j, t ) + beta * m_Wmatrix.BBT( l, t );
		}
		if ( i != t )
			throw CException( "i!=t in CUkkonen::update." );
		m_Wmatrix.AAT( i, i ) = alpha2 * m_Wmatrix.AAT( j, j ) + beta2 * m_Wmatrix.AAT( l, l ) + 
			z + albe2 * m_Wmatrix.AAT( j, l );
		m_Wmatrix.BBT( i, i ) = alpha2 * m_Wmatrix.BBT( j, j ) + beta2 * m_Wmatrix.BBT( l, l ) + 
			albe2 * m_Wmatrix.BBT( j, l );
		t = m_Wmatrix.AdvanceIndex( t );
		for ( ; t > 0L; t = m_Wmatrix.AdvanceIndex( t ) )
		{
			m_Wmatrix.AAT( i, t ) = alpha * m_Wmatrix.AAT( j, t ) + beta * m_Wmatrix.AAT( l, t );
			m_Wmatrix.BBT( i, t ) = alpha * m_Wmatrix.BBT( j, t ) + beta * m_Wmatrix.BBT( l, t );
		}
	}
	else
	{
		for ( t = m_Wmatrix.InitIndex(); t > 0L && t < i; t = m_Wmatrix.AdvanceIndex( t ) )
		{
			m_Wmatrix.AAT( t, i ) = alpha * m_Wmatrix.AAT( j, t );
			m_Wmatrix.BBT( t, i ) = alpha * m_Wmatrix.BBT( j, t );
		}
		if ( i != t )
			throw CException( "i!=t in CUkkonen." );
		m_Wmatrix.AAT( i, i ) = alpha2 * m_Wmatrix.AAT( j, j ) + z;
		m_Wmatrix.BBT( i, i ) = alpha2 * m_Wmatrix.BBT( j, j );
		t = m_Wmatrix.AdvanceIndex( t );
		for ( ; t > 0L; t = m_Wmatrix.AdvanceIndex( t ) )
		{
			m_Wmatrix.AAT( i, t ) = alpha * m_Wmatrix.AAT( j, t );
			m_Wmatrix.BBT( i, t ) = alpha * m_Wmatrix.BBT( j, t );
		}
	}
}

long CUkkonen::Insert( enum operation op, long j,  double vi, double vj , long l, double vl )
{
	long i;
	CUkkonenCol *left = 0;
	CUkkonenCol *right = 0;
	CUkkonenCol *result = 0;

	double alpha;
	double beta;

	double z;

	if ( j < 0L )
		j = -j;
	if ( l < 0L )
		l = -l;
	left = &m_Wmatrix.GetVariable( j );
//	result = &m_Wmatrix.GetVariable( i = m_Wmatrix.Require() );
	if ( op == id )
	{
/*		result->SetKind( left->GetKind() );
		update( i, j, 1.0, );
		return result->IsConst() ? -i : i;
*/
		left->IncRefCounter();
		return left->IsConst() ? -j : j;
		
	}
	if ( op < sqroot )
		right = &m_Wmatrix.GetVariable( l );
	if ( left->IsConst() && vj == 0.0 )
	{
		if ( op == add )
		{
/*			result->SetKind( right->GetKind() );
			update( i, l, 1.0, );
			return result->IsConst() ? -i : i;
*/
			right->IncRefCounter();
			return right->IsConst() ? -l : l;
		}
		else if ( op == sub )
		{
			if ( right->IsConst() && vl == 0.0 )
			{
/*				result->SetKind( right->GetKind() );
				update( i, l, 1.0, );
				return -i;
*/
				right->IncRefCounter();
				return -l;
			}
			j = l;
			left = right;
			op = neg;
		}
/*		result->MarkAsConst();
		update( i, j, 1.0, );
		return -i;
*/
		else
		{
			left->IncRefCounter();
			return -j;
		}
	}
	else if ( left->IsConst() && vj == 1.0 )
	{
		if ( op == mul )
		{
/*			result->SetKind( right->GetKind() );
			update( i, l, 1.0, );
			return result->IsConst() ? -i : i;
*/
			right->IncRefCounter();
			return right->IsConst() ? -l : l;
		}
		if ( op == sqroot )
		{
/*			result->SetKind( left->GetKind() );
			update( i, j, 1.0, );
			return -i;
*/
			left->IncRefCounter();
			return -j;
		}
	}
	if ( op < sqroot )
	{
		if ( right->IsConst() && vl == 0.0 )
		{
			if ( op <= sub )
			{
/*				result->SetKind( left->GetKind() );
				update( i, j, 1.0, );
				return result->IsConst() ? -i : i;
*/
				left->IncRefCounter();
				return left->IsConst() ? -j : j;
			}
			else if ( op == mul )
			{
/*				result->SetKind( right->GetKind() );
				update( i, l, 1.0, );
				return -i;
*/
				right->IncRefCounter();
				return -l;
			}
			else if ( op == per )
				throw CException( div_by_con0 );
		}
		else if ( right->IsConst() && vl == 1.0 )
		{
			if ( op == mul || op == per )
			{
/*				result->SetKind( left->GetKind() );
				update( i, j, 1.0, );
				return result->IsConst() ? -i : i;
*/
				left->IncRefCounter();
				return left->IsConst() ? -j : j;
			}
		}
	}
	result = &m_Wmatrix.GetVariable( i = m_Wmatrix.Require() );
	result->MarkAsInterm();
	result->SetValue( vi );
	alpha = 1.0;
	beta = 1.0;
	switch ( op )
	{
	case add:
		alpha = 1.0;
		beta = 1.0;
		break;
	case sub:
		alpha = 1.0;
		beta = -1.0;
		break;
	case mul:
		alpha = vl;
		beta = vj;
		break;
	case per:
		alpha = 1 / vl;
		beta = -vi / vl;
		break;
	case sqroot:
		alpha = 0.5 / vi;
		beta = 0.0;
		break;
	case neg:
		alpha = -1.0;
		beta = 0.0;
		break;
	}
    if ( left->IsConst() )
		alpha = 0.0;
	
	if ( op < sqroot && right->IsConst() )
		beta = 0.0;

	if ( op == neg || m_bErrorFree )
		z = 0.0;
	else
		z = vi*vi;
	update( i, j, alpha, z, l, beta );
	++m_nOperCounter;
	return i;
}

int CUkkonen::Composition( void )
{
	long i;
	long j;
	long first;
	CUkkonenCol *variable;
	double value;

	if ( m_bErrorFree )
		throw CException( multiple_comp_call );

	first = m_Wmatrix.InitIndex();
	for( j = first; j > 0L; j = m_Wmatrix.AdvanceIndex( j ) )
	{
		for( i = first; i < j; i = m_Wmatrix.AdvanceIndex( i ) )
			m_Wmatrix.AAT( i, j ) = 0.0;
		variable = &m_Wmatrix.GetVariable( j );
		if ( variable->IsInterm() || variable->IsInput() || variable->IsOutput() )
		{
			value = variable->GetValue();
			m_Wmatrix.AAT( j, j ) = value * value;
		}
		else
			m_Wmatrix.AAT( j, j ) = 0.0;
	}
	m_bErrorFree = true;
	return 1;

}




long CUkkonen::GetOutSize( void )
{
	long i;
	long counter = 0L;

	for ( i = m_Wmatrix.InitIndex(); i > 0L; i = m_Wmatrix.AdvanceIndex( i ) )
	{
		if ( m_Wmatrix.GetVariable( i ).IsOutput() )
			++counter;
	}
	if ( counter != m_nOutputCounter )
		throw CException( "counter != m_nOutputCounter in CUkkonen::GetOutSize" );

	return counter;
}
long CUkkonen::GetOutput( double *output )
{
	long i;
	long counter = 0L;
	CUkkonenCol *ptr;
	for ( i = m_Wmatrix.InitIndex(); i > 0L; i = m_Wmatrix.AdvanceIndex( i ) )
	{
		if ( ( ptr = &m_Wmatrix.GetVariable( i ) )->IsOutput() )
			output[ ptr->GetKind() - 1L ] = ptr->GetValue();
	}
	return counter;
}


