#ifndef _CUKKONEN_
#define _CUKKONEN_
#include <math.h>
#include "CWmatrices.h"

class CUkkonen
{
protected:
	CWmatrices m_Wmatrix;
private:
	double *m_pDATA;
	long m_nDATSIZE;
	long m_nInputCounter;
	long m_nOutputCounter;
	long m_nOperCounter;

	long m_nNPICK;
	bool m_bEachEntry;
	bool m_bLargestEntry;
	bool m_bWK;
	bool m_bER;
	bool m_bCN;

	bool m_bErrorFree;

	double m_dDatNorm;

	void Init( double *pDATA, long nNPICK, long nDATSIZE = -1L );
protected:
	void SetOnlyA( void )
	{
		m_Wmatrix.OnlyA();
	}

	virtual void RunAlgorithm( int AlgIndex, CGlobals &globals );

public:
	CUkkonen( double *pDATA, long nDATSIZE, long nNPICK );

	virtual ~CUkkonen( void )
	{
	}

	virtual CUkkonen *clone( void )
	{
		return new CUkkonen( *this );
	}

	void Derive( double *input, long nNPICK, CGlobals &globals, int AlgIndex = 1 );

	virtual void GetMatrices( double *A, double *B = 0 );

	virtual void KahanCorrection( void );

	virtual void CndNumCorrection( void );

	double *GetPackedA( void )
	{
		if ( m_Wmatrix.GetLimit() > 0 )
			return &m_Wmatrix.AAT( 1L, 1L );
		else
			return 0;
	}

	virtual double *GetPackedB( void );


	long Insert( void )
	{
		long index;
		m_Wmatrix.InitToConst( index = m_Wmatrix.Require() );
		return -index;
	}

	long Insert( double value )
	{
		long index;
		index = Insert();
		( m_Wmatrix.GetVariable( -index ) ).SetValue( value );
		return index;
	}



//	virtual long input( double *value );

	long input( double *value )
	{
		if( m_nInputCounter >= m_nDATSIZE )
			throw CException( input_too_large );
		*value = m_pDATA[ m_nInputCounter++ ];
		return m_nInputCounter;
	}


	virtual void update( long i, long j,  double alpha, double z = 0.0, long l = 0L, double beta = 0.0 );

	long Insert( enum operation op, long j,  double vi = 0.0, double vj = 0.0, long l = 0L, double vl = 0.0 );

	long Copy( long j, long i = 0L )
	{
		CUkkonenCol *ptrWhat;
		
//		CUkkonenCol *ptrWhere;
		if ( i < 0L )
			i = -i;
		if ( j < 0L )
			j = -j;
		ptrWhat = &m_Wmatrix.GetVariable( j );
		if ( i == j )
			return ptrWhat->IsConst() ? -j : j;
		
//		ptrWhere = m_Wmatrix.GetVariable( i );
//		ptrWhere->SetKind( ptrWhat->GetKind() );
//		update( i, j, 1.0, );
		ptrWhat->IncRefCounter();
		if ( i != 0 )
			Release( i );
		return ptrWhat->IsConst() ? -j : j;
	}



	

	void Release( long i )
	{
		CUkkonenCol *ColPtr;
		if ( i < 0L )
			i = -i;
		ColPtr = &m_Wmatrix.GetVariable( i );
		if ( !ColPtr->IsOutput() )
			m_Wmatrix.Release( *ColPtr );

	}

	virtual void SetData( double *input, long nDatSize, bool ParChanged );

	long GetOutSize( void );

	long GetInput( double *&pData )
	{
		pData = m_pDATA;
		return m_nDATSIZE;
	}

	double GetInputNorm( void )
	{
		return m_dDatNorm;
	}

	long GetInputSize( void )
	{
		return m_nDATSIZE;
	}

	void Output( long i, double value )
	{
		CUkkonenCol *ColPtr;
		if ( i < 0L )
			i = -i;
		ColPtr = &m_Wmatrix.GetVariable( i );
		if ( ColPtr->IsOutput() )
			return;
		if ( !ColPtr->IsInterm() )
			throw CException( illegal_output );
		
		ColPtr->SetValue( value );
		ColPtr->MarkAsOutput( ++m_nOutputCounter );
		
	}
	long GetOutput( double *output );

	void ReleaseAll( void )
	{
		m_Wmatrix.ReleaseAll();
	}

	int Composition( void );

	long GetNumOfOpers( void )
	{
		return m_nOperCounter;
	}

	long GetMatrixSize( void )
	{
		return m_Wmatrix.GetSize();
	}

	virtual void GetGraph( long *SizeArray, double *&cons, long *&left, long *&oper, long *&right, long *&output )
	{
		SizeArray[0] = SizeArray[1] = SizeArray[2] = 0L;
		cons = 0;
		left = oper = right = output = 0;
	}



	virtual long GetMemAlloc( void ) const
	{
		return m_Wmatrix.GetMemAlloc();
	}

	virtual long GetMemUsed( void ) const
	{
		return m_Wmatrix.GetMemUsed();
	}

	void Pack( void )
	{
		m_Wmatrix.OutputOnly();
		m_Wmatrix.Pack();
	}

	void SetErrorFree( void )
	{
		m_bErrorFree = true;
	}

	void ClearErrorFree( void )
	{
		m_bErrorFree = false;
	}





};

#endif

