#include "CUkkonenGraph.h"

void CUkkonenGraph::SetData( double *input, long nDatSize, bool ParChanged )
{
	CUkkonen::SetData( input, nDatSize, ParChanged );
	m_TheGraph.SetData( input, nDatSize, ParChanged );
}

void CUkkonenGraph::RunAlgorithm( int AlgIndex, CGlobals &globals )
{
	double *pDATA;
	long nvary;
	long nop;
	long nfind;
	long nfix;

	long *lop;
	long *oper;
	long *rop;

	long *ndxout;
	double *cons;

	long nodesize;

	long i;
	long j;
	long nSpace;
	long tmp;
	double value;
	double LeftArg;
	double RightArg;

	long *LeftIndex;
	long *RightIndex;

	
	enum operation op;

	long index;
	
    long nZero = 0L;

	GetInput( pDATA );
	m_TheGraph.SetAlgIndex( AlgIndex );
	m_TheGraph.CreateGraph( pDATA, globals );
	m_TheGraph.GetSizes( nop, nvary, nfind, nfix );
	m_TheGraph.GetOperations( lop, oper, rop );
	m_TheGraph.GetPointers( ndxout, cons, pDATA );

	nodesize = nop + nvary;
	if ( nodesize > m_nNodeSize )
	{
		delete [] m_pLeftValue;
		m_pLeftValue = 0;
		m_pLeftValue = new long[ nodesize ];
		m_nNodeSize = nodesize;
	}

	for ( i = 0; i < nodesize; ++i )
		m_pLeftValue[i] = 0L; 

	if ( nfix > m_nConSize )
	{
		delete [] m_pConstLeftValue;
		m_pConstLeftValue = 0;
		m_pConstLeftValue = new long[ nfix ];
		m_nConSize = nfix;
	}

	for ( i = 0; i < nfix; ++i )
		m_pConstLeftValue[i] = 0; 

	for ( i = 0; i < nfind; ++i )
		m_pLeftValue[ ndxout[i] - 1L ] = 1L;

    tmp = 0L;
	nSpace = nfind;
	for ( i = nodesize - 1; i >= nvary; --i )
	{
		m_pLeftValue[ i ] = 0L;
		if ( lop[i] > 0L && m_pLeftValue[ j = ( lop[i] - 1L ) ] == 0L )
		{
			m_pLeftValue[j] = 1L;
			m_pLeftValue[i] += 1L;
			++nSpace;
		}
		else if ( lop[i] < 0L && m_pConstLeftValue[ j = ( -lop[i] - 1L ) ] == 0L )
		{
			m_pConstLeftValue[j] = 1L;
			m_pLeftValue[i] += 1L;
			++nSpace;
		}

		if ( rop[i] > 0L && m_pLeftValue[ j = ( rop[i] - 1L ) ] == 0L )
		{
			m_pLeftValue[j] = 1L;
			m_pLeftValue[i] += 2L;
			++nSpace;
		}
		else if ( rop[i] < 0L && m_pConstLeftValue[ j = ( -rop[i] - 1L ) ] == 0L )
		{
			m_pConstLeftValue[j] = 1L;
			m_pLeftValue[i] += 2L;
			++nSpace;
		}
		if ( nSpace > tmp )
			tmp = nSpace;
		--nSpace;
	}
	if ( nSpace != nvary )
		throw CException( "nSpace != nvary in CUkkonenGraph::RunAlgorithm." );
	nSpace = tmp;
	m_Wmatrix.Allocate( nSpace );
	m_Wmatrix.InitInput( pDATA, nvary, GetInputNorm() );

	for ( i = 0; i < nvary; ++i )
		m_pLeftValue[i] = input( &value );
	

	for ( i = 0; i < nfix; ++i )
		m_pConstLeftValue[i] = Insert( cons[i] ); 

	for ( i = nvary; i < nodesize; ++i )
	{
		LeftIndex = &nZero;
		LeftArg = 0.0;
		if ( lop[i] > 0 )
			LeftIndex = m_pLeftValue + ( lop[i] - 1L );
		else if ( lop[i] < 0 )
			LeftIndex = m_pConstLeftValue + ( -lop[i] - 1L );
		LeftArg = ( m_Wmatrix.GetVariable( *LeftIndex ) ).GetValue();

		if ( oper[i] < 0 )
		{
			op = ( enum operation )( -oper[i] );
			SetErrorFree();
		}
		else
		{
			op = ( enum operation )( oper[i] );
			ClearErrorFree();
		}
		RightIndex = &nZero;
		RightArg = 0.0;
		if ( op <= per )
		{
			if ( rop[i] > 0 )
				RightIndex = m_pLeftValue + ( rop[i] - 1L );
			else if ( rop[i] < 0 )
				RightIndex = m_pConstLeftValue + ( -rop[i] - 1L );
			RightArg = ( m_Wmatrix.GetVariable( *RightIndex ) ).GetValue();
		}
		value = 0.0;

		switch ( op )
		{
		case add:
			value = LeftArg + RightArg;
			break;
		case sub:
			value = LeftArg - RightArg;
			break;
		case mul:
			value = LeftArg * RightArg;
			break;
		case per:
			if ( RightArg == 0.0 )
				throw CException( div_by_zero );
			value = LeftArg / RightArg;
			break;
		case sqroot:
			if ( LeftArg <= 0.0 )
				throw CException( sqrt_neg );
			value = sqrt( LeftArg );
			break;
		case neg:
			value = -LeftArg;
			break;
		}
		
		index = Insert( op, *LeftIndex, value, LeftArg, *RightIndex, RightArg );

		if ( m_pLeftValue[i] >= 2L )
		{
			Release( *RightIndex );
			*RightIndex = 0L;
			m_pLeftValue[i] -= 2L;
		}
		if ( m_pLeftValue[i] >= 1L )
		{
			Release( *LeftIndex );
			*LeftIndex = 0L;
		}
		m_pLeftValue[i] = index;
	}
	for ( i = 0; i < nfind; ++i )
	{
		index = m_pLeftValue[ ndxout[i] - 1L ];
		value = ( m_Wmatrix.GetVariable( index ) ).GetValue();
		Output( index, value );
	}
}
