#ifndef _CWMATRICES_
#define _CWMATRICES_
#include "CUkkonenQueue.h"
class CWmatrices
{
private:

	CUkkonenCol *m_Wmatrix;
	long m_nLimit;
	long m_nSize;
	CUkkonenQueue m_Unallocated;	
	CUkkonenQueue m_Allocated;
//	long m_nPreReqCounter;
	bool m_bOnlyA;

	double *m_pDATA;
	long m_nDATSIZE;
	double m_dDatNorm;

	void SelfInit( const CWmatrices &other, long required );
	void SelfInit( const CWmatrices &other );

//	void InitInput( long first, long last );
	

public:
	CWmatrices( void )
	{
		m_Wmatrix = 0;
		m_nLimit = 0L;
		m_nSize = 0L;

//		m_nPreReqCounter = 0L;

		m_bOnlyA = false;

		m_pDATA = 0;
		m_nDATSIZE = 0L;
		m_dDatNorm = 0.0;

	}

	CWmatrices( const CWmatrices &other );

	~CWmatrices( void )
	{
		
		
		if  ( m_Wmatrix != 0 )
		{
			delete [] &m_Wmatrix->GetA( 1L );
			if ( !m_bOnlyA )
				delete [] &m_Wmatrix->GetB( 1L );
			delete [] m_Wmatrix;
		}
	}

	CWmatrices &operator=( const CWmatrices &other );

	void OnlyA( void )
	{
		m_bOnlyA = true;
	}

	void Allocate( long size )
	{
		if ( size > m_nLimit )
		{
			m_nSize = size;
			PreRequire();
			ReleaseAll();
		}
	}


	long GetLimit( void )
	{
		return m_nLimit;
	}

	long GetSize( void )
	{
		return m_nSize;
	}


	void InitInput( double *pDATA, long nDATSIZE, double dDatNorm );

	void PreRequire( void );

	long Require( void )
	{
		long index;
		CUkkonenCol *ptr;
		ptr = m_Unallocated.Out();
		if ( ptr == 0 )
		{
			index = ++m_nSize;
		}
		else
		{
			ptr->IncRefCounter();
			if ( ptr->GetIndex() > 1 )
				index = ( ( ptr - 1 )->InsertAfter( *ptr ) )->GetIndex();
			else
				index = m_Allocated.In( *ptr )->GetIndex();
			if ( index > m_nSize )
				m_nSize = index;
		}
		return index;
	}

	CUkkonenCol &GetVariable( long index )
	{
		if ( m_nSize > m_nLimit )
			PreRequire();
		if ( index > m_nLimit || index <= 0 )
			throw CException("Index out of range in wmatrix.");
		return m_Wmatrix[ index - 1 ]; 

	}




	long InitIndex( void )
	{
		CUkkonenCol *ptr;
		if ( m_nSize > m_nLimit )
			PreRequire();
		if ( ( ptr = m_Allocated.GetFirst() ) == 0 )
			return 0L;
		return ptr->GetIndex();
	}

	long AdvanceIndex( long index )
	{
		CUkkonenCol *ptr;
		if ( index == 0L )
			return 0L;
		ptr = m_Wmatrix[ index - 1 ].GetNext();
		if ( ptr == 0 )
			return 0L;
		return ptr->GetIndex();

	}


		

	double &AAT( long i, long j )
	{
		if ( i > j )
		{
			long tmp;
			tmp = i;
			i = j;
			j = tmp;
		}
		if ( j > m_nLimit )
			throw CException( "Index out of range in wmatrix."  );
		return m_Wmatrix[ j - 1 ].GetA( i );
	}

	double &BBT( long i, long j )
	{
		if ( i > j )
		{
			long tmp;
			tmp = i;
			i = j;
			j = tmp;
		}
		if ( j > m_nLimit )
			throw CException( "Index out of range in wmatrix."  );
		return m_Wmatrix[ j - 1 ].GetB( i );
	}

	void Release( CUkkonenCol &Var );

	void InitToConst( long index );


	void ReleaseAll( void );

	long GetMemUsed( void ) const
	{
		return m_nSize * sizeof( CUkkonenCol ) 
			+ ( ( ( m_nSize + 1L ) * m_nSize ) / ( m_bOnlyA ? 2L : 1L ) ) 
			* sizeof( double );
	}

	long GetMemAlloc( void ) const
	{
		return m_nLimit * sizeof( CUkkonenCol ) 
			+ ( ( ( m_nLimit + 1L ) * m_nLimit ) / ( m_bOnlyA ? 2L : 1L ) ) 
			* sizeof( double );
	}

	void OutputOnly( void );

	void Pack( void );

};

#endif