#include <string.h>
#include <stdio.h>
#include "miller.h"

#include "CGlobals.h"
#include "CMaxim.h"

void GetEcwStr( char *name, long nNPICK, int nNumOfAlgs )
	{
		if ( nNumOfAlgs == 1 )
		{
			switch ( nNPICK )
			{
			case -1:
				strcpy( name, "ERE" );
				break;
			case -3:
				strcpy( name, "ERL" );
				break;
			case 1:
				strcpy( name, "JWE" );
				break;
			case 2:
				strcpy( name, "WKE" );
				break;
			case 3:
				strcpy( name, "JWL" );
				break;
			case 4:
				strcpy( name, "WKL" );
				break;
			case 5:
				strcpy( name, "EXP" );
				break;
		
			}
		}
		else
		{
			switch ( nNPICK )
			{
			case -1:
				strcpy( name, "ER1/2" );
				break;
			case -2:
				strcpy( name, "ER2/1" );
				break;
			case 1:
				strcpy( name, "JW1/2" );
				break;
			case 2:
				strcpy( name, "JW2/1" );
				break;
			}

		}

	}

void GetMthdStr( char *name, int nMethod )
	{
		switch ( nMethod )
		{
		case 1:
			strcpy( name, "Rosenbrock" );
			break;
		case 2:
			strcpy( name, "MDS" );
			break;
		case 3:
			strcpy( name, "NMS" );
			break;
		}
	}


double Call_it( double *input, long &ierr, CGlobals &globals )
	{
		
		double rho;
		double penalty;
		ierr = 0;
		try
		{
			rho = globals.round( input, penalty );
	
		}
		catch ( CException exception )
		{
			int extype;
			rho = 0.0;
			penalty = 1.0;
			extype = exception.handler( globals );
			if ( extype > 0 )
			{
				ierr = 1;
				globals.SetTerminated();
			}
			else if ( extype < 0 )
			{
				CRoundBase *EnginePtr;
				EnginePtr = globals.GetTheEngine();
				rho = EnginePtr->GetRho();
				penalty = EnginePtr->GetPenalty();
				ierr = 1;
				globals.SetStop();
			}
		}
		try
		{
			globals.GetTheEngine()->IncCounter();
		}
		catch ( CException exception )
		{
			
				if ( exception.handler( globals ) )
				{				
					ierr = 1;
					globals.SetStop();
				}
			
		}
		return penalty * rho;
	}

double ComputeRho( long &ierr, CGlobals &globals )
	{
		
		double rho;
		double penalty;
		ierr = 0;
		try
		{
			rho = globals.ComputeRho( penalty );
	
		}
		catch ( CException exception )
		{
			int extype;
			rho = 0.0;
			penalty = 1.0;
			extype = exception.handler( globals );
			if ( extype > 0 )
			{
				ierr = 1;
				globals.SetTerminated();
			}
			else if ( extype < 0 )
			{
				CRoundBase *EnginePtr;
				EnginePtr = globals.GetTheEngine();
				rho = EnginePtr->GetRho();
				penalty = EnginePtr->GetPenalty();
				ierr = 1;
				globals.SetStop();
			}
		}
		try
		{
			globals.GetTheEngine()->IncCounter();
		}
		catch ( CException exception )
		{
			
				if ( exception.handler( globals ) )
				{				
					ierr = 1;
					globals.SetStop();
				}
			
		}
		return penalty * rho;
	}


/*
void Miller( double *pDATA, long nDATSIZE, long nNPICK, double dSTOPX, int nMethod, int nNumOfAlgs, CGlobals &globals  )
	{
		long ierr;
		double *pD;
		double *pC;
		long i;
		char OutStr[256];
		char name[15];
        double SaveRho = 0.0;
		CRoundBase *pEngine;
//		tombmeret=allokalt=max_allokalt = 0L;
	
		globals.SetHandlerMode( normal );
		pEngine = globals.InitEngine( pDATA, nDATSIZE, nNPICK, dSTOPX, nNumOfAlgs );

		
		
		if ( nMethod > 0 && nNPICK != 0 )
		{
			*name = '\0';
			GetEcwStr( name, nNPICK, nNumOfAlgs );
			sprintf( OutStr, "\n The chosen error measuring number is %s.\n",  
				 name);
            mPrint( OutStr );
			Call_it( pDATA, ierr, globals );
			SaveRho = pEngine->GetRho();
    		sprintf( OutStr, "The error comparing value at the initial data: %e\n\n", 
				SaveRho );
			mPrint( OutStr );


			pEngine->ReportConstraints();

			if ( pEngine->GetPenalty() <= 0.0 )
			{
				sprintf( OutStr, "\n The initial data are out of the constraints.\n" );
				throw CException( OutStr );
			}
			sprintf( OutStr, "\n The stopping value is %.6e.\n", dSTOPX );
			mPrint( OutStr );

			if ( SaveRho > dSTOPX )
			{
				sprintf( OutStr, 
				"\n The error measuring value at the initial data exceeds the stopping criterion. No maximization performed. \n");
				mPrint( OutStr );
			}
			else
			{
				long jvary;
				double *pTempData;
				*name = '\0';
                
                jvary = nDATSIZE;
				globals.SetHandlerMode( search );
				globals.ClearErr();
				pEngine->ResetCounter();
				if ( nMethod < 4 )
				{
					GetMthdStr( name, nMethod );
					sprintf( OutStr, 
						"\n Applying the maximizer. The chosen search method is %s.\n",
						name );
					mPrint( OutStr );
					pTempData = new double[ nDATSIZE ];
					for( i = 0; i < nDATSIZE; ++i )
						pTempData[i] = pDATA[i];
					pDATA = pTempData;

					if ( nMethod == 1 )
					{
						CMaxim maxim;
						maxim.rosen( pDATA, &jvary, &globals );
					}
					else if ( nMethod == 2 )
					{
						CMaxim maxim;
						maxim.mds( pDATA, &jvary, &globals );
					}
					else if ( nMethod ==3 )
					{
						CMaxim maxim;
						maxim.nms( pDATA, &jvary, &globals );
					}

					SaveRho = pEngine->GetRho();
					globals.SetHandlerMode( normal );
					if ( nNumOfAlgs == 1 && SaveRho > dSTOPX && pEngine->GetPenalty() > 0.0 )
					{
						double rho;
						long SavePick;
						SavePick = nNPICK;
						pEngine->SetEcw( 6 );
						Call_it( pDATA, ierr, globals );
						rho = pEngine->GetRho();
						if ( rho > 0 )
						{
							sprintf( OutStr, "The condition number is %.6e.\n", rho );
								mPrint( OutStr );
						}
						else
						{
							sprintf( OutStr, "An error occured while computing the conditional number." );
							mWarning( OutStr );
						}
						pEngine->SetEcw( SavePick );
					}
					globals.ListExceptions();
					globals.SetScalarLhs( SaveRho );
					pD = globals.GetArrayLhs( nDATSIZE );
					if ( pD != 0 )
					{
						for ( i = 0; i < nDATSIZE; ++i )
							pD[i] = pDATA[i];
					}
					delete [] pDATA;
				}
				else
				{
					mWarning( "Not yet implemented!\n" );
					return;
				}
			}
		}
		else
		{

			Call_it( pDATA, ierr, globals );
			if ( nNPICK != 0 )
				globals.SetScalarLhs( pEngine->GetRho() );
			else
			{
				pD = globals.GetArrayLhs( pEngine->GetOutSize() );
				if ( pD != 0 )
					pEngine->GetOutput( pD );
				
			}
		}
		pC = globals.GetArrayLhs( pEngine->GetNumOfConstr() );
		if ( pC != 0 )
			pEngine->GetConstraints( pC );

		globals.ClearEngine();
//		mSetRho( max_allokalt );
//		mSetRho(allokalt);
	}

*/
extern "C"
{
	double FNCTN( double *input, long *ierr, CGlobals *globals )
	{
       
		return Call_it( input, *ierr, *globals );
	}
}
/*
double Function( double *input )
{
	double RetValue;
	long ierr;
	if ( !CException::IsSearch() )
		mError( "The analyser has not been initialized for maximizing!" );
	RetValue = Call_it( input, ierr );
	if ( ierr == 0L )
		return RetValue;
	else
		return -1.0;
} 
*/

