#define MATLAB_MEX_FILE
#include <mex.h>
#include <string.h>
#include <stdio.h>
#include "miller.h"
#include "CGlobals.h"
#include "CMaxim.h"
#include "CRoundUkkonen.h"




mxArray *CreateMATLABptr( void *ptr )
{
	mxArray *data;
	mxArray *MATLABptr;
	const char *FieldName = "Miller_ptr";
	double *tmp;
	

	int i;

	data = mxCreateDoubleMatrix( sizeof ( void* ), 1, mxREAL );

	tmp = mxGetPr( data );
	for ( i = 0; i < sizeof ( void* ); ++i )
	{
		tmp[i] = ( ( unsigned char * )&ptr )[i];
	}

	MATLABptr = mxCreateStructMatrix( 1, 1, 1, &FieldName );

	mxSetField( MATLABptr, 0, FieldName, data );
	return MATLABptr;


}

void DestroyMATLABptr( mxArray *MATLABptr )
{
	mxArray *data;
	data = mxGetField( MATLABptr, 0, "Miller_ptr" );
	if ( data != 0 )
	{
		mxSetField( MATLABptr, 0, "Miller_ptr", 0 );
		mxDestroyArray( MATLABptr );
		mxDestroyArray( data );
	}
	else
		throw CException( "Corrupt MATLAB-pointer!" );
}

void *DecodeMATLABptr( const mxArray *MATLABptr )
{
	double *tmp;
	int i;
	unsigned char GlobPtr[128];
	void ** ptr;


	mxArray *data;
	if ( ( data = mxGetField( MATLABptr, 0, "Miller_ptr" ) ) == 0 )
		return 0;

	if ( mxGetNumberOfElements( data ) != sizeof ( void* ) )
		return 0;
	if ( ( tmp = mxGetPr( data ) ) == 0 )
		return 0;
	
	for ( i = 0; i < sizeof ( void* ); ++i )
	{
		GlobPtr[i] = (unsigned char) tmp[i];
	}
	ptr = (void **)GlobPtr;

	return *ptr;



}

int GetEcwCode( char *ecw, int nNumOfAlgs, bool &Ukkonen )
{
	    int npick = 0;
		Ukkonen = false;
		if ( ecw[0] == 'U' || ecw[0] == 'u' )
		{
			Ukkonen = true;
			npick = 1;
			while ( ( ecw[ npick - 1 ] = ecw[ npick ] ) != '\0' )
				++npick;
			npick = 0;
		}
		if ( ecw[0] == '\0' )
			npick = 10;


	    if ( ecw[0] == 'J' || ecw[0] == 'j' )
        {
                if ( nNumOfAlgs == 2 )
				{
					if ( ecw[2] == '1' )
					{
						strcpy( ecw, "JW1/2" );
						npick = 1;
					}
					else if ( ecw[2] == '2' )
					{
						strcpy( ecw, "JW2/1" );
						npick = 2;
					}
				}
				else if ( ecw[2] == 'E' || ecw[2] == 'e' )
                {
                        strcpy( ecw, "JWE" );
                        npick = 1;
                }
                else if ( ecw[2] == 'L' || ecw[2] == 'l' )
                {
                        strcpy( ecw, "JWL" );
                        npick = 3;
                }
        }
        else if ( ( ecw[0] == 'W' || ecw[0] == 'w' ) && nNumOfAlgs == 1 )
        {
                if ( ecw[2] == 'E' || ecw[2] == 'e' )
                {
                        strcpy( ecw, "WKE" );
                        npick = 2;
                }
                else if ( ecw[2] == 'L' || ecw[2] == 'l' )
                {
                        strcpy( ecw, "WKL" );
                        npick = 4;
                }
        }
        else if ( ecw[0] == 'E' || ecw[0] == 'e' )
        {
                if ( nNumOfAlgs == 2 )
				{
					if ( ecw[2] == '1' )
					{
						strcpy( ecw, "ER1/2" );
						npick = -1;
					}
					else if ( ecw[2] == '2' )
					{
						strcpy( ecw, "ER2/1" );
						npick = -2;
					}
				}
				else if ( ecw[2] == 'E' || ecw[2] == 'e' )
                {
                        strcpy( ecw, "ERE" );
                        npick = -1;
                }
                else if ( ecw[2] == 'L' || ecw[2] == 'l' )
                {
                        strcpy( ecw, "ERL" );
                        npick = -3;
                }
                else if ( ecw[2] == 'P' || ecw[2] == 'p' )
                {
                        strcpy( ecw, "EXP" );
                        npick = 5;
                }
        }
		return npick;
}


int GetMthdCode( char *mthd )
{
	int method = 0;

    if ( strcmp( mthd, "ROS" ) == 0 )
		method = 1;
    else if ( strcmp( mthd, "MDS" ) == 0 )
        method = 2;
    else if ( strcmp( mthd, "NMS" ) == 0 )
        method = 3;
	else if ( strcmp( mthd, "EXT" ) == 0 )
		method = 4;

	return method;
}



void CallAlgorithm( int index, CGlobals &globals )
{
	mxArray *prhs;
	void *ptr;
	int ierr;



	ptr = &globals;

	prhs = CreateMATLABptr( ptr );

	mexSetTrapFlag( 1 );

	ierr = mexCallMATLAB( 0, 0, 1, &prhs, globals.GetAlgName( index ) );

	mexSetTrapFlag( 0 );

	DestroyMATLABptr( prhs );


	globals.ThrowException();
	if ( ierr )
	{
		char ErrorTxt[300];
		sprintf( ErrorTxt, "Unhandled exception in %s", globals.GetAlgName( index ) );
		throw CException( ErrorTxt );
	}

}
/*
void Init( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )
{
    char ecw[7];
    char mthd[7];


	double *pDATA = 0;
	long nDATSIZE = 0L;
	long nNPICK = 0L;
	double dSTOPX = 0.0;
	int nMethod = 0;
	int nNumOfAlgs = 1;
	bool bUkkonen;

		
	CGlobals globals;
	bUkkonen = false;
	
	char ErrorText[256];
	ecw[0] = mthd[0] = '\0';



	if ( nrhs < 2 )
		throw CException( "At least two input arguments are required!" );
	if ( !mxIsChar( prhs[0] ) )
	    throw CException( "The first argument must be a string!" );
	if ( ( nNumOfAlgs = globals.SetAlgName( prhs[0] ) ) == 0 )
		throw CException( "An error occured while preocessing the first input argument!" );

		
	if ( !mxIsDouble( prhs[1] ) )
		throw CException( "The second input argument must be a double precision MATLAB array!" );
	pDATA = mxGetPr( prhs[1] );
	nDATSIZE = mxGetNumberOfElements( prhs[1] );
	if ( nDATSIZE == 0 )
		throw CException( "Initial data argument is an empty array!" );
	if ( nrhs > 2 )
	{
		if ( !mxIsChar( prhs[2] ) )
		{
			throw CException( "The third argument must be a string!" );
		}
		if ( mxGetString( prhs[2], ecw, 7 ) != 0 )
			throw CException( "An error occured while processing the 3. input argument!" );
		nNPICK = GetEcwCode( ecw, nNumOfAlgs, bUkkonen );
		if ( nNPICK == 0 )
		{
			if ( nNumOfAlgs == 1 )
				sprintf( ErrorText, "\"%s\" is an invalid error measuring number specifier\n"
					"for analysing a single algorithm!", ecw );
			else
				sprintf( ErrorText, "\"%s\" is an invalid error measuring number specifier\n"
					"for comparing two algorithms!", ecw );
			throw CException( ErrorText );
		}
		if ( nNPICK == 10 )
			nNPICK = 0;
	}
	else if ( nNumOfAlgs == 2 && nNPICK == 0 )
	{
		throw CException( "Code of error measuring number should be given for comparing!" );
	}
	if ( nrhs > 3 )
	{
		if ( !mxIsChar( prhs[3] ) )
		{
			throw CException( "The fourth argument must be a string!" );
		}
		if ( mxGetNumberOfElements( prhs[3] ) < 3 )
		{
			throw CException( "The name of the searching method number must be 3 characters long!" );
		}
		if ( mxGetString( prhs[3], mthd, 4 ) != 0 )
			throw CException( "An error occured while processing the 4-th input argument!" );
		if ( !( nMethod = GetMthdCode( mthd ) ) )
		{
			sprintf( ErrorText, "\"%s\" is an invalid search method specifier!", mthd );
			throw CException( ErrorText );
		}
	}
	dSTOPX = 10000.0;
	if ( nrhs > 4 )
	{
		if ( !mxIsNumeric(prhs[4]) )
		{
			throw CException("Stopping value (fifth input argument) must be numeric!");
		}
		dSTOPX = mxGetScalar( prhs[4] );
	}
	globals.SetMatlabLhs( nlhs, plhs );
	if ( bUkkonen )
		globals.SetUkkonen();
	globals.SetCallBack();
	Miller( pDATA, nDATSIZE, nNPICK, dSTOPX, nMethod, nNumOfAlgs, globals );
	globals.ClearCallBack();
	return;
}
*/
		

void mexFunction( int nlhs, mxArray *plhs[],
             int nrhs, const mxArray *prhs[] )
{
	

	void *ptr;
	

	if ( nrhs == 0 )
	{
		CGlobals *GlobalsPtr = new CGlobals;
		plhs[0] = CreateMATLABptr( GlobalsPtr );
		return;
	}
	else if ( nrhs > 0 && ( ptr = DecodeMATLABptr( prhs[0] ) ) != 0 )
	{
		CGlobals &globals = *( CGlobals * )ptr;
		if ( globals.IsCallBack() )
		{
			globals.CallBack( plhs, prhs + 1 );
			return;
		}
		else //if ( nrhs < 2 )
		{
/*		
			if ( nrhs == 1 )
			{
				if ( mxIsDouble( prhs[0] ) )
				{
					pDATA = mxGetPr( prhs[0] );
					globals.SetCallBack();
					parg = mxCreateScalarDouble( Function( pDATA ) );
					globals.ClearCallBack();
				}
				else
					mexErrMsgTxt( "First argument must be a double precision MATLAB array." );
			}
			else
				mexErrMsgTxt( "Too few input arguments." );

			mexErrMsgTxt( "Calling roundoff with globals struct is not implemented!" );

*/
			int MethodCode;
			MethodCode = int( mxGetScalar( prhs[1] ) );
			try
			{
				CRoundBase *EnginePtr;
				switch ( MethodCode )
				{
				//SET ALG. NAME
				case 1000:
					{
						double tmp;
						globals.ClearEngine();
						
						if ( globals.SetAlgName( prhs[2] ) && ( tmp = mxGetScalar( prhs[3] ) ) >= 1.0 )
						{
							globals.SetUkkonen();
							if ( tmp == 2.0 )
								globals.SetUkkonenGraph();
						}
					}
					break;
				//SET
				case 1001:
					
					{
						double tmp;
						int PropertyCode;
						PropertyCode = int( mxGetScalar( prhs[2] ) );
						EnginePtr = globals.GetTheEngine();
						switch ( PropertyCode )
						{
						case 3:
							globals.SetHandlerMode( EnumMode( int ( mxGetScalar( prhs[3] ) ) ) );
							break;
						case 4:
							tmp = mxGetScalar( prhs[3] );
							if ( EnginePtr == 0 )
								EnginePtr = globals.InitEngine( 0, 0, long(tmp) );
							else if ( tmp != EnginePtr->GetEcw() )
								EnginePtr->SetEcw( long(tmp) );
							break;
						case 5:
							tmp = mxGetScalar( prhs[3] );
							if ( EnginePtr == 0 )
								EnginePtr = globals.InitEngine( 0, 0, 0, tmp );
							else
								EnginePtr->SetStopCrit( tmp );
							break;
						}
					}
					break;
				//GET
				case 1002:

					{
						int PropertyCode;
						PropertyCode = int( mxGetScalar( prhs[2] ) );
						const char *AlgName;
						char NullStr = '\0';
						int NumOfAlgs;
						long size = 0L;
						double *pr;
						double *pr2;
						double value = 0;
						EnginePtr = globals.GetTheEngine();
						switch ( PropertyCode )
						{
						case 1:
							AlgName = globals.GetAlgName( 1 );
							if ( AlgName == 0 )
								AlgName = &NullStr;
							plhs[0] = mxCreateString( AlgName );
							break;
						case 2:
							AlgName = globals.GetAlgName( 2 );
							if ( AlgName == 0 )
								AlgName = &NullStr;
							plhs[0] = mxCreateString( AlgName );
							break;
						case 3:
							if ( globals.IsUkkonen() )
							{
								if ( globals.IsUkkonenGraph() )
								{
									value = 2.0;
								}
								else
								{
									value = 1.0;
								}
							}
							else
							{
								value = 0.0;
							}
							plhs[0] = mxCreateDoubleScalar( value );
							break;
						case 4:
							if ( EnginePtr != 0 )
								value = EnginePtr->GetEcw();
							plhs[0] = mxCreateDoubleScalar( value );
							break;
						case 5:
							if ( EnginePtr != 0 )
								value = EnginePtr->GetStopCrit();
							plhs[0] = mxCreateDoubleScalar( value );
							break;
						case 6:
							if ( EnginePtr != 0 )
								value = EnginePtr->GetInputSize();
							plhs[0] = mxCreateDoubleScalar( value );
							break;
						case 7:
							NumOfAlgs = globals.GetAlgName( 2 ) == 0 ? 1 : 2;
							plhs[0] = mxCreateDoubleMatrix( 1, NumOfAlgs, mxREAL );
							pr = mxGetPr( plhs[0] );
				            pr[0] = 0.0;
							if ( NumOfAlgs == 2 )
								pr[1] = 0.0;
							if ( EnginePtr != 0 )
							{
								pr[0] = EnginePtr->GetNumOfOpers( 1 );
								if ( NumOfAlgs == 2 )
									pr[1] = EnginePtr->GetNumOfOpers( 2 );
							}
							break;
						case 8:
							NumOfAlgs = globals.GetAlgName( 2 ) == 0 ? 1 : 2;
							plhs[0] = mxCreateDoubleMatrix( 1, NumOfAlgs, mxREAL );
							pr = mxGetPr( plhs[0] );
					        pr[0] = 0.0;
							if ( NumOfAlgs == 2 )
								pr[1] = 0.0;
							if ( EnginePtr != 0 )
							{
								pr[0] = EnginePtr->GetNumOfConstants( 1 );
								if ( NumOfAlgs == 2 )
									pr[1] = EnginePtr->GetNumOfConstants( 2 );
							}
							break;
						case 9:
							if ( EnginePtr != 0 )
								value = EnginePtr->GetOutSize();
							plhs[0] = mxCreateDoubleScalar( value );
							break;
						case 10:
							if ( EnginePtr != 0 )
								value = EnginePtr->GetNumOfConstr();
							plhs[0] = mxCreateDoubleScalar( value );
							break;
						case 11:
							if ( EnginePtr != 0 )
								value = EnginePtr->GetCounter();
							plhs[0] = mxCreateDoubleScalar( value );
							break;
						case 12:
							NumOfAlgs = globals.GetAlgName( 2 ) == 0 ? 1 : 2;
							if ( EnginePtr != 0 )
								size = EnginePtr->GetOutSize();
							plhs[0] = mxCreateDoubleMatrix( size, NumOfAlgs, mxREAL );
							pr = mxGetPr( plhs[0] );
							if ( NumOfAlgs == 2 )
								pr2 = pr + size;
							else
								pr2 = 0;
							if ( size > 0 )
								EnginePtr->GetOutput( pr, pr2 );
							break;
						case 13:
							if ( EnginePtr != 0 )
								size = EnginePtr->GetNumOfConstr();
							plhs[0] = mxCreateDoubleMatrix( size, 1, mxREAL );
							if ( size > 0 )
								EnginePtr->GetConstraints( mxGetPr( plhs[0] ) );
							break;
						case 14:
							if ( EnginePtr != 0 )
								value = EnginePtr->GetRho();
							plhs[0] = mxCreateDoubleScalar( value );
							break;
						case 15:
							value = 1.0;
							if ( EnginePtr != 0 )
								value = EnginePtr->GetPenalty();
							plhs[0] = mxCreateDoubleScalar( value );
							break;
						case 16:
							if ( ( !globals.IsUkkonen() || globals.IsUkkonenGraph() )
								&& EnginePtr != 0 )
							{
								NumOfAlgs = globals.GetAlgName( 2 ) == 0 ? 1 : 2;
							}
							else
							{
								NumOfAlgs = 0;
							}
							{
								mxArray *parg;
								double *prLeft;
								double *prRight;
								double *cons;
								long *left;
								long *oper;
								long *right;
								long *dxout;
								long i;
								int algIndex;
								long SizeArray[ 3 ];
	
								const char *fieldName[] = { "constant", "left", "operator", 
									"right", "output", "data_size" };

								plhs[0] = mxCreateStructMatrix( 1, NumOfAlgs, 6, fieldName );
								for ( algIndex = 0; algIndex < NumOfAlgs; ++algIndex )
								{
									EnginePtr->GetGraph( SizeArray, cons, left, oper, right, dxout, algIndex + 1 );
									size = SizeArray[ 0 ];
									pr = mxGetPr( parg = mxCreateDoubleMatrix( size, 1, mxREAL ) );
									mxSetFieldByNumber( plhs[0], algIndex, 0, parg );

									for ( i = 0; i < size; ++i )
										pr[i] = cons[i];
								
									size = SizeArray[ 1 ];
									prLeft = mxGetPr( parg = mxCreateDoubleMatrix( size, 1, mxREAL ) );
									mxSetFieldByNumber( plhs[0], algIndex, 1, parg );
									pr = mxGetPr( parg = mxCreateDoubleMatrix( size, 1, mxREAL ) );
									mxSetFieldByNumber( plhs[0], algIndex, 2, parg );
									prRight = mxGetPr( parg = mxCreateDoubleMatrix( size, 1, mxREAL ) );
									mxSetFieldByNumber( plhs[0], algIndex, 3, parg );

									for ( i = 0; i < size; ++i )	
									{
										prLeft[i] = left[i];
										pr[i] = oper[i];
										prRight[i] = right[i];
									}

									size = SizeArray[ 2 ];
									pr = mxGetPr( parg = mxCreateDoubleMatrix( size, 1, mxREAL ) );
									mxSetFieldByNumber( plhs[0], algIndex, 4, parg );
	
									for ( i = 0; i < size; ++i )
										pr[i] = dxout[i];
					
									mxSetFieldByNumber( plhs[0], algIndex, 5, 
									mxCreateDoubleScalar( EnginePtr->GetInputSize() ) );
								}
							}
							break;
						case 17:
							pr = mxGetPr( plhs[0] = mxCreateDoubleMatrix( 1, 2, mxREAL ) );
							pr[0] = pr[1] = 0.0;
							if ( EnginePtr != 0 )
							{
								pr[0] = EnginePtr->GetMemAlloc();
								pr[1] = EnginePtr->GetMemUsed();
							}
							break;
						case 18:
							pr = mxGetPr( plhs[0] = mxCreateDoubleMatrix( 11, 1, mxREAL ) );
							globals.GetExceptions( pr );
							break;
						case 19:
							plhs[0] = mxCreateScalarDouble( globals.GetMode() );
							break;
						case 20:
							plhs[0] = mxCreateDoubleScalar( globals.IsStop() );
							break;
						case 21:
							plhs[0] = mxCreateDoubleScalar( globals.IsTerminated() );
							break;
						case 22:
							NumOfAlgs = globals.GetAlgName( 2 ) == 0 ? 1 : 2;
							plhs[0] = mxCreateDoubleMatrix( 1, NumOfAlgs, mxREAL );
							pr = mxGetPr( plhs[0] );
					        pr[0] = 0.0;
							if ( NumOfAlgs == 2 )
								pr[1] = 0.0;
							if ( EnginePtr != 0 && globals.IsUkkonen() )
							{
								pr[0] = ( (CRoundUkkonen *)( EnginePtr ) )->GetMatrixSize( 1 );
								if ( NumOfAlgs == 2 )
									pr[1] = ( (CRoundUkkonen *)( EnginePtr ) )->GetMatrixSize( 1 );
							}
							break;
						}
					}
					break;
				//CALL
				case 1003:
				
					{
						double *pDATA = 0;
						double *pPar = 0;
						long nDATSIZE = 0L;
						long ierr = 0L;
						long size = 0;
						bool ParChanged;
						
						pDATA = mxGetPr( prhs[2] );
						
						if ( globals.GetMode() == normal )
						{
							nDATSIZE = mxGetNumberOfElements( prhs[2] );
							if ( nrhs > 3 )
							{
								size = mxGetNumberOfElements( prhs[3] );
								pPar = mxGetPr( prhs[3] );
							}
							ParChanged = globals.SetParams( size, pPar );

							if ( ( EnginePtr = globals.GetTheEngine() ) == 0 )
								EnginePtr = globals.InitEngine( pDATA, nDATSIZE, 0 );
							else
								EnginePtr->SetData( pDATA, nDATSIZE, ParChanged );
							globals.SetMatlabArgs( nlhs - 1, plhs + 1 );
							globals.SetCallBack();
							Call_it( pDATA, ierr, globals );
							globals.ClearCallBack();
							globals.ClearMatlabArgs();
							if ( EnginePtr->GetEcw() != 0 )
							{
								plhs[0] = mxCreateDoubleScalar( EnginePtr->GetRho() );
							}
							else
							{
								int NumOfAlgs = globals.GetAlgName( 2 ) == 0 ? 1 : 2;
								double *pr;
								double *pr2;
								if ( EnginePtr != 0 )
									size = EnginePtr->GetOutSize();
								plhs[0] = mxCreateDoubleMatrix( size, NumOfAlgs, mxREAL );
								pr = mxGetPr( plhs[0] );
								if ( NumOfAlgs == 2 )
									pr2 = pr + size;
								else
									pr2 = 0;
								if ( size > 0 )
									EnginePtr->GetOutput( pr, pr2 );

/*

								size = EnginePtr->GetOutSize();
								plhs[0] = mxCreateDoubleMatrix( size, 1, mxREAL );
								if ( size > 0 )
									EnginePtr->GetOutput( mxGetPr( plhs[0] ) );				
*/
									
							}

						}
						else
						{
							
							globals.SetCallBack();
							plhs[0] = mxCreateDoubleScalar( Call_it( pDATA, ierr, globals ) );
							if ( nlhs > 1 )
								plhs[1] = mxCreateDoubleScalar( ierr );
							globals.ClearCallBack();
				
						}
					}
					break;
				//MAXIMIZE
				case 1004:
					{
						int nMethod;
						double *pDATA = 0;
						long nDATSIZE = 0L;
						long jvary;
						double *pTempData;
						long i;
						

						pDATA = mxGetPr( prhs[2] );
						nDATSIZE = mxGetNumberOfElements( prhs[2] );

						if ( ( EnginePtr = globals.GetTheEngine() ) == 0 )
							EnginePtr = globals.InitEngine( pDATA, nDATSIZE, 4L );
						else
							EnginePtr->SetData( pDATA, nDATSIZE, false );
						nMethod = int( mxGetScalar( prhs[3] ) );
						jvary = nDATSIZE;
						globals.SetHandlerMode( search );
						pTempData = new double[ nDATSIZE ];
						for( i = 0; i < nDATSIZE; ++i )
							pTempData[i] = pDATA[i];
						pDATA = pTempData;
						
						globals.SetCallBack();
						if ( nMethod == 1 )
						{
							CMaxim maxim;
							maxim.rosen( pDATA, &jvary, &globals );
						}
						else if ( nMethod == 2 )
						{
							CMaxim maxim;
							maxim.mds( pDATA, &jvary, &globals );
						}
						else if ( nMethod ==3 )
						{
							CMaxim maxim;
							maxim.nms( pDATA, &jvary, &globals );
						}
						
						globals.ClearCallBack();
						globals.SetHandlerMode( normal );
						plhs[0] = mxCreateDoubleScalar( EnginePtr->GetRho() );
						plhs[1] = mxCreateDoubleMatrix( nDATSIZE, 1, mxREAL );
						pTempData = mxGetPr( plhs[1] );
						if ( pTempData != 0 )
						{
							for ( i = 0; i < nDATSIZE; ++i )
								pTempData[i] = pDATA[i];
						}
						delete [] pDATA;
					}
					break;
				//RESETCOUNTER
				case 1005:
				
					{
						globals.VerifyAlgName();
						globals.ClearErr();
						if ( ( EnginePtr = globals.GetTheEngine() ) != 0 )
							EnginePtr->ResetCounter();
					}
					break;
				//DESTROY
				case 1006:
				
					{
						CGlobals *gptr = &globals;
						delete gptr;
					}
					break;
				//COPY CONSTRUCTOR
				case 1007:
				
					{
						CGlobals *ptrNew;
						ptrNew = new CGlobals( globals );
						plhs[0] = CreateMATLABptr( ptrNew );
					}
					break;
                //ASSIGNMENT OPERATOR
				case 1008:
	
					{
						if ( ( ptr = DecodeMATLABptr( prhs[2] ) ) != 0 )
						{
							const CGlobals &other = *( CGlobals * )ptr;
							globals = other;
						}
						else
						{
							throw CException( "Invalid object to copy!" );
						}
					}
					break;
				//RESET
				case 1009:
				
					globals.reset();
					break;
				//FILL BEGIN
				case 1010:
					{
						double *pDATA = 0;
						double *pPar = 0;
						long nDATSIZE = 0L;
						long ierr = 0L;
						long size = 0;
						bool ParChanged;
						
						if ( globals.GetMode() == normal )
						{
							if ( nrhs > 2 )
							{
								pDATA = mxGetPr( prhs[2] );						
								nDATSIZE = mxGetNumberOfElements( prhs[2] );
							}
							pDATA = globals.SetDataCopy( nDATSIZE, pDATA );
							if ( nrhs > 3 )
							{
								size = mxGetNumberOfElements( prhs[3] );
								pPar = mxGetPr( prhs[3] );
							}
							ParChanged = globals.SetParams( size, pPar );
							if ( ( EnginePtr = globals.GetTheEngine() ) == 0 )
								EnginePtr = globals.InitEngine( pDATA, nDATSIZE, 0 );
							else
								EnginePtr->SetData( pDATA, nDATSIZE, ParChanged );
							globals.SetCallBack();
							if ( globals.FillBegin( pDATA ) )
							{
								plhs[0] = mxCreateDoubleScalar( 1.0 );
							}
							else
							{
								globals.ClearCallBack();
								plhs[0] = mxCreateDoubleScalar( 0.0 );
							}

						}
						else
						{
							CAssert::Assert( false );
						}
					}
					break;
				//COMPUTE RHO
				case 1011:
					{
						if ( globals.GetMode() == normal )
						{
							double *pDATA = 0;
							long nDATSIZE = 0L;
							long ierr = 0L;
                            if ( nrhs > 2 )
							{
								pDATA = mxGetPr( prhs[2] );
								nDATSIZE = mxGetNumberOfElements( prhs[2] );
							}
							else
							{
								globals.GetDataCopy( nDATSIZE, pDATA );
							}
							if ( ( EnginePtr = globals.GetTheEngine() ) == 0 )
							{
								EnginePtr = globals.InitEngine( pDATA, nDATSIZE, 0 );
//								plhs[0] = mxCreateDoubleScalar( 0.0 );
							}
							else if ( nDATSIZE > 0 )
							{
								EnginePtr->SetData( pDATA, nDATSIZE, false );
							}
							ComputeRho( ierr, globals );
							plhs[0] = mxCreateDoubleScalar( EnginePtr->GetRho() );
						}
						else
						{
							plhs[0] = mxCreateDoubleScalar( 0.0 );
							CAssert::Assert( false );
						}
					}
					break;
				default:
					throw CException( "Invalid method code!" );
					break;
				}
			}
				
			catch ( CException exception )
			{
				globals.ClearCallBack();
				exception.FinalHandler();
			}
			return;
		}

	}
	else
	{
		try
		{
			throw CException( "Miller object is invalid" );
			//Init( nlhs, plhs, nrhs, prhs );
		}
		catch ( CException exception )
		{
			exception.FinalHandler();
		}
	}




}
	


void mPrint( const char *szoveg )
{
    mexPrintf("%s\n", szoveg );
}
extern "C"
{
	void CKIIR( char *szoveg )
	{	
		mPrint( szoveg );
	}
}

void mTerminate( const char *szoveg )
{
	mexErrMsgTxt( szoveg );

}

void mWarning( const char *szoveg )
{
    mexWarnMsgTxt( szoveg );
}


void *operator new( size_t siz )
{
	void *ptr;
	ptr = mxMalloc( siz );
	mexMakeMemoryPersistent( ptr );
//	ptr_tomb[tombmeret] = ptr;
//	meret_tomb[tombmeret++] = siz;
//	allokalt+=siz;
//	if ( allokalt > max_allokalt )
//		max_allokalt = allokalt;
    return ptr;
}

void operator delete( void *m )
{
/*	long i;
    for ( i = 0L; i < tombmeret && ptr_tomb[i] != m; ++i )
		;
	if ( i < tombmeret )
	{
		ptr_tomb[i] = 0;
		allokalt-=meret_tomb[i];
		meret_tomb[i] = 0L;
	}
*/  
	mxFree( m );
}


		






        
            
                      
            
         
 
  


    

